@@ -46,12 +46,14 @@
 #include "fs_local.h"
 #include <arladeb.h>
 
-RCSID("$Id: fs.c,v 1.65 1999/03/06 14:36:37 lha Exp $");
+RCSID("$Id: fs.c,v 1.72 1999/07/25 00:12:16 lha Exp $");
 
 static int empty_cmd (int argc, char **argv);
 static int apropos_cmd (int argc, char **argv);
 static int arladebug_cmd (int argc, char **argv);
+static int calculate_cmd (int argc, char **argv);
 static int checkservers_cmd (int argc, char **argv);
+static int copyacl_cmd (int argc, char **argv);
 static int diskfree_cmd (int argc, char **argv);
 static int examine_cmd (int argc, char **argv);
 static int flush_cmd (int argc, char **argv);
@@ -83,6 +85,7 @@ static int setprio_cmd (int argc, char **argv);
 static int setmaxprio_cmd (int argc, char **argv);
 static int setquota_cmd (int argc, char **argv);
 static int setcrypt_cmd (int argc, char **argv);
+static int strerror_cmd (int argc, char **argv);
 static int sysname_cmd (int argc, char **argv);
 static int fsversion_cmd (int argc, char **argv);
 static int venuslog_cmd (int argc, char **argv);
@@ -95,10 +98,11 @@ static int xfsdebug_print_cmd (int argc, char **argv);
 static SL_cmd cmds[] = {
     {"apropos",        apropos_cmd,	"locate commands by keyword"},
     {"arladebug",      arladebug_cmd,	"tweek arla-debugging flags"},
+    {"calculate cache", calculate_cmd,  "calculate the usege of cache"},
     {"checkservers",   checkservers_cmd,"check if servers is up"},
     {"checkvolumes",   empty_cmd,	"lookup mappings between volume-Id's and names"},
     {"cleanacl",       empty_cmd,	"clear out numeric acl-entries"},
-    {"copyacl",        empty_cmd,	"copy acl"},
+    {"copyacl",        copyacl_cmd,	"copy acl"},
     {"diskfree",       diskfree_cmd,	"show free partition space"},
     {"examine",        examine_cmd,	"examine volume status"},
     {"flush",          flush_cmd,	"remove file from cache"},
@@ -146,6 +150,7 @@ static SL_cmd cmds[] = {
     {"setcrypt",       setcrypt_cmd,	"set encryption on/off"},
     {"setvol",         empty_cmd,	"change status of volume"},
 /*  {"storebehind",    empty_cmd,	""}, */
+    {"strerror",       strerror_cmd,    "expand what errorcode this might be"},
     {"suidcells",      suidcells_cmd,	"list status of cells"},
     {"sysname",        sysname_cmd,	"read/change sysname"},
     {"version",        fsversion_cmd,   "get version of fs and fs_lib"},
@@ -319,6 +324,21 @@ connect_cmd(int argc, char **argv)
     return 0;
 }
 
+static int
+copyacl_cmd (int argc, char **argv)
+{
+    argc--;
+    argv++;
+    
+    if (argc != 2) {
+	printf ("fs: copyacl: Too many or too few arguments\n");
+	return 0;
+    }
+
+    afs_copyacl (argv[0], argv[1]);
+
+    return 0;
+}
 
 static int
 diskfree_cmd (int argc, char **argv)
@@ -686,6 +706,28 @@ listcells_cmd (int argc, char **argv)
     return 0;
 }
 
+static int
+strerror_cmd (int argc, char **argv)
+{
+    if (argc == 0) {
+	fprintf (stderr, "usage: fs strerror errno ...\n");
+    } else {
+	int error;
+	argv++;
+
+	while (--argc) {
+	    if (sscanf (argv[0], "%d", &error) != 1) {
+		fprintf (stderr, "warning: %s isn't a number\n", argv[0]);
+	    } else {
+		printf ("%d => %s\n", error, koerr_gettext(error));
+	    }
+	    argv++;
+	}
+    }
+	
+    return 0;
+}
+
 static int
 suidcells_cmd (int argc, char **argv)
 {
@@ -1072,7 +1114,7 @@ fsversion_cmd(int argc, char **argv)
 	printf ("version: extraneous arguments ignored\n");
 
     printf("fs: %s\nfs_lib: %s\n",
-	   "$Id: fs.c,v 1.65 1999/03/06 14:36:37 lha Exp $",
+	   "$Id: fs.c,v 1.72 1999/07/25 00:12:16 lha Exp $",
 	   fslib_version());
     return 0;
 }
@@ -1295,6 +1337,24 @@ arladebug_cmd (int argc, char **argv)
     return 0;
 }
 
+static int
+calculate_cmd (int argc, char **argv)
+{
+    int ret;
+    u_int32_t used, calc;
+
+    ret = fs_calculate_cache(&calc, &used);
+    if (ret)
+	warn ("fs_calculate_cache");
+    else
+	printf ("usedbytes:   %10d\n"
+		"calculated:  %10d\n"
+		"diff: c - u  %10d\n",
+		used, calc, calc - used );
+		
+    return 0;
+}
+
 void
 afs_getfid(char *path)
 {
@@ -1318,6 +1378,44 @@ afs_getfid(char *path)
 	   fid.fid.Vnode, fid.fid.Unique, cellname, fid.Cell);
 }
 
+void
+afs_copyacl(char *fromdir, char *todir)
+{
+    struct Acl *acl;
+    struct AclEntry *position;
+    struct ViceIoctl a_params;
+    int i;
+    char acltext[MAXSIZE];
+    char tmpstr[MAXSIZE];
+
+    if((acl=afs_getacl(fromdir))==NULL)
+	exit(1);
+
+    sprintf(acltext,"%d\n%d\n", acl->NumPositiveEntries,
+	    acl->NumNegativeEntries);
+    position=acl->pos;
+    for(i=0; i<acl->NumPositiveEntries; i++) {
+	sprintf(tmpstr, "%s %d\n", position->name, position->RightsMask);
+	strcat(acltext,tmpstr);
+	position=position->next;
+    }
+    position=acl->neg;
+    for(i=0; i<acl->NumNegativeEntries; i++) {
+	sprintf(tmpstr, "%s %d\n", position->name, position->RightsMask);
+	strcat(acltext,tmpstr);
+	position=position->next;
+    }
+
+    a_params.in_size=strlen(acltext);
+    a_params.out_size=0;
+    a_params.in=acltext;
+    a_params.out=0;
+
+    if(k_pioctl(todir,VIOCSETAL,&a_params,1)==-1) {
+	fserr(PROGNAME, errno, todir);
+	return;
+    }
+}
 
 void
 afs_listacl(char *path)
@@ -1438,34 +1536,42 @@ afs_setacl(char *path, char *user, char *rights)
     }
 
     if(!foundit) {
-	acl->NumPositiveEntries++;
-
-	/* We should already be at the last entry */
-	position->next=malloc(sizeof(struct AclEntry));
-	if(position->next==NULL) {
+	if (position) {
+	    position->next=malloc(sizeof(struct AclEntry));
+	    position=position->next;
+	} else {
+	    acl->pos = malloc(sizeof(struct AclEntry));
+	    position = acl->pos;
+	}
+	if(position==NULL) {
 	    printf("fs: Out of memory\n");
 	    exit(1);
 	}
-	position=position->next;
+	acl->NumPositiveEntries++;
+
 	position->next=NULL;
 	strncpy(position->name, user, MAXNAME);
 	position->RightsMask=newrights;
     }
 
-    sprintf(acltext,"%d\n%d\n", acl->NumPositiveEntries,
-	    acl->NumNegativeEntries);
-    position=acl->pos;
-    for(i=0; i<acl->NumPositiveEntries; i++) {
-	sprintf(tmpstr, "%s %d\n", position->name, position->RightsMask);
-	strcat(acltext,tmpstr);
-	position=position->next;
+    acltext[0] = 0;
+    for(position=acl->pos; position; position = position->next) {
+	if (position->RightsMask) {
+	    sprintf(tmpstr, "%s %d\n", position->name, position->RightsMask);
+	    strcat(acltext,tmpstr);
+	} else
+	    acl->NumPositiveEntries--;
     }
-    position=acl->neg;
-    for(i=0; i<acl->NumNegativeEntries; i++) {
-	sprintf(tmpstr, "%s %d\n", position->name, position->RightsMask);
-	strcat(acltext,tmpstr);
-	position=position->next;
+    for(position=acl->neg; position; position = position->next) {
+	if (position->RightsMask) {
+	    sprintf(tmpstr, "%s %d\n", position->name, position->RightsMask);
+	    strcat(acltext,tmpstr);
+	} else
+	    acl->NumNegativeEntries--;
     }
+    strncpy (tmpstr, acltext, MAXSIZE);
+    sprintf(acltext,"%d\n%d\n%s", acl->NumPositiveEntries,
+	    acl->NumNegativeEntries, tmpstr);
 
     a_params.in_size=strlen(acltext);
     a_params.out_size=0;
@@ -1577,7 +1683,7 @@ afs_print_sysname (void)
     if (ret)
 	fserr (PROGNAME, ret, NULL);
     else
-	printf ("Current sysname is `%s'\n", buf);
+	printf ("Current sysname is '%s'\n", buf);
 }
 
 /*
@@ -1722,12 +1828,12 @@ afs_whereis(char *path)
 
     while(curptr[i] && i<8) {
 	struct hostent *h;
-	addr.s_addr=htonl(curptr[i]);
+	addr.s_addr = curptr[i];
 	h=gethostbyaddr((const char *) &addr, sizeof(addr), AF_INET);
 	if (h == NULL)
 	    printf (" %s", inet_ntoa (addr));
 	else {
-	    printf(" %s",h->h_name);
+	    printf(" %s", h->h_name);
 	}
 	i++;
     }
@@ -2052,21 +2158,55 @@ skipline(char **curptr)
 static int
 getcache_cmd (int argc, char **argv)
 {
-    int ret;
-    u_int32_t max_kbytes, used_kbytes, max_vnodes, used_vnodes;
+    int ret, help = 0;
+    u_int32_t max_bytes, used_bytes, max_vnodes, used_vnodes;
+    int optind = 0, bytes_flag = 0;
 
-    ret = fs_getfilecachestats (&max_kbytes,
-				&used_kbytes,
+    struct getargs ncargs[] = {
+	{"byte", 'b', arg_flag,  
+	 NULL, "show result in byte instead of kbyte", NULL},
+	{"help", 'h', arg_flag,
+	 NULL, "get help", NULL},
+        {NULL,      0, arg_end, NULL}}, 
+				  *arg;
+			       
+    arg = ncargs;
+    arg->value = &bytes_flag; arg++;
+    arg->value = &help; arg++;
+
+    if (getarg (ncargs, argc, argv, &optind, ARG_AFSSTYLE)) {
+	arg_printusage(ncargs, "getcacheparams", NULL, ARG_AFSSTYLE);
+	return 0;
+    }
+
+    if (help) {
+	arg_printusage(ncargs, "getcacheparams", NULL, ARG_AFSSTYLE);
+	return 0;
+    }
+
+
+    ret = fs_getfilecachestats (&max_bytes,
+				&used_bytes,
 				&max_vnodes,
 				&used_vnodes);
     if (ret) {
 	fserr (PROGNAME, ret, NULL);
 	return 0;
     }
-    printf("Arla is using %lu of the cache's available %lu 1K byte blocks\n",
-	   (unsigned long)used_kbytes, (unsigned long)max_kbytes);
-    if (max_vnodes)
-	printf("(and %lu of the cache's available %lu vnodes)\n",
-	       (unsigned long)used_vnodes, (unsigned long)max_vnodes);
+    if (bytes_flag) {
+	printf("Arla is using %lu of the cache's available "
+	       "%lu byte blocks\n",
+	       (unsigned long)used_bytes, (unsigned long)max_bytes);
+	if (max_vnodes)
+	    printf("(and %lu of the cache's available %lu vnodes)\n",
+		   (unsigned long)used_vnodes, (unsigned long)max_vnodes);
+    } else {
+	printf("Arla is using %lu of the cache's available"
+	       "%lu 1K byte blocks\n",
+	       (unsigned long)used_bytes/1024, (unsigned long)max_bytes/1024);
+	if (max_vnodes)
+	    printf("(and %lu of the cache's available %lu vnodes)\n",
+		   (unsigned long)used_vnodes, (unsigned long)max_vnodes);
+    }
     return 0;
 }
