@@ -44,7 +44,7 @@
 #include <kafs.h>
 #include <ctype.h>
 
-RCSID("$Id: pts.c,v 1.21 1999/03/04 09:17:30 lha Exp $");
+RCSID("$Id: pts.c,v 1.23 1999/06/20 14:20:42 hin Exp $");
 
 static int help_cmd (int argc, char **argv);
 
@@ -178,24 +178,17 @@ static int
 examine_id(struct rx_connection *conn, int32_t id, char *idname)
 {
     prcheckentry ent;
-    int res;
+    int error;
     namelist nlist;
     idlist ilist;
     char flags_str[6];
 
-    res = PR_ListEntry(conn, id, &ent);
-    if(res != 0) {
-        if(res == PRPERM)
-            errx(1, "No permissions");
-        else if(res == PRNOENT) {
-            errx(1, "User or group doesn't exist"
-		 "; unable to find entry for %s", idname);
-	}
-        else if (res == PRDBBAD)
-            errx(1, "An error was found relating to the PDB");
-        else
-            errx(1, "PR_ListEntry returned errorcode %d", res); 
-    }
+    error = PR_ListEntry(conn, id, &ent);
+    if (error) {
+        fprintf(stderr, "examine: error %s (%d)\n",
+		koerr_gettext(error), error);
+        return 1;
+    } 
 
     nlist.len = 3;
     nlist.val = malloc(sizeof(prname) * nlist.len);
@@ -215,7 +208,6 @@ examine_id(struct rx_connection *conn, int32_t id, char *idname)
 
     flags_to_string(ent.flags << 16, flags_str); /* XXX why do i have to shift by 16? seems strange */
 
-    printf("----- %x\n", ent.flags);
     printf("Name: %s, id: %d, owner: %s, creator: %s,\n", 
            nlist.val[0], ent.id, nlist.val[1], nlist.val[2]);
     printf("  membership: %d, flags: %s, group quota: %d.\n",
@@ -351,8 +343,14 @@ pr_create(char *cell, char *name, int32_t *id, int flag, arlalib_authflags_t aut
 	printf("PR_NewEntry(%s, %d, %d, OUT)\n",
 	       name, flag, *rid);
 
-    error = PR_NewEntry(conn, name, flag, *rid, rid);
-    
+    if(*id == 0)
+	error = PR_NewEntry(conn, name, flag, *rid, rid);
+    else { 
+	error = PR_INewEntry(conn, name, *rid, 0);
+	if(error)
+	    fprintf(stderr, "pts create: error %s (%d)\n",
+		    koerr_gettext(error), error);
+    }
     arlalib_destroyconn(conn);
     return error;
 }
@@ -950,8 +948,8 @@ chown_cmd(int argc, char **argv)
 	{"cell",	0, arg_string,  NULL, "what cell to use", NULL},
 	{"noauth",	0, arg_flag,    NULL, "don't authenticate", NULL},
 	{NULL,      0, arg_end, NULL}}, 
-					 *arg;
-
+				    *arg;
+    
     arg = chownarg;
     arg->value = &name;   arg++;
     arg->value = &owner;  arg++;
@@ -982,51 +980,36 @@ chown_cmd(int argc, char **argv)
 static int
 examine_cmd (int argc, char **argv)
 {
-    const char *host = NULL;
-    const char *cell = NULL;
+    char *host;
+    char *cell = (char *) cell_getthiscell();
     struct rx_connection *connptdb = NULL;
     namelist nlist;
     idlist ilist;
     int i = 0;
-    int switches = 0;
     int noauth = 1;
+    getarg_strings users = {0 , NULL };
+    int optind = 0;
+    
+    struct getargs examinearg[] = {
+	{"nameorid",	0, arg_strings,  NULL, "user or group name",
+	 NULL, arg_mandatory},
+	{"cell",	0, arg_string,  NULL, "what cell to use",
+	 NULL, arg_optional},
+	{"noauth",	0, arg_flag,    NULL, "don't authenticate",
+	 NULL, arg_optional},
+	{NULL,      0, arg_end, NULL}}, *arg;
+    
+    arg = examinearg;
+    arg->value = &users;   arg++;
+    arg->value = &cell;   arg++;
+    arg->value = &noauth; arg++;
 
-    argc--; argv++; /* Get rid of command name */
-
-    /* pts examine needs at least one argument */
-    if(argc < 1) {
-	examine_usage();
+    if (getarg (examinearg, argc, argv, &optind, ARG_AFSSTYLE)) {
+	arg_printusage(examinearg, NULL, "chown", ARG_AFSSTYLE);
 	return 0;
     }
 
-    /* get rid of -nameorid that is allowed in front of usernames */
-    if(strcmp(argv[0], "-nameorid") == 0) {
-	argv++;
-	argc--;
-    }
-
-    /* scan for switches */
-    for(i = 0; i < argc; i++) {
-	if(strncmp(argv[i], "-cell", 2) == 0) {
-	    if(cell != NULL || (argc - 1 <= i) ) {
-		examine_usage();
-		return 0;
-	    }		
-	    switches+=2;
-	    cell = argv[i+1];
-	    argv[i] = NULL; /* don't examine "-cell" as a user/group */
-	    argv[i+1] = NULL; /* don't examine cell name as a user/group */
-	    i+=1;
-	} else if(strncmp(argv[i], "-noauth", 2) == 0) {
-	    noauth = 1;
-	    switches++;
-	}
-    }
-
-    if (cell == NULL)
-	cell = cell_getthiscell();
-
-    host = cell_findnamedbbyname (cell);
+    host = (char *) cell_findnamedbbyname (cell);
 
     if(host == NULL)
 	errx(1, "Can't find cell %s", cell);
@@ -1038,7 +1021,7 @@ examine_cmd (int argc, char **argv)
     if (connptdb == NULL)
 	errx(1, "Could not connect to ptserver");
 
-    nlist.len = argc - switches;
+    nlist.len = users.num_strings;
     nlist.val = malloc(sizeof(prname) * nlist.len);
     if(nlist.val == NULL)
 	errx(1, "Out of memory.");
@@ -1048,11 +1031,8 @@ examine_cmd (int argc, char **argv)
     if(ilist.val == NULL)
 	errx(1, "Out of memory.");
 
-    for(i = 0; i < argc - switches; i++) {
-	while(strcmp(argv[0], "") == 0)
-	    argv++;
-	strncpy(nlist.val[i], argv[0], sizeof(prname));
-	argv++;
+    for(i = 0; i < nlist.len; i++) {
+	strncpy(nlist.val[i], users.strings[i], sizeof(prname));
     }
 
     PR_NameToID(connptdb, &nlist, &ilist);
@@ -1089,22 +1069,25 @@ listmax_cmd (int argc, char **argv)
     int32_t gid = 0;
     int32_t res = 0;
     const char *host = NULL;
-    const char *cell = NULL;
+    const char *cell  = (char *) cell_getthiscell();
     struct rx_connection *connptdb = NULL;
+    int optind = 0;
+    int noauth = 0;
 
-    argc--; argv++; /* Get rid of command name */
+    struct getargs examinearg[] = {
+	{"cell",	0, arg_string,  NULL, "what cell to use",
+	 NULL, arg_optional},
+	{"noauth",	0, arg_flag,    NULL, "don't authenticate",
+	 NULL, arg_optional},
+	{NULL,      0, arg_end, NULL}}, *arg;
+    
+    arg = examinearg;
+    arg->value = &cell;   arg++;
+    arg->value = &noauth; arg++;
 
-    while(argc) {
-	if(strcmp(argv[0], "-cell") == 0 && argc > 0) {
-	    cell = argv[1];
-	    argv++;
-	    argc--;
-	} else {
-	    listmax_usage();
-	    return 0;
-	}
-	argv++;
-	argc--;
+    if (getarg (examinearg, argc, argv, &optind, ARG_AFSSTYLE)) {
+	arg_printusage(examinearg, NULL, "chown", ARG_AFSSTYLE);
+	return 0;
     }
 
     if (cell == NULL)
@@ -1190,16 +1173,10 @@ listmembershipbyname(struct rx_connection *conn, char *name)
     int32_t res = 0;
     int32_t id = 0;
     res = pts_name_to_id(conn, name, &id);
-    if(res != 0) {
-	if(res == PRPERM) {
-	    errx(1, "pts: Permission denied ; unable to get membership "
-		 "of %s (id: %d)", name, id);
-	}
-	else if(res == PRNOENT)
-	    errx(1, "pts: User or group doesn't exist so couldn't look up"
-		 "id for %s (id: %d)", name, id);
-	else 
-	    errx(1, "pts: pts_name_to_id(..) returned %d", res); /* shouldnt happen */
+    if(res) {
+	fprintf(stderr, "membership: error %s (%d)\n",
+		koerr_gettext(res), res);
+        return;
     }
     else {
 	int32_t over;
@@ -1213,19 +1190,8 @@ listmembershipbyname(struct rx_connection *conn, char *name)
 	    errx(1, "Out of memory");
 	res = PR_ListElements(conn, id, &elist, &over);
 	if(res != 0) {
-	    if(res == PRPERM) {
-		if(id == ANONYMOUSID) /* this sucks */
-		    errx(1, "pts: User or group doesn't exist so couldn't"
-			 "look up id for %s (id: %d)", name, id);
-		else
-		    errx(1, "pts: Permission denied ; unable to get "
-			 "membership of %s (id: %d)", name, id);
-	    }
-	    else if(res == PRNOENT)
-		errx(1, "pts: User or group doesn't exist so couldn't look up"
-		     "id for %s (id: %d)", name, id);
-	    else 
-		errx(1, "pts: PR_ListElements(..) returned %d", res);
+	    fprintf(stderr, "membership: error %s (%d)\n",
+		    koerr_gettext(res), res);
 	} 
 	else {
 	    if(id>=0)
@@ -1258,39 +1224,31 @@ member_cmd (int argc, char **argv)
     struct rx_connection *connptdb = NULL;
     int32_t noauth = 0;
     int i = 0;
-    int switches = 0;
+    int optind = 0;
+    getarg_strings users = {0 , NULL };
 
-    if(argc <= 1) {
-	member_usage();
+    struct getargs memarg[] = {
+	{"nameorid",	0, arg_strings,  NULL, "user or group name",
+	 NULL, arg_mandatory},
+	{"cell",	0, arg_string,  NULL, "what cell to use",
+	 NULL, arg_optional},
+	{"noauth",	0, arg_flag,    NULL, "don't authenticate",
+	 NULL, arg_optional},
+	{NULL,      0, arg_end, NULL}}, *arg;
+    
+    arg = memarg;
+    arg->value = &users;  arg++;
+    arg->value = &cell;   arg++;
+    arg->value = &noauth; arg++;
+    
+    if (getarg (memarg, argc, argv, &optind, ARG_AFSSTYLE)) {
+	arg_printusage(memarg, NULL, "membership", ARG_AFSSTYLE);
 	return 0;
     }
 
-    argc--; argv++; /* Get rid of command name */
-
-    /* get rid of -nameorid that is allowed on commandline */
-    if(strcmp("-nameorid", argv[0]) == 0) {
-	argv++;
-	argc--;
-    }
-
-    /* scan for switches */
-    for(i = 0; i < argc; i++) {
-	if(strncmp(argv[i], "-cell", 2) == 0) {
-	    if(cell != NULL || (argc - 1 <= i) ) {
-		member_usage();
-		return 0;
-	    }		
-	    switches+=2;
-	    cell = argv[i+1];
-	    argv[i] = NULL;
-	    argv[i+1] = NULL;
-	    i+=1;
-	} else if(strncmp(argv[i], "-noauth", 2) == 0) {
-	    noauth = 1;
-	    switches++;
-	}
-    }    
-
+    for(i=0;i<users.num_strings;i++)
+	printf("%s\n", users.strings[i]);
+    
     if (cell == NULL)
 	cell = cell_getthiscell();
  
@@ -1304,11 +1262,8 @@ member_cmd (int argc, char **argv)
     if (connptdb == NULL)
 	errx(1, "Could not connect to ptserver");
     
-    for(i = 0; i < argc - switches; i++) {
-	if(argv[0] != NULL)
-	    listmembershipbyname(connptdb, argv[0]);
-	argv++;
-    }
+    for(i = 0; i < users.num_strings; i++)
+	listmembershipbyname(connptdb, users.strings[i]);
 
     arlalib_destroyconn(connptdb);
     return 0;
@@ -1320,19 +1275,6 @@ listowned_usage()
     printf("Usage: pts [-id] <user or group> [-cell <cell>] [-noauth] [-help]\n");
 }
 
-static char *listowned_id;
-static char *listowned_cell;
-static int listowned_noauth;
-static int listowned_help;
-
-static struct getargs listowned_args[] = {
-    {"id",	0, arg_string,  &listowned_id,  "id of user/group", NULL, arg_mandatory},
-    {"cell",	0, arg_string,  &listowned_cell, "what cell to use", NULL},
-    {"noauth",	0, arg_flag,    &listowned_noauth, "don't authenticate", NULL},
-    {"help",	0, arg_flag,    &listowned_help, NULL, NULL},
-    {NULL,      0, arg_end, NULL}
-};
-
 int
 listowned_cmd(int argc, char **argv)
 {
@@ -1346,47 +1288,45 @@ listowned_cmd(int argc, char **argv)
     int32_t id;
     prname *name;
 
-    listowned_noauth = listowned_help = 0;
-    listowned_id = listowned_cell = NULL;
+    char *user = NULL;
+    char *cell = (char *) cell_getthiscell();
+    int noauth = 0;
+    
+    static struct getargs listowned_args[] = {
+	{"id",	   0, arg_string,  NULL,  "id of user/group",
+	 NULL, arg_mandatory},
+	{"cell",   0, arg_string,  NULL, "what cell to use",
+	 NULL},
+	{"noauth", 0,   arg_flag,    NULL, "don't authenticate", NULL},
+	{NULL,     0,    arg_end,   NULL}}, *arg;
+
+    arg = listowned_args;
+    arg->value = &user;   arg++;
+    arg->value = &cell;   arg++;
+    arg->value = &noauth; arg++;
 
     if (getarg (listowned_args, argc, argv, &optind, ARG_AFSSTYLE)) {
 	listowned_usage();
 	return 0;
     }
 
-    if(listowned_help) {
-	listowned_usage();
-	return 0;
-    }
+    printf("%s\n", user);
     
-    argc -= optind;
-    argv += optind;
-    
-    if (listowned_cell == NULL)
-	listowned_cell = (char *) cell_getthiscell();
- 
-    host = cell_findnamedbbyname (listowned_cell);
+    host = cell_findnamedbbyname (cell);
 
-    connptdb = arlalib_getconnbyname(listowned_cell,
+    connptdb = arlalib_getconnbyname(cell,
 				     host,
 				     afsprport,
 				     PR_SERVICE_ID,
-				     arlalib_getauthflag (listowned_noauth,
-							  0, 0, 0)); 
+				     arlalib_getauthflag (noauth, 0, 0, 0)); 
     if (connptdb == NULL)
 	errx(1, "Could not connect to ptserver");
 
-    res = pts_name_to_id(connptdb, listowned_id, &id);
+    res = pts_name_to_id(connptdb, user, &id);
     if(res != 0) {
-	if(res == PRPERM) {
-	    errx(1, "pts: Permission denied ; unable to get owner list "
-		 "for %s (id: %d)", listowned_id, id);
-	}
-	else if(res == PRNOENT)
-	    errx(1, "pts: User or group doesn't exist so couldn't look up"
-		 "id for %s (id: %d)", listowned_id, id);
-	else 
-	    errx(1, "pts: pts_name_to_id(..) returned %d", res); /* shouldnt happen */
+        fprintf(stderr, "listowned: error %s (%d)\n",
+		koerr_gettext(res), res);
+        return 1;
     }
 
     pr.len = PR_MAXGROUPS;
@@ -1394,9 +1334,13 @@ listowned_cmd(int argc, char **argv)
     if(pr.val == NULL)
 	errx(1, "Out of memory");
 
+    printf("%d\n",id);
     res = PR_ListOwned(connptdb, id, &pr, &over);
-    if(res != 0)
-	errx(1, "PR_ListOwned() returned %d", res); 
+    if(res != 0) {
+        fprintf(stderr, "listowned: error %s (%d)\n",
+		koerr_gettext(res), res);
+        return 1;
+    }
 
     printf("id = %d\n", id);
     printf("pr.len = %d\n", pr.len);
@@ -1406,7 +1350,7 @@ listowned_cmd(int argc, char **argv)
     if(name == NULL)
 	errx(1, "Out of memory");
 
-    printf("Groups owned by %s (id: %d) are:\n", listowned_id, id);
+    printf("Groups owned by %s (id: %d) are:\n", user, id);
 
     while(pr.val[i] != 0 && i < pr.len) {
 	pts_id_to_name(connptdb, pr.val[i++], name);
@@ -1456,7 +1400,7 @@ static SL_cmd cmds[] = {
     {"groups"},
     {"removeuser",  removeuser_cmd, "remove user from group"},
     {"rename",      rename_cmd,     "rename user/group"},
-    {"setfields",   setfields_cmd,      "not yet implemented"},
+    {"setfields",   setfields_cmd,  "not yet implemented"},
     {"setmax",      empty_cmd,      "not yet implemented"},
     {"syncdb",      syncdb_cmd,     "sync ptsdb with /etc/passwd"},
     {NULL}
@@ -1466,14 +1410,7 @@ static SL_cmd cmds[] = {
 static int
 help_cmd (int argc, char **argv)
 {
-    SL_cmd  *cmd = cmds;
-
-    while (cmd->name != NULL) {
-        if (cmd->usage != NULL)
-	  printf ("%-20s%s\n", cmd->name, cmd->usage);
-	cmd++;
-    }
-
+    sl_help(cmds, argc, argv);
     return 0;
 }
 
