@@ -47,7 +47,7 @@
 
 #include <version.h>
 
-RCSID("$Id: arla.c,v 1.100 1999/03/24 22:31:07 assar Exp $") ;
+RCSID("$Id: arla.c,v 1.107 1999/07/20 19:51:16 lha Exp $") ;
 
 enum connected_mode connected_mode = CONNECTED;
 
@@ -105,12 +105,14 @@ print_dir (VenusFid *fid, const char *name, void *v)
 	    fid->fid.Unique, name);
 }
 
-/*
- * Return 0 iff OK.
+/* An emulation of kernel lookup, convert (startdir, fname) into
+ * (startdir).  Strips away leading /afs, removes duobles slashes,
+ * and resolves symlinks.
+ * Return 0 for success, otherwise -1.
  */
 
 static int
-newwalk (VenusFid *startdir, char *fname)
+walk (VenusFid *startdir, char *fname)
 {
      VenusFid cwd = *startdir;
      char *base;
@@ -152,7 +154,7 @@ newwalk (VenusFid *startdir, char *fname)
                 break;
             }
 	 }
-	 ret = cm_lookup (cwd, base, &file, &tmpce);
+	 ret = cm_lookup (cwd, base, &file, &tmpce, TRUE);
 	 if (ret.res) {
 	     arla_warn (ADEBWARN, ret.error, "lookup(%s)", base);
 	     return -1;
@@ -254,7 +256,8 @@ arla_chdir (int argc, char **argv)
 	return 0;
     }
 
-    newwalk (&cwd, argv[1]);
+    if(walk (&cwd, argv[1]))
+	printf ("walk %s failed", argv[1]);
     return 0;
 }
 
@@ -305,7 +308,7 @@ arla_cat_et_wc (int argc, char **argv, int do_cat, int out_fd)
 	return 0;
     }
     fid = cwd;
-    if( newwalk (&fid, argv[1]) == 0) {
+    if(walk (&fid, argv[1]) == 0) {
 
 	ret = fcache_get (&e, fid, tmpce);
 	if (ret) {
@@ -390,28 +393,28 @@ help (int argc, char **argv)
 static int
 arla_conn_status (int argc, char **argv)
 {
-    conn_status (stderr);
+    conn_status ();
     return 0;
 }
 
 static int
 arla_vol_status (int argc, char **argv)
 {
-    volcache_status (stderr);
+    volcache_status ();
     return 0;
 }
 
 static int
 arla_cred_status (int argc, char **argv)
 {
-    cred_status (stderr);
+    cred_status ();
     return 0;
 }
 
 static int
 arla_fcache_status (int argc, char **argv)
 {
-    fcache_status (stderr);
+    fcache_status ();
     return 0;
 }
 
@@ -471,6 +474,33 @@ store_state (void)
     cm_store_state ();
 }
 
+char *pid_filename;
+
+static void
+write_pid_file (const char *progname)
+{
+    FILE *fp;
+
+    asprintf (&pid_filename, "/var/run/%s.pid", progname);
+    if (pid_filename == NULL)
+	return;
+    fp = fopen (pid_filename, "w");
+    if (fp == NULL)
+	return;
+    fprintf (fp, "%u", (unsigned)getpid());
+    fclose (fp);
+}
+
+static void
+delete_pid_file (void)
+{
+    if (pid_filename != NULL) {
+	unlink (pid_filename);
+	free (pid_filename);
+	pid_filename = NULL;
+    }
+}
+
 /*
  * signal handlers...
  */
@@ -480,6 +510,7 @@ sigint (int foo)
 {
     arla_warnx (ADEBMISC, "fatal signal received");
     store_state ();
+    delete_pid_file ();
     exit (0);
 }
 
@@ -487,6 +518,7 @@ static void
 sighup (int foo)
 {
     store_state ();
+    delete_pid_file ();
 }
 
 static void
@@ -625,7 +657,7 @@ krb_get_err_text(int code)
 #endif
 
 static unsigned low_vnodes, high_vnodes, low_bytes, high_bytes;
-static unsigned numcreds, numconns, numvols;
+static unsigned numcreds, numconns, numvols, dynrootlevel;
 
 static struct conf_param conf_params[] = {
     {"low_vnodes",		&low_vnodes,	 ARLA_LOW_VNODES},
@@ -636,6 +668,7 @@ static struct conf_param conf_params[] = {
     {"numconns",		&numconns,	 ARLA_NUMCREDS},
     {"numvols",			&numvols,	 ARLA_NUMVOLS},
     {"fpriority",               &fprioritylevel, FPRIO_DEFAULT},
+    {"dynroot",                 &dynrootlevel,   DYNROOT_DEFAULT},
     {NULL,			NULL,		 0}};
 
 static int test_flag;
@@ -652,9 +685,10 @@ static char *root_volume;
 static char *cache_dir;
 static int version_flag;
 static int help_flag;
-static int no_fork;
+static int fork_flag = 1;
 static int client_port = 4711;
 static int recover = 1;
+static int dynroot_enable = 0;
 static int num_workers = 16;
 
 static struct getargs args[] = {
@@ -670,7 +704,9 @@ static struct getargs args[] = {
      "the XFS device to use [/dev/xfs0]", NULL},
     {"connected-mode", 0, arg_string,	&connected_mode_string,
      "initial connected mode [conncted|fetch-only|disconnected]", NULL},
-    {"no-fork",	'n',	arg_flag,	&no_fork,
+    {"dynroot", 'D', arg_flag,	&dynroot_enable,
+     "if dynroot is enabled", NULL},
+    {"fork",	'n',	arg_negative_flag,	&fork_flag,
      "don't fork and demonize", NULL},
 #ifdef KERBEROS
     {"rxkad-level", 'r', arg_string,	&rxkad_level_string,
@@ -809,7 +845,7 @@ main (int argc, char **argv)
     signal (SIGHUP, sighup);
     umask (S_IRWXG|S_IRWXO); /* 077 */
 
-    if (!test_flag && !no_fork) {
+    if (!test_flag && fork_flag) {
 	default_log_file = "syslog";
     } else {
 	default_log_file = "/dev/stderr";
@@ -841,11 +877,15 @@ main (int argc, char **argv)
     if (cache_dir == NULL)
 	cache_dir = ARLACACHEDIR;
 
-    mkdir (cache_dir, 0777);
+    if (mkdir (cache_dir, 0777) < 0 && errno != EEXIST)
+	arla_err (1, ADEBERROR, errno, "mkdir %s", cache_dir);
     if (chdir (cache_dir) < 0)
 	arla_err (1, ADEBERROR, errno, "chdir %s", cache_dir);
 
 
+    if (dynrootlevel || dynroot_enable)
+	dynroot_setenable (TRUE);
+
     snprintf(fpriofile, sizeof(fpriofile), "%s/%s", cache_dir, "fprio");
 
     /*
@@ -943,9 +983,12 @@ main (int argc, char **argv)
 			       "Kernel-interface", &kernelpid))
 	    arla_errx (1, ADEBERROR,
 		       "Cannot create kernel-interface process");
-	if (!no_fork)
+	if (fork_flag) {
 	    daemonify ();
+	    clock_ReInit ();
+	}
 
+	write_pid_file ("arlad");
 	LWP_WaitProcess ((char *)main);
 	abort ();
     }
