@@ -43,7 +43,7 @@
 
 #include "arla_local.h"
 #ifdef RCSID
-RCSID("$Id: conn.c,v 1.50 1999/03/02 07:54:31 assar Exp $") ;
+RCSID("$Id: conn.c,v 1.53 1999/05/29 18:30:01 assar Exp $") ;
 #endif
 
 #define CONNCACHESIZE 101
@@ -452,8 +452,10 @@ internal_get (int32_t cell,
     ConnCacheEntry *e;
     ConnCacheEntry key;
 
+#if 0
     if (connected_mode == DISCONNECTED)
 	return NULL;
+#endif
 
     key.host          = host;
     key.port          = port;
@@ -665,12 +667,11 @@ static Bool
 print_conn (void *ptr, void *arg)
 {
     ConnCacheEntry *e = (ConnCacheEntry *)ptr;
-    FILE *f = (FILE *)arg;
     struct in_addr tmp;
 
     tmp.s_addr = e->host;
 
-    fprintf (f, "host = %s, port = %d, service = %d, "
+    arla_log(ADEBVLOG, "host = %s, port = %d, service = %d, "
 	     "cell = %d (%s), "
 	     "securityindex = %d, cred = %u, "
 	     "conn = %p, alive = %d, "
@@ -688,11 +689,11 @@ print_conn (void *ptr, void *arg)
  */
 
 void
-conn_status (FILE *f)
+conn_status (void)
 {
-    fprintf (f, "%u(%u) connections\n",
+    arla_log(ADEBVLOG, "%u(%u) connections\n",
 	     nactive_connections, nconnections);
-    hashtabforeach (connhtab, print_conn, f);
+    hashtabforeach (connhtab, print_conn, NULL);
 }
 
 struct clear_state {
