@@ -42,7 +42,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: fcache.c,v 1.186 1999/03/19 08:27:03 lha Exp $") ;
+RCSID("$Id: fcache.c,v 1.214.2.2 1999/08/22 12:12:08 assar Exp $") ;
 
 /*
  * Local data for this module.
@@ -185,12 +185,34 @@ expiration_time_cmp (const void *a, const void *b)
     return f1->callback.ExpirationTime - f2->callback.ExpirationTime;
 }
 
+void
+recon_hashtabadd(FCacheEntry *entry)
+{
+    hashtabadd(hashtab,entry);
+}
+ 
+void
+recon_hashtabdel(FCacheEntry *entry)
+{
+   hashtabdel(hashtab,entry);
+}
+
 /*
  * Globalnames 
  */
 
 char arlasysname[SYSNAMEMAXLEN];
 
+/*
+ * return the directory name of the cached file for `entry'
+ */
+
+int
+fcache_dir_name (FCacheEntry *entry, char *s, size_t len)
+{
+    return snprintf (s, len, "%02X", entry->index / 0x100);
+}
+
 /*
  * return the file name of the cached file for `entry'.
  */
@@ -198,7 +220,8 @@ char arlasysname[SYSNAMEMAXLEN];
 int
 fcache_file_name (FCacheEntry *entry, char *s, size_t len)
 {
-    return snprintf (s, len, "%04X", (unsigned)entry->index);
+    return snprintf (s, len, "%02X/%02X",
+		     entry->index / 0x100, entry->index % 0x100);
 }
 
 /*
@@ -214,8 +237,11 @@ fcache_extra_file_name (FCacheEntry *entry, char *s, size_t len)
 	    entry->flags.extradirp &&
 	    entry->status.FileType == TYPE_DIR);
 
-    ret = fcache_file_name (entry, s, len);
-    *s += 0x10;
+    ret = fcache_file_name (entry, s, len - 1);
+    if (ret < len - 1) {
+	s[ret++] = '@';
+	s[ret]   = '\0';
+    }
     return ret;
 }
 
@@ -226,10 +252,22 @@ static int fhopen_working;
  */
 
 static int
-fhopen (xfs_cache_handle *handle, int flags)
+fcache_fhopen (xfs_cache_handle *handle, int flags)
 {
     struct ViceIoctl vice_ioctl;
 
+#if defined(HAVE_GETFH) && defined(HAVE_FHOPEN)
+    if (xfs_probe_have (xfsv_fhget_native)) {
+	int ret;
+	fhandle_t fh;
+
+	memcpy (&fh, handle, sizeof(fh));
+	ret = fhopen (&fh, flags);
+	if (ret >= 0)
+	    return ret;
+    }
+#endif
+
     vice_ioctl.in      = (caddr_t)handle;
     vice_ioctl.in_size = sizeof(*handle);
 
@@ -246,18 +284,32 @@ fhopen (xfs_cache_handle *handle, int flags)
 int
 fcache_fhget (char *filename, xfs_cache_handle *handle)
 {
-    struct ViceIoctl vice_ioctl;
+#if defined(HAVE_GETFH) && defined(HAVE_FHOPEN)
+    if (xfs_probe_have (xfsv_fhget_native)) {
+	int ret;
+	fhandle_t fh;
 
-    if (!fhopen_working)
-	return 0;
+	ret = getfh (filename, &fh);
+	if (ret == 0)
+	    memcpy (handle, &fh, sizeof(fh));
 
-    vice_ioctl.in      = NULL;
-    vice_ioctl.in_size = 0;
-
-    vice_ioctl.out      = (caddr_t)handle;
-    vice_ioctl.out_size = sizeof(*handle);
-
-    return k_pioctl (filename, VIOC_FHGET, &vice_ioctl, 0);
+	return ret;
+    }
+#endif
+    {
+	struct ViceIoctl vice_ioctl;
+	
+	if (!fhopen_working)
+	    return 0;
+	
+	vice_ioctl.in      = NULL;
+	vice_ioctl.in_size = 0;
+	
+	vice_ioctl.out      = (caddr_t)handle;
+	vice_ioctl.out_size = sizeof(*handle);
+	
+	return k_pioctl (filename, VIOC_FHGET, &vice_ioctl, 0);
+    }
 }
 
 /*
@@ -273,7 +325,21 @@ fcache_create_file (FCacheEntry *entry)
 
     fcache_file_name (entry, fname, sizeof(fname));
     fd = open (fname, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, 0666);
-    assert (fd >= 0);
+    if (fd < 0) {
+	if (errno == ENOENT) {
+	    char dname[MAXPATHLEN];
+
+	    fcache_dir_name (entry, dname, sizeof(dname));
+	    ret = mkdir (dname, 0777);
+	    if (ret < 0)
+		arla_err (1, ADEBERROR, errno, "mkdir %s", dname);
+	    fd = open (fname, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, 0666);
+	    if (fd < 0)
+		arla_err (1, ADEBERROR, errno, "open %s", fname);
+	} else {
+	    arla_err (1, ADEBERROR, errno, "open %s", fname);
+	}
+    }
     ret = close (fd);
     assert (ret == 0);
     return fcache_fhget (fname, &entry->handle);
@@ -290,7 +356,7 @@ fcache_open_file (FCacheEntry *entry, int flag)
     char fname[MAXPATHLEN];
 
     if (fhopen_working) {
-	ret = fhopen (&entry->handle, flag);
+	ret = fcache_fhopen (&entry->handle, flag);
 	if (ret < 0 && errno == EINVAL)
 	    fhopen_working = 0;
 	else
@@ -370,9 +436,6 @@ throw_data (FCacheEntry *entry)
 	assert (ret == 0);
 	return;
     }
-    if (entry->status.FileType != TYPE_DIR) {
-	assert (sb.st_size == entry->status.Length);
-    }
     if (ftruncate (fd, 0) < 0) {
 	arla_warn (ADEBFCACHE, errno, "ftruncate");
 	ret = close (fd);
@@ -387,7 +450,7 @@ throw_data (FCacheEntry *entry)
 	fcache_extra_file_name (entry, fname, sizeof(fname));
 	unlink (fname);
     }
-    assert(usedbytes >= entry->status.Length);
+    assert(usedbytes >= sb.st_size);
     usedbytes -= sb.st_size;
     entry->flags.datap = FALSE;
     entry->flags.extradirp = FALSE;
@@ -418,8 +481,9 @@ throw_entry (FCacheEntry *entry)
     AFSCBs cbs;
     int ret;
 
-    assert (entry->flags.usedp && entry->volume != NULL
+    assert (entry->flags.usedp
 	    && CheckLock(&entry->lock) == -1);
+    /*  && entry->volume != NULL */
 
     hashtabdel (hashtab, entry);
 
@@ -449,9 +513,10 @@ throw_entry (FCacheEntry *entry)
 		arla_warn (ADEBFCACHE, ret, "RXAFS_GiveUpCallBacks");
 	}
     }
-    volcache_free (entry->volume);
-    entry->volume = NULL;
-/*    entry->inode  = 0;*/
+    if (entry->volume) {
+	volcache_free (entry->volume);
+	entry->volume = NULL;
+    }
     entry->flags.attrp = FALSE;
     entry->flags.usedp = FALSE;
     --usedvnodes;
@@ -480,8 +545,24 @@ create_node (FCacheEntry *e)
     fcache_create_file (e);
 }
 
+/*
+ *
+ */
+
+static void
+nonblockingsleep (time_t sec, int usec)
+{
+    struct timeval tv;
+
+    tv.tv_sec	= sec;
+    tv.tv_usec	= usec;
+
+    IOMGR_Select(0, NULL, NULL, NULL, &tv);
+}
+
 /*
  * Pre-create cache nodes for all entries in lrulist that doesn't have any.
+ * XXXLWP - race in listprev (nonblockingsleep).
  */
 
 static void
@@ -503,15 +584,12 @@ create_nodes (char *arg)
 	if (!entry->flags.usedp
 	    && CheckLock(&entry->lock) == 0
 	    && entry->index == 0) {
-	    struct timeval tv;
 
 	    ObtainWriteLock (&entry->lock);
 	    create_node (entry);
 	    ReleaseWriteLock (&entry->lock);
 	    ++count;
-	    tv.tv_sec = 0;
-	    tv.tv_usec = 1000;
-	    IOMGR_Select(0, NULL, NULL, NULL, &tv);
+	    nonblockingsleep (0, 1000);
 	}
     }
 
@@ -520,7 +598,7 @@ create_nodes (char *arg)
 }
 
 /*
- *
+ * Clean out nodes when we are low on things.
  */
 
 static void
@@ -542,12 +620,20 @@ cleaner (char *arg)
 
 	    while (usedvnodes > lowvnodes) {
 
+		nonblockingsleep (0, 100);
+
 		for (item = listtail (lrulist);
 		     item && usedvnodes > lowvnodes;
 		     item = prev) {
 		    prev = listprev (lrulist, item);
 		    entry = (FCacheEntry *)listdata (item);
 		    assert (entry->lru_le == item);
+
+		    if (entry->flags.kernelp
+			&& entry->priority < prio
+			&& entry->refcount == 0)
+			xfs_send_message_gc_nodes (kernel_fd, 1, &entry->fid);
+
 		    if (entry->flags.usedp
 			&& !entry->flags.attrusedp
 			&& !entry->flags.datausedp
@@ -561,6 +647,9 @@ cleaner (char *arg)
 			entry->lru_le = listaddtail (lrulist, entry);
 			break;
 		    }
+		    /* The lru changed under our feet abort, restart */
+		    if (entry->lru_le != item)
+			break;
 		}
 		if (item == NULL)
 		    break;
@@ -571,6 +660,15 @@ cleaner (char *arg)
 		 item = listprev (lrulist, item)) {
 		entry = (FCacheEntry *)listdata (item);
 		assert (entry->lru_le == item);
+
+		nonblockingsleep (0, 100);
+
+		if (entry->flags.datap
+		    && entry->flags.datausedp 
+		    && entry->priority < prio
+		    && entry->refcount == 0)
+		    xfs_send_message_gc_nodes (kernel_fd, 1, &entry->fid);
+
 		if (entry->flags.datap
 		    && !entry->flags.datausedp
 		    && entry->priority < prio
@@ -580,6 +678,9 @@ cleaner (char *arg)
 		    throw_data (entry);
 		    ReleaseWriteLock (&entry->lock);
 		}
+		/* The lru changed under our feet abort, restart */
+		if (entry->lru_le != item)
+		    break;
 	    }
 	}
 
@@ -647,6 +748,96 @@ add_to_invalidate (FCacheEntry *e)
     LWP_NoYieldSignal (invalid_heap);
 }
 
+/*
+ *
+ */
+
+static void
+emergency_remove_data (size_t sz)
+{
+    FCacheEntry *entry = NULL;
+    Listitem *item;
+    int prio;
+     
+    /*
+     * This doesn't throw away data its just destroy's our cache
+     */
+
+    xfs_send_message_gc_nodes (kernel_fd, 0, NULL);
+
+    for (prio = 0 ; prio <= fprioritylevel ; prio += 10) {
+
+	do {
+
+	    for (item = listtail (lrulist);
+		 item && usedbytes + sz > highbytes;
+		 item = listprev (lrulist, item)) {
+		entry = (FCacheEntry *)listdata (item);
+
+
+		if (entry->flags.datap
+		    && !entry->flags.datausedp
+		    && entry->priority < prio
+		    && CheckLock(&entry->lock) == 0) {
+		    ObtainWriteLock (&entry->lock);
+		    throw_data (entry);
+		    ReleaseWriteLock (&entry->lock);
+		}
+		/* The lru changed under our feet abort, restart */
+		if (entry->lru_le != item)
+		    break;
+	    }
+	} while (item && usedbytes + sz > highbytes);
+    }
+}
+
+/*
+ *
+ */
+
+static void
+emergency_remove_node (int num)
+{
+     FCacheEntry *entry = NULL;
+     Listitem *item;
+     int prio;
+
+     assert (num < highvnodes);
+
+     /*
+      * This doesn't throw away data its just destroy's our cache
+      */
+
+     xfs_send_message_gc_nodes (kernel_fd, 0, NULL);
+
+     for (prio = 0 ; prio <= fprioritylevel && num; prio += 10) {
+     
+	 do {
+	 for (item = listtail (lrulist);
+	      item && num;
+	      item = listprev (lrulist, item)) {
+	     entry = (FCacheEntry *)listdata (item);
+
+
+	     if (entry->flags.usedp
+		 && !entry->flags.datausedp
+		 && entry->priority < prio
+		 && CheckLock(&entry->lock) == 0) {
+		 ObtainWriteLock (&entry->lock);
+		 if (entry->flags.datap)
+		 throw_data (entry);
+		 throw_entry (entry);
+		 num--;
+		 ReleaseWriteLock (&entry->lock);
+	     }
+		 /* The lru changed under our feet abort and restart */
+		 if (entry->lru_le != item)
+		     break;
+	 }
+	 } while (item && num);
+     }
+}
+
 /*
  * Remove the entry least-recently used and return it.  Sleep until
  * there's an entry.
@@ -657,6 +848,9 @@ unlink_lru_entry (void)
 {
      FCacheEntry *entry = NULL;
      Listitem *item;
+
+     if (usedvnodes > lowvnodes)
+	 emergency_remove_node (1);
      
      for (;;) {
 
@@ -676,6 +870,7 @@ unlink_lru_entry (void)
 	 }
 
 	 assert (!listemptyp (lrulist));
+	 do {
 	 for (item = listtail (lrulist);
 	      item;
 	      item = listprev (lrulist, item)) {
@@ -691,42 +886,17 @@ unlink_lru_entry (void)
 		 throw_entry (entry);
 		 return entry;
 	     }
+		 /* The lru changed under our feet abort, restart */
+		 if (entry->lru_le != item)
+		     break;
 	 }
+	 } while (item);
 
 	 arla_warnx (ADEBFCACHE, "unlink_lru_entry: sleeping");
 	 LWP_WaitProcess (lrulist);
      }
 }
 
-/*
- *
- */
-
-static void
-emergency_remove_data (size_t sz)
-{
-     FCacheEntry *entry = NULL;
-     Listitem *item;
-     int prio;
-
-     for (prio = 0 ; prio <= fprioritylevel ; prio += 10) {
-     
-	 for (item = listtail (lrulist);
-	      item && usedbytes + sz > highbytes;
-	      item = listprev (lrulist, item)) {
-	     entry = (FCacheEntry *)listdata (item);
-	     if (entry->flags.datap
-		 && !entry->flags.datausedp
-		 && entry->priority < prio
-		 && CheckLock(&entry->lock) == 0) {
-		 ObtainWriteLock (&entry->lock);
-		 throw_data (entry);
-		 ReleaseWriteLock (&entry->lock);
-	     }
-	 }
-     }
-}
-
 /*
  * Return a usable entry.
  */
@@ -853,6 +1023,7 @@ fcache_recover_state (void)
 	e->flags.extradirp = tmp.flags.extradirp;
 	e->flags.mountp    = tmp.flags.mountp;
 	e->flags.sentenced = FALSE;
+	e->flags.silly 	   = FALSE;
 	e->tokens = tmp.tokens;
 	e->parent = tmp.parent;
 	e->realfid = tmp.realfid;
@@ -893,6 +1064,16 @@ findaccess (pag_t cred, AccessEntry *ae, AccessEntry **pos)
      return FALSE;
 }
 
+/*
+ * Initialize a `fs_server_context'.
+ */
+
+static void
+init_fs_server_context (fs_server_context *context)
+{
+    context->num_conns = 0;
+}
+
 /*
  * Find the next fileserver for the request in `context'.
  * Returns a ConnCacheEntry or NULL.
@@ -941,16 +1122,20 @@ find_first_fs (FCacheEntry *e,
     int cell = e->fid.Cell;
     int ret;
 
-    if (connected_mode == DISCONNECTED) {
-	context->num_conns = 0;
-	return NULL;
+    if (ve == NULL) {
+	int32_t type;
+
+	ret = volcache_getbyid (e->fid.fid.Volume, e->fid.Cell,
+				ce, &e->volume, &type);
+	if (ret)
+	    return NULL;
+	ve = e->volume;
     }
 
     ret = volume_make_uptodate (ve, ce);
     if (ret)
 	return NULL;
 
-
     if (ve->entry.volumeId[RWVOL] == e->fid.fid.Volume
 	&& ve->entry.flags & VLF_RWEXISTS)
 	bit = VLSF_RWVOL;
@@ -1123,6 +1308,7 @@ find_entry (VenusFid fid)
 
 /*
  * Mark `e' as having `callback' and notify the kernel.
+ * This might be overly hash to opened files.
  */
 
 static void
@@ -1140,6 +1326,8 @@ stale (FCacheEntry *e, AFSCallBack callback)
 	e->tokens   = 0;
 	if (e->flags.kernelp)
 	    break_callback (e->fid);
+	if (e->flags.datap)
+	    throw_data (e);
 	e->flags.kernelp   = FALSE;
 	e->flags.attrp     = FALSE;
 	e->flags.datap     = FALSE;
@@ -1300,7 +1488,7 @@ update_entry (FCacheEntry *entry,
     AccessEntry *ae;
     unsigned long bitmask = 0141777; /* REG, DIR, STICKY, USR, GRP, OTH */
 
-    if (cell_issuid_by_num (entry->volume->cell))
+    if (entry->volume && cell_issuid_by_num (entry->volume->cell))
 	bitmask |= 0006000; /* SUID, SGID */
 
     gettimeofday (&tv, NULL);
@@ -1314,7 +1502,8 @@ update_entry (FCacheEntry *entry,
     }
     if (volsync) {
 	entry->volsync  = *volsync;
-	volcache_update_volsync (entry->volume, *volsync);
+	if (entry->volume)
+	    volcache_update_volsync (entry->volume, *volsync);
     }
     entry->host     = host;
 
@@ -1338,7 +1527,10 @@ fcache_giveup_all_callbacks (void)
 	 item = listprev(lrulist, item)) {
 	FCacheEntry *entry = (FCacheEntry *)listdata(item);
 
-	if (entry->flags.attrp && entry->host != 0) {
+	if (entry->flags.attrp && 
+	    entry->flags.silly == FALSE &&
+	    entry->host != 0) {
+
 	    CredCacheEntry *ce;	
 	    ConnCacheEntry *conn;
 	    AFSCBFids fids;
@@ -1382,18 +1574,31 @@ fcache_reobtain_callbacks (void)
 	 item = listprev(lrulist, item)) {
 	FCacheEntry *entry = (FCacheEntry *)listdata(item);
 
-	if (entry->flags.usedp && entry->host != 0) {
+	if (entry->flags.usedp && 
+	    entry->flags.silly == FALSE &&
+	    entry->host != 0) {
+
 	    CredCacheEntry *ce;	
 	    ConnCacheEntry *conn;
 	    AFSFetchStatus status;
 	    AFSCallBack callback;
 	    AFSVolSync volsync;
+	    VolCacheEntry *vol;
+	    int32_t type;
 
 	    ce = cred_get (entry->fid.Cell, 0, CRED_ANY);
 	    assert (ce != NULL);
 
 	    conn = conn_get (entry->fid.Cell, entry->host, afsport,
 			     FS_SERVICE_ID, fs_probe, ce);
+	    /*
+	     * does this belong here?
+	     */
+
+	    ret = volcache_getbyid (entry->fid.fid.Volume,
+				    entry->fid.Cell, ce, &vol, &type);
+	    if (ret == 0)
+		entry->volume = vol;
 	    cred_free (ce);
 
 	    if (conn != NULL) {
@@ -1429,9 +1634,9 @@ try_next_fs (int error)
     case ARLA_VOFFLINE :
     case ARLA_VBUSY :
     case ARLA_VIO :
+    case ARLA_VNOVOL :
 	return TRUE;
     case ARLA_VMOVED :
-    case ARLA_VNOVOL :
     case 0 :
 	return FALSE;
     default :
@@ -1443,7 +1648,10 @@ try_next_fs (int error)
 /*
  * Fetch the attributes for the file in `entry' from the file_server,
  * using the credentials in `ce' and returning the connection in
- * `ret_conn' */
+ * `ret_conn'
+ *
+ * `entry' must be write-locked.
+ */
 
 static int
 do_read_attr (FCacheEntry *entry,
@@ -1459,6 +1667,20 @@ do_read_attr (FCacheEntry *entry,
 
     assert (CheckLock(&entry->lock) == -1);
 
+    /* XXX */
+    if (entry->flags.silly) {
+	entry->tokens |= XFS_ATTR_R;
+	entry->flags.attrp = TRUE;
+	return 0;
+    }
+
+    if (connected_mode == DISCONNECTED) {
+	if (entry->flags.attrp)
+	    return 0;
+	else
+	    return ENETDOWN;
+    }
+
     for (conn = find_first_fs (entry, ce, ret_context);
 	 conn != NULL;
 	 conn = find_next_fs (ret_context, conn)) {
@@ -1475,8 +1697,6 @@ do_read_attr (FCacheEntry *entry,
 
     if (ret) {
 	if (ret == RX_CALL_DEAD) {
-	    if (connected_mode == DISCONNECTED && entry->flags.attrp)
-		return 0;
 	    ret = ENETDOWN;
 	}
 	arla_warn (ADEBFCACHE, ret, "fetch-status");
@@ -1505,6 +1725,7 @@ do_read_attr (FCacheEntry *entry,
 
 /*
  * Read the attributes of `entry' from the file server and store them.
+ * `e' must be write-locked.
  */
 
 int
@@ -1516,10 +1737,14 @@ read_attr (FCacheEntry *entry, CredCacheEntry *ce)
 
     assert (CheckLock(&entry->lock) == -1);
 
+    if (dynroot_is_dynrootp (entry))
+	return dynroot_get_attr (entry);
+
+    init_fs_server_context (&context);
     ret = do_read_attr (entry, ce, &conn, &context);
-    free_fs_server_context (&context);
     if (ret)
 	return ret;
+    free_fs_server_context (&context);
     return 0;
 }
 
@@ -1579,7 +1804,7 @@ read_data (FCacheEntry *entry, ConnCacheEntry *conn, CredCacheEntry *ce)
     assert (CheckLock(&entry->lock) == -1);
 
     if (ret != sizeof(sizefs)) {
-	ret = rx_Error(call);
+	ret = conv_to_arla_errno(rx_Error(call));
 	arla_warn (ADEBFCACHE, ret, "Error reading length");
 	rx_EndCall(call, 0);
 	goto out;
@@ -1599,9 +1824,16 @@ read_data (FCacheEntry *entry, ConnCacheEntry *conn, CredCacheEntry *ce)
 	goto out;
     }
 
+    if (ftruncate(fd, sizefs) < 0) {
+	close(fd);
+	ret = errno;
+	rx_EndCall(call, 0);
+	goto out;
+    }
+
     assert (CheckLock(&entry->lock) == -1);
 
-    ret = copyrx2fd (call, fd, sizefs);
+    ret = copyrx2fd (call, fd, 0, sizefs);
     close (fd);
     if (ret) {
 	arla_warn (ADEBFCACHE, ret, "copyrx2fd");
@@ -1611,7 +1843,6 @@ read_data (FCacheEntry *entry, ConnCacheEntry *conn, CredCacheEntry *ce)
 
     assert (CheckLock(&entry->lock) == -1);
 
-    usedbytes += sizefs;		/* XXX - sync */
 
     ret = rx_EndCall (call, EndRXAFS_FetchData (call,
 						&status,
@@ -1627,6 +1858,7 @@ read_data (FCacheEntry *entry, ConnCacheEntry *conn, CredCacheEntry *ce)
     update_entry (entry, &status, &callback, &volsync,
 		  rx_HostOf(rx_PeerOf(conn->connection)),
 		  ce->cred);
+    usedbytes += sizefs;		/* XXX - sync */
 
     entry->flags.datap = TRUE;
     entry->tokens |= XFS_DATA_R | XFS_DATA_W | XFS_OPEN_NR | XFS_OPEN_NW;
@@ -1659,7 +1891,10 @@ write_data (FCacheEntry *entry, AFSStoreStatus *storestatus,
 
      assert (CheckLock(&entry->lock) == -1);
 
-     fd = fcache_open_file (entry, O_RDONLY);
+     if (entry->flags.silly)
+	 return 0;
+
+     fd = fcache_open_file (entry, O_RDWR);
      if (fd < 0) {
 	 ret = errno;
 	 arla_warn (ADEBFCACHE, ret, "open cache file %u",
@@ -1714,7 +1949,7 @@ write_data (FCacheEntry *entry, AFSStoreStatus *storestatus,
 	     break;
 	 }
 
-	 ret = copyfd2rx (fd, call, sizefs);
+	 ret = copyfd2rx (fd, call, 0, sizefs);
 	 if (ret) {
 	     rx_EndCall(call, ret);
 	     arla_warn (ADEBFCACHE, ret, "copyfd2rx");
@@ -1739,10 +1974,17 @@ write_data (FCacheEntry *entry, AFSStoreStatus *storestatus,
      }
 
      if (conn != NULL) {
-	 if (ret == 0)
+	 if (ret == 0) {
 	     update_entry (entry, &status, &callback, &volsync,
 			   rx_HostOf(rx_PeerOf(conn->connection)),
 			   ce->cred);
+	 } else {
+	     close_ret = ftruncate (fd, 0);
+	     assert (close_ret == 0);
+	     usedbytes -= sizefs;
+	     entry->flags.attrp = FALSE;
+	     entry->flags.datap = FALSE;
+	 }
      } else {
 	 ret = ENETDOWN;
      }
@@ -1829,7 +2071,7 @@ truncate_file (FCacheEntry *entry, off_t size, CredCacheEntry *ce)
 
 	sizefs = htonl (0);
 	if (rx_Write (call, &sizefs, sizeof(sizefs)) != sizeof(sizefs)) {
-	    ret = rx_Error(call);
+	    ret = conv_to_arla_errno(rx_Error(call));
 	    arla_warn (ADEBFCACHE, ret, "writing length");
 	    rx_EndCall(call, 0);
 	    break;
@@ -1876,44 +2118,64 @@ write_attr (FCacheEntry *entry,
 	    const AFSStoreStatus *store_status,
 	    CredCacheEntry *ce)
 {
-    ConnCacheEntry *conn;
     int ret = 0;
     AFSFetchStatus status;
     AFSVolSync volsync;
-    fs_server_context context;
     u_int32_t host;
 
     assert (CheckLock(&entry->lock) == -1);
 
-    for (conn = find_first_fs (entry, ce, &context);
-	 conn != NULL;
-	 conn = find_next_fs (&context, conn)) {
+    if (entry->flags.silly)
+	return 0;
 
-	host = rx_HostOf (rx_PeerOf (conn->connection));
+    if (connected_mode == CONNECTED) {
+	ConnCacheEntry *conn;
+	fs_server_context context;
 
-	ret = RXAFS_StoreStatus (conn->connection,
-				 &entry->fid.fid,
-				 store_status,
-				 &status,
-				 &volsync);
-	if (ret == RX_CALL_DEAD) {
-	    continue;
-	} else if (ret) {
-	    arla_warn (ADEBFCACHE, ret, "store-status");
+
+	for (conn = find_first_fs (entry, ce, &context);
+	     conn != NULL;
+	     conn = find_next_fs (&context, conn)) {
+
+	    host = rx_HostOf (rx_PeerOf (conn->connection));
+
+	    ret = RXAFS_StoreStatus (conn->connection,
+				     &entry->fid.fid,
+				     store_status,
+				     &status,
+				     &volsync);
+	    if (ret == RX_CALL_DEAD) {
+		continue;
+	    } else if (ret) {
+		arla_warn (ADEBFCACHE, ret, "store-status");
+		goto out;
+	    }
+	    break;
+	}
+	free_fs_server_context (&context);
+
+	if (conn == NULL) {
+	    ret = ENETDOWN;
 	    goto out;
 	}
-	break;
-    }
 
-    if (conn == NULL) {
-	ret = ENETDOWN;
-	goto out;
+	update_entry (entry, &status, NULL, &volsync, host, ce->cred);
+    } else {
+	if (store_status->Mask & SS_MODTIME) {
+	    entry->status.ClientModTime = store_status->ClientModTime;
+	    entry->status.ServerModTime = store_status->ClientModTime;
+	}
+	if (store_status->Mask & SS_OWNER)
+	    entry->status.Owner = store_status->Owner;
+	if (store_status->Mask & SS_GROUP)
+	    entry->status.Group = store_status->Group;
+	if (store_status->Mask & SS_MODEBITS)
+	    entry->status.UnixModeBits = store_status->UnixModeBits;
+	if (store_status->Mask & SS_SEGSIZE)
+	    entry->status.SegSize = store_status->SegSize;
     }
 
-    update_entry (entry, &status, NULL, &volsync, host, ce->cred);
-
 out:
-    free_fs_server_context (&context);
     return ret;
 }
 
@@ -1935,13 +2197,13 @@ create_file (FCacheEntry *dir_entry,
     AFSCallBack callback;
     AFSVolSync volsync;
     int fd;
-    fs_server_context context;
     u_int32_t host;
 
     assert (CheckLock(&dir_entry->lock) == -1);
 
     if (connected_mode == CONNECTED) {
 	ConnCacheEntry *conn;
+	fs_server_context context;
 
 	for (conn = find_first_fs (dir_entry, ce, &context);
 	     conn != NULL;
@@ -1976,33 +2238,36 @@ create_file (FCacheEntry *dir_entry,
 	update_entry (dir_entry, &status, &callback, &volsync,
 		      host, ce->cred);
     } else {
-	static int fakefid = 100;
+	static int fakefid = 1001;
 
 	OutFid.Volume = dir_entry->fid.fid.Volume;
 	OutFid.Vnode  = fakefid;
 	OutFid.Unique = fakefid;
-	++fakefid;
+	fakefid += 2;
 
-	fetch_attr->ClientModTime    = store_attr->ClientModTime;
-	fetch_attr->Owner            = store_attr->Owner;
-	fetch_attr->Group            = store_attr->Group;
-	fetch_attr->UnixModeBits     = store_attr->UnixModeBits;
-	fetch_attr->SegSize          = store_attr->SegSize;
-	fetch_attr->ServerModTime    = store_attr->ClientModTime;
-	fetch_attr->FileType         = TYPE_FILE;
-	fetch_attr->DataVersion      = 1;
 	fetch_attr->InterfaceVersion = 1;
-	fetch_attr->Author           = fetch_attr->Owner;
+	fetch_attr->FileType         = TYPE_FILE;
 	fetch_attr->LinkCount        = 1;
+	fetch_attr->Length	     = 0;
+	fetch_attr->DataVersion      = 1;
+	fetch_attr->Author           = store_attr->Owner;
+	fetch_attr->Owner            = store_attr->Owner;
+	fetch_attr->CallerAccess     = dir_entry->status.CallerAccess;
+	fetch_attr->AnonymousAccess  = dir_entry->status.AnonymousAccess;
+	fetch_attr->UnixModeBits     = store_attr->UnixModeBits;
+	fetch_attr->ParentVnode      = dir_entry->fid.fid.Vnode;
+	fetch_attr->ParentUnique     = dir_entry->fid.fid.Unique;
+	fetch_attr->SegSize          = store_attr->SegSize;
+	fetch_attr->ClientModTime    = store_attr->ClientModTime;
+	fetch_attr->ServerModTime    = store_attr->ClientModTime;
+	fetch_attr->Group            = store_attr->Group;
 	fetch_attr->SyncCount        = 0;
 	fetch_attr->spare1           = 0;
 	fetch_attr->spare2           = 0;
 	fetch_attr->spare3           = 0;
 	fetch_attr->spare4           = 0;
-	fetch_attr->ParentVnode      = dir_entry->fid.fid.Vnode;
-	fetch_attr->ParentUnique     = dir_entry->fid.fid.Unique;
-	fetch_attr->CallerAccess     = dir_entry->status.CallerAccess;
-	fetch_attr->AnonymousAccess  = dir_entry->status.AnonymousAccess;
+
+	host = dir_entry->host;
     }
 
     child_fid->Cell = dir_entry->fid.Cell;
@@ -2060,7 +2325,6 @@ create_directory (FCacheEntry *dir_entry,
 		  CredCacheEntry *ce)
 {
     int ret = 0;
-    ConnCacheEntry *conn;
     AFSFid OutFid;
     FCacheEntry *child_entry;
     AFSFetchStatus status;
@@ -2071,39 +2335,74 @@ create_directory (FCacheEntry *dir_entry,
 
     assert (CheckLock(&dir_entry->lock) == -1);
 
-    for (conn = find_first_fs (dir_entry, ce, &context);
-	 conn != NULL;
-	 conn = find_next_fs (&context, conn)) {
+    if (connected_mode == CONNECTED) {
+	ConnCacheEntry *conn;
 
-	host = rx_HostOf(rx_PeerOf(conn->connection));
+	for (conn = find_first_fs (dir_entry, ce, &context);
+	     conn != NULL;
+	     conn = find_next_fs (&context, conn)) {
 
-	ret = RXAFS_MakeDir (conn->connection,
-			     &dir_entry->fid.fid,
-			     name,
-			     store_attr,
-			     &OutFid,
-			     fetch_attr,
-			     &status,
-			     &callback,
-			     &volsync);
+	    host = rx_HostOf(rx_PeerOf(conn->connection));
 
-	if (ret == RX_CALL_DEAD) {
-	    continue;
-	} else if (ret) {
-	    free_fs_server_context (&context);
-	    arla_warn (ADEBFCACHE, ret, "MakeDir");
-	    goto out;
+	    ret = RXAFS_MakeDir (conn->connection,
+				 &dir_entry->fid.fid,
+				 name,
+				 store_attr,
+				 &OutFid,
+				 fetch_attr,
+				 &status,
+				 &callback,
+				 &volsync);
+
+	    if (ret == RX_CALL_DEAD) {
+		continue;
+	    } else if (ret) {
+		free_fs_server_context (&context);
+		arla_warn (ADEBFCACHE, ret, "MakeDir");
+		goto out;
+	    }
+	    break;
 	}
-	break;
+	free_fs_server_context (&context);
+
+	if (conn == NULL)
+	    return ENETDOWN;
+
+	status.Length = dir_entry->status.Length;
+	update_entry (dir_entry, &status, &callback, &volsync,
+		      host, ce->cred);
+    } else {
+	static int fakedir = 1000;
+
+	OutFid.Volume = dir_entry->fid.fid.Volume;
+	OutFid.Vnode  = fakedir;
+	OutFid.Unique = fakedir;
+	fakedir += 2;
+
+	fetch_attr->InterfaceVersion = 1;
+	fetch_attr->FileType         = TYPE_DIR;
+	fetch_attr->LinkCount        = 2;
+	fetch_attr->Length           = AFSDIR_PAGESIZE;
+	fetch_attr->DataVersion      = 1;
+	fetch_attr->Author           = store_attr->Owner;
+	fetch_attr->Owner            = store_attr->Owner;
+	fetch_attr->CallerAccess     = dir_entry->status.CallerAccess;
+	fetch_attr->AnonymousAccess  = dir_entry->status.AnonymousAccess;
+	fetch_attr->UnixModeBits     = store_attr->UnixModeBits;
+	fetch_attr->ParentVnode      = dir_entry->fid.fid.Vnode;
+	fetch_attr->ParentUnique     = dir_entry->fid.fid.Unique;
+	fetch_attr->SegSize          = store_attr->SegSize;
+	fetch_attr->ClientModTime    = store_attr->ClientModTime;
+	fetch_attr->ServerModTime    = store_attr->ClientModTime;
+	fetch_attr->Group            = store_attr->Group;
+	fetch_attr->SyncCount        = 0;
+	fetch_attr->spare1           = 0;
+	fetch_attr->spare2           = 0;
+	fetch_attr->spare3           = 0;
+	fetch_attr->spare4           = 0;
+
+	host = dir_entry->host;
     }
-    free_fs_server_context (&context);
-
-    if (conn == NULL)
-	return ENETDOWN;
-
-    status.Length = dir_entry->status.Length;
-    update_entry (dir_entry, &status, &callback, &volsync,
-		  host, ce->cred);
 
     child_fid->Cell = dir_entry->fid.Cell;
     child_fid->fid  = OutFid;
@@ -2114,6 +2413,9 @@ create_directory (FCacheEntry *dir_entry,
 	goto out;
     }
 
+    if(child_entry->flags.datap)
+	throw_data (child_entry);
+
     update_entry (child_entry, fetch_attr, NULL, NULL,
 		  host, ce->cred);
 
@@ -2140,6 +2442,9 @@ out:
 
 /*
  * Create a symbolic link.
+ *
+ * Note: create_symlink->flags.kernelp is not set on success
+ * and that must be done by the caller.
  */
 
 int
@@ -2160,6 +2465,9 @@ create_symlink (FCacheEntry *dir_entry,
 
     assert (CheckLock(&dir_entry->lock) == -1);
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     for (conn = find_first_fs (dir_entry, ce, &context);
 	 conn != NULL;
 	 conn = find_next_fs (&context, conn)) {
@@ -2204,10 +2512,14 @@ create_symlink (FCacheEntry *dir_entry,
 
     update_entry (child_entry, fetch_attr, NULL, NULL,
 		  host, ce->cred);
-    usedbytes += child_entry->status.Length;
+
+    /* 
+     * flags.kernelp is set in cm_symlink since the symlink
+     * might be a mountpoint and this entry is never install
+     * into the kernel.
+     */
 
     child_entry->flags.attrp = TRUE;
-    child_entry->flags.kernelp = TRUE;
     child_entry->tokens |= XFS_ATTR_R;
 	
     fcache_release(child_entry);
@@ -2236,6 +2548,9 @@ create_link (FCacheEntry *dir_entry,
 
     assert (CheckLock(&dir_entry->lock) == -1);
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     for (conn = find_first_fs (dir_entry, ce, &context);
 	 conn != NULL;
 	 conn = find_next_fs (&context, conn)) {
@@ -2291,6 +2606,9 @@ remove_file (FCacheEntry *dir_entry, const char *name, CredCacheEntry *ce)
 
     assert (CheckLock(&dir_entry->lock) == -1);
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     for (conn = find_first_fs (dir_entry, ce, &context);
 	 conn != NULL;
 	 conn = find_next_fs (&context, conn)) {
@@ -2343,6 +2661,9 @@ remove_directory (FCacheEntry *dir_entry,
 
     assert (CheckLock(&dir_entry->lock) == -1);
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     for (conn = find_first_fs (dir_entry, ce, &context);
 	 conn != NULL;
 	 conn = find_next_fs (&context, conn)) {
@@ -2398,6 +2719,9 @@ rename_file (FCacheEntry *old_dir,
     assert (CheckLock(&old_dir->lock) == -1
 	    && CheckLock(&new_dir->lock) == -1);
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     for (conn = find_first_fs (old_dir, ce, &context);
 	 conn != NULL;
 	 conn = find_next_fs (&context, conn)) {
@@ -2451,25 +2775,40 @@ getroot (VenusFid *res, CredCacheEntry *ce)
      const char *root_volume = volcache_get_rootvolume ();
      int32_t type;
      int ret;
+     const char *this_cell = cell_getthiscell ();
+     int32_t this_cell_id;
 
-     ret = volcache_getbyname (root_volume, 0, ce, &ve, &type);
-     if (ret) {
-	 arla_warnx (ADEBFCACHE, "Cannot find the root volume");
-	 return ret;
+     if (dynroot_enablep()) {
+	 res->Cell = dynroot_cellid ();
+	 res->fid.Volume = dynroot_volumeid ();
+	 res->fid.Vnode = fid.fid.Unique = 1;
+
+	 return 0;
      }
+	 
 
-     fid.Cell = 0;
+     this_cell_id = cell_name2num (this_cell);
+     if (this_cell_id == -1)
+	 arla_errx (1, ADEBERROR, "cell %s does not exist", this_cell);
+
+     ret = volcache_getbyname (root_volume, this_cell_id, ce, &ve, &type);
+     if (ret)
+	 arla_errx (1, ADEBERROR,
+		     "Cannot find the root volume (%s) in cell %s",
+		     root_volume, this_cell);
+
+     fid.Cell = this_cell_id;
      if (ve->entry.flags & VLF_ROEXISTS) {
 	 fid.fid.Volume = ve->entry.volumeId[ROVOL];
      } else if (ve->entry.flags & VLF_RWEXISTS) {
 	 arla_warnx(ADEBERROR,
-		    "getroot: %s is missing a RO clone, not good",
-		    root_volume);
+		    "getroot: %s in cell %s is missing a RO clone, not good",
+		    root_volume, this_cell);
 	 fid.fid.Volume = ve->entry.volumeId[RWVOL];
      } else {
 	 arla_errx(1, ADEBERROR,
-		   "getroot: %s has no RW or RO clone?",
-		   root_volume);
+		   "getroot: %s in cell %s has no RW or RO clone?",
+		   root_volume, this_cell);
      }
      fid.fid.Vnode = fid.fid.Unique = 1;
 
@@ -2519,6 +2858,7 @@ fcache_get (FCacheEntry **res, VenusFid fid, CredCacheEntry *ce)
     FCacheEntry *old;
     FCacheEntry *e;
     int32_t type;
+    int error;
 
     old = find_entry (fid);
     if (old) {
@@ -2554,6 +2894,7 @@ fcache_get (FCacheEntry **res, VenusFid fid, CredCacheEntry *ce)
     e->flags.mountp    = FALSE;
     e->flags.kernelp   = FALSE;
     e->flags.sentenced = FALSE;
+    e->flags.silly     = FALSE;
     e->host	       = 0;
     e->priority	       = fprio_get(fid);
     e->volume          = NULL;
@@ -2564,8 +2905,14 @@ fcache_get (FCacheEntry **res, VenusFid fid, CredCacheEntry *ce)
     if (connected_mode != DISCONNECTED) {
 	VolCacheEntry *vol;
 
-	if(volcache_getbyid (fid.fid.Volume, fid.Cell, ce, &vol, &type) == 0)
-	    e->volume = vol;
+	error = volcache_getbyid (fid.fid.Volume, fid.Cell, ce, &vol, &type);
+	if (error) {
+	    e->volume = NULL;
+	    *res = NULL;
+	    ReleaseWriteLock (&e->lock);
+	    return error;
+	}
+	e->volume = vol;
     }
 
     *res = e;
@@ -2618,6 +2965,7 @@ uptodatep (FCacheEntry *e)
 
 /*
  * Make sure that `e' has attributes and that they are up-to-date.
+ * `e' must be write-locked.
  */
 
 int
@@ -2642,6 +2990,9 @@ do_read_data (FCacheEntry *e, CredCacheEntry *ce)
     fs_server_context context;
     ConnCacheEntry *conn;
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     for (conn = find_first_fs (e, ce, &context);
 	 conn != NULL;
 	 conn = find_next_fs (&context, conn)) {
@@ -2671,6 +3022,12 @@ fcache_get_data (FCacheEntry *e, CredCacheEntry *ce)
 
     assert (CheckLock(&e->lock) == -1);
 
+    if (dynroot_is_dynrootp (e))
+	return dynroot_get_data (e);
+
+    if (e->flags.silly)
+	return 0;
+
     if (e->flags.attrp && uptodatep(e)) {
 	if (e->flags.datap)
 	    return 0;
@@ -2678,10 +3035,8 @@ fcache_get_data (FCacheEntry *e, CredCacheEntry *ce)
 	    return do_read_data (e, ce);
     } else {
 	ret = do_read_attr (e, ce, &conn, &context);
-	if (ret) {
-	    free_fs_server_context (&context);
+	if (ret)
 	    return ret;
-	}
 	if (e->flags.datap) {
 	    free_fs_server_context (&context);
 	    return 0;
@@ -2952,11 +3307,20 @@ static Bool
 print_entry (void *ptr, void *arg)
 {
     FCacheEntry *e = (FCacheEntry *)ptr;
-    FILE *f = (FILE *)arg;
 
-    fprintf (f, "(%d, %u, %u, %u)\n",
+    arla_log(ADEBVLOG, "(%d, %u, %u, %u)%s%s%s%s%s%s%s%s%s length: %ld\n",
 	     e->fid.Cell,
-	     e->fid.fid.Volume, e->fid.fid.Vnode, e->fid.fid.Unique);
+	     e->fid.fid.Volume, e->fid.fid.Vnode, e->fid.fid.Unique,
+	     e->flags.usedp?" used":"",
+	     e->flags.attrp?" attr":"",
+	     e->flags.datap?" data":"",
+	     e->flags.attrusedp?" attrused":"",
+	     e->flags.datausedp?" dataused":"",
+	     e->flags.extradirp?" extradir":"",
+	     e->flags.mountp?" mount":"",
+	     e->flags.kernelp?" kernel":"",
+	     e->flags.sentenced?" sentenced":"",
+	     e->status.Length);
     return FALSE;
 }
 
@@ -2966,13 +3330,13 @@ print_entry (void *ptr, void *arg)
  */
 
 void
-fcache_status (FILE *f)
+fcache_status (void)
 {
-    fprintf (f, "%lu (%lu-%lu) files\n"
+    arla_log(ADEBVLOG, "%lu (%lu-%lu) files\n"
 	     "%lu (%lu-%lu) bytes\n",
 	     usedvnodes, lowvnodes, highvnodes,
 	     usedbytes, lowbytes, highbytes);
-    hashtabforeach (hashtab, print_entry, f);
+    hashtabforeach (hashtab, print_entry, NULL);
 }
 
 /*
@@ -3031,6 +3395,9 @@ getacl(VenusFid fid,
 	return ret;
     }
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     ret = RX_CALL_DEAD;
 
     for (conn = find_first_fs (dire, ce, &context);
@@ -3074,6 +3441,9 @@ setacl(VenusFid fid,
 	return EINVAL;
     }
 
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     ret = RX_CALL_DEAD;
 
     for (conn = find_first_fs (dire, ce, &context);
@@ -3116,6 +3486,9 @@ getvolstat(VenusFid fid, CredCacheEntry *ce,
     int ret;
     fs_server_context context;
   
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     ret = fcache_get (&dire, fid, ce);
     if (ret) {
 	arla_warn (ADEBFCACHE, ret, "fcache_get");
@@ -3160,6 +3533,9 @@ setvolstat(VenusFid fid, CredCacheEntry *ce,
     int ret;
     fs_server_context context;
   
+    if (connected_mode != CONNECTED)
+	return EINVAL;
+
     ret = fcache_get (&dire, fid, ce);
     if (ret) {
 	arla_warn (ADEBFCACHE, ret, "fcache_get");
@@ -3187,3 +3563,85 @@ setvolstat(VenusFid fid, CredCacheEntry *ce,
     fcache_release (dire);
     return ret;
 }
+
+/*
+ * Get `fbuf' from `centry' that is opened with openflags
+ * `open_flags' and fbuf flags with `fbuf_flags'
+ *
+ * Assume that data is valid and `centry' is exclusive locked.
+ */
+
+int
+fcache_get_fbuf (FCacheEntry *centry, fbuf *fbuf,
+		 int open_flags, int fbuf_flags)
+{
+    int ret;
+    int fd;
+    unsigned len;
+    struct stat sb;
+
+    assert (CheckLock (&centry->lock) == -1);
+
+    fd = fcache_open_file (centry, open_flags);
+    if (fd < 0)
+	return errno;
+
+    if (fstat (fd, &sb)) {
+	int close_ret;
+
+	close_ret = close (fd);
+	assert (close_ret == 0);
+	return errno;
+    }
+
+    len = sb.st_size;
+
+    ret = fbuf_create (fbuf, fd, len, fbuf_flags);
+    if (ret)
+	return ret;
+    return 0;
+}
+
+/*
+ *
+ */
+Bool 
+sum_node (List *list, Listitem *li, void *arg)
+{
+    u_long *a = (u_long *) arg;
+    FCacheEntry *e = listdata (li);
+    int fd, ret;
+    struct stat sb;
+
+
+    if (e->flags.attrp &&
+	e->flags.datap &&
+	!dynroot_is_dynrootp (e)) {
+
+	fd = fcache_open_file (e, O_RDONLY);
+	if (fd < 0)
+	    return FALSE;
+	    
+	ret = fstat (fd, &sb);
+
+	*a += sb.st_size;
+
+	assert (sb.st_size == e->status.Length);
+
+	ret = close(fd);
+	assert (ret == 0);
+    }
+    
+    return FALSE;
+}
+
+
+u_long
+fcache_calculate_usage (void)
+{
+    u_long size = 0;
+
+    listiter (lrulist, sum_node, &size);
+    
+    return size;
+}
