@@ -40,7 +40,7 @@
  * The interface for the file-cache.
  */
 
-/* $Id: fcache.h,v 1.48 1999/01/10 20:25:57 map Exp $ */
+/* $Id: fcache.h,v 1.55 1999/07/25 00:09:53 lha Exp $ */
 
 #ifndef _FCACHE_H_
 #define _FCACHE_H_
@@ -107,6 +107,7 @@ typedef struct {
 	unsigned mountp : 1;	/* Is this an AFS mount point? */
 	unsigned kernelp : 1;	/* Does this entry exist in the kernel? */
 	unsigned sentenced : 1;	/* This entry should die */
+	unsigned silly : 1;	/* Instead of silly-rename */
     } flags;
     u_int tokens;		/* read/write tokens for the kernel */
 #if 0
@@ -115,11 +116,7 @@ typedef struct {
     VenusFid parent;
     VenusFid realfid;		/* Real fid (mountpoints) */
     Listitem *lru_le;		/* lru */
-#if 0
-    Listitem *invalid_le;	/* invalidation list */
-#else
     heap_ptr invalid_ptr;	/* pointer into the heap */
-#endif
     VolCacheEntry *volume;	/* pointer to the volume entry */
     unsigned priority;		/* the priority of keeping the file 0-100 */
 } FCacheEntry;
@@ -170,6 +167,9 @@ fcache_stale_entry (VenusFid fid, AFSCallBack callback);
 int
 fcache_file_name (FCacheEntry *entry, char *s, size_t len);
 
+int
+fcache_dir_name (FCacheEntry *entry, char *s, size_t len);
+
 int
 fcache_extra_file_name (FCacheEntry *entry, char *s, size_t len);
 
@@ -255,7 +255,7 @@ int
 followmountpoint (VenusFid *fid, VenusFid *parent, CredCacheEntry **ce);
 
 void
-fcache_status (FILE *f);
+fcache_status (void);
 
 int
 fcache_store_state (void);
@@ -324,4 +324,17 @@ find_next_fs (fs_server_context *context, ConnCacheEntry *prev_conn);
 void
 free_fs_server_context (fs_server_context *context);
 
+void
+recon_hashtabadd(FCacheEntry *entry);
+ 
+void
+recon_hashtabdel(FCacheEntry *entry);
+
+int
+fcache_get_fbuf (FCacheEntry *centry, fbuf *fbuf,
+		 int open_flags, int fbuf_flags);
+
+u_long
+fcache_calculate_usage (void);
+
 #endif /* _FCACHE_H_ */
