@@ -42,7 +42,7 @@
 
 #include "arla_local.h"
 #include <kafs.h>
-RCSID("$Id: fprio.c,v 1.4 1999/01/10 18:42:09 assar Exp $");
+RCSID("$Id: fprio.c,v 1.6 1999/04/20 20:58:08 map Exp $");
 
 /* Hashtable of entries by name */
 static Hashtab *fpriohashtab;
@@ -260,7 +260,6 @@ static Bool
 fprio_print_entry (void *ptr, void *arg)
 {
     struct fpriorityentry *n = (struct fpriorityentry *)ptr;
-    FILE *f = (FILE *) ptr;
     const char *cell = cell_num2name(n->fid.Cell);
     char *comment;
 
@@ -269,9 +268,9 @@ fprio_print_entry (void *ptr, void *arg)
     else
 	comment = "";
 
-    fprintf(f, "%s%d:%s:%d:%d:%d", 
-	    comment, n->priority, cell?cell:"unknowncell", n->fid.fid.Volume, 
-	    n->fid.fid.Vnode, n->fid.fid.Unique);
+    arla_log(ADEBVLOG, "%s%d:%s:%d:%d:%d", 
+	     comment, n->priority, cell?cell:"unknowncell", n->fid.fid.Volume, 
+	     n->fid.fid.Vnode, n->fid.fid.Unique);
 
     return FALSE;
 }
@@ -281,16 +280,16 @@ fprio_print_entry (void *ptr, void *arg)
  */
 
 void
-fprio_status (FILE *f)
+fprio_status (void)
 {
     time_t the_time = time(NULL);
 
-    fprintf (f, "#fprio entries\n#\n#  Date: %s\n#\n"
+    arla_log(ADEBVLOG, "#fprio entries\n#\n#  Date: %s\n#\n"
 	     "#priority range from %d to %d\n#\n"
 	     "#Syntax: (# means comment)\n"
 	     "#priority:cell:volume:vnode:unique\n",
 	     ctime(&the_time), FPRIO_MIN, FPRIO_MAX);
-    hashtabforeach (fpriohashtab, fprio_print_entry, f);
+    hashtabforeach (fpriohashtab, fprio_print_entry, NULL);
 }
 
 
