@@ -41,7 +41,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: inter.c,v 1.71 1999/02/05 02:43:33 assar Exp $") ;
+RCSID("$Id: inter.c,v 1.75 1999/07/20 19:40:44 lha Exp $") ;
 
 #include <xfs/xfs_message.h>
 
@@ -488,14 +488,15 @@ expand_sys (char *dest, size_t dst_sz, const char *src,
  * Find this entry in the directory. If the entry happens to point to
  * a mount point, then we follow that and return the root directory of
  * the volume. Hopefully this is the only place where we need to think
- * about mount points.
+ * about mount points (which are followed iff follow_mount_point).
  */
 
 Result
 cm_lookup (VenusFid dir_fid,
 	   const char *name,
 	   VenusFid *res,
-	   CredCacheEntry** ce)
+	   CredCacheEntry** ce,
+	   int follow_mount_point)
 {
      char tmp_name[MAXPATHLEN];
      Result ret;
@@ -517,14 +518,14 @@ cm_lookup (VenusFid dir_fid,
 	  ret.error = error;
 	  return ret;
      }
-     error = followmountpoint (res, &dir_fid, ce);
-     if (error) {
-	  ret.res   = -1;
-	  ret.error = error;
+     if (follow_mount_point &&
+	 (error = followmountpoint (res, &dir_fid, ce) != 0)) {
+	 ret.res   = -1;
+	 ret.error = error;
      } else {
-	  ret.res    = 0;
-	  ret.tokens = 0;
-	  ret.error  = 0;
+	 ret.res    = 0;
+	 ret.tokens = 0;
+	 ret.error  = 0;
      }
 
      /* Assume that this means a bad .. */
@@ -760,6 +761,8 @@ cm_symlink (VenusFid dir_fid,
 	 goto out;
      }
 
+     symlink_entry->flags.kernelp = TRUE;
+     
      *fetch_attr = symlink_entry->status;
      fcache_release (symlink_entry);
      ret.res   = 0;
@@ -1038,7 +1041,9 @@ cm_rename(VenusFid old_parent_fid, const char *old_name,
 	       * Lookup the old name (to get the fid of the new name)
 	       */
 
-	      error = adir_lookup_fcacheentry (old_dir, old_name, &new_fid, ce);
+	      error = adir_lookup_fcacheentry (old_dir,
+					       old_parent_fid,
+					       old_name, &new_fid, ce);
 
 	      if (error) {
 		  ret.res   = -1;
@@ -1051,7 +1056,9 @@ cm_rename(VenusFid old_parent_fid, const char *old_name,
 	       * XXX Should we check the lnkcount and clear it from fcache ?
 	       */
 
-	      error = adir_lookup_fcacheentry (new_dir, new_name, &old_fid, ce);
+	      error = adir_lookup_fcacheentry (new_dir,
+					       new_parent_fid,
+					       new_name, &old_fid, ce);
 	      if (error == 0)
 		  adir_remove (new_dir, new_name);
 
