@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -37,7 +37,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: kernel.c,v 1.17 1998/12/06 20:48:40 lha Exp $");
+RCSID("$Id: kernel.c,v 1.20 1999/07/24 14:20:28 lha Exp $");
 
 /*
  * The fd we use to talk with the kernel on.
@@ -129,6 +129,16 @@ sub_thread (void *v_myself)
     }
 }
 
+PROCESS version_pid;
+
+static void
+version_thread (void *foo)
+{
+    xfs_probe_capability (kernel_fd);
+    xfs_probe_print (NULL);
+}
+
+
 #define WORKER_STACKSIZE (16*1024)
 
 void
@@ -144,8 +154,8 @@ kernel_interface (struct kernel_args *args)
 
      workers = malloc (sizeof(*workers) * args->num_workers);
      if (workers == NULL)
-	 arla_err (1, ADEBERROR, errno, "malloc %u failed",
-		   sizeof(*workers) * args->num_workers);
+	 arla_err (1, ADEBERROR, errno, "malloc %lu failed",
+		   (unsigned long)sizeof(*workers) * args->num_workers);
 
      workers_high = args->num_workers;
      workers_used = 0;
@@ -159,6 +169,11 @@ kernel_interface (struct kernel_args *args)
 	     arla_errx (1, ADEBERROR, "CreateProcess of worker failed");
      }
 
+     if (LWP_CreateProcess (version_thread, WORKER_STACKSIZE, 1,
+			    NULL,
+			    "version", &version_pid))
+	 arla_errx (1, ADEBERROR, "CreateProcess of version thread failed");
+
      arla_warnx(ADEBKERNEL, "Arla: selecting on fd: %d", fd);
 
      for (;;) {
