@@ -37,7 +37,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: messages.c,v 1.122 1999/03/27 11:54:18 assar Exp $");
+RCSID("$Id: messages.c,v 1.152.2.1 1999/08/08 14:04:54 assar Exp $");
 
 #include <xfs/xfs_message.h>
 
@@ -52,9 +52,6 @@ RCSID("$Id: messages.c,v 1.122 1999/03/27 11:54:18 assar Exp $");
 int Log_is_open;
 DARLA_file log_data;
 
-static int 
-xfs_message_wakeup (int, struct xfs_message_wakeup*, u_int);
-
 static int 
 xfs_message_getroot (int, struct xfs_message_getroot*, u_int);
 
@@ -100,10 +97,8 @@ xfs_message_rename (int fd, struct xfs_message_rename *h, u_int size);
 static int
 xfs_message_pioctl (int fd, struct xfs_message_pioctl *h, u_int size) ;
 
-typedef int 
-(*xfs_message_function) (int, struct xfs_message_header*, u_int);
 
-static xfs_message_function rcvfuncs[] = {
+xfs_message_function rcvfuncs[] = {
 NULL,						/* version */
 (xfs_message_function)xfs_message_wakeup,	/* wakeup */
 (xfs_message_function)xfs_message_getroot,	/* getroot */
@@ -129,53 +124,8 @@ NULL,						/* invalidnode */
 (xfs_message_function)xfs_message_pioctl,	/* pioctl */
 NULL,	                                        /* wakeup_data */
 NULL,						/* updatefid */
-NULL						/* advlock */
-};
-
-static u_int *seqnums;
-
-static List *sleepers;
-
-/* number of times each type of message has been sent */
-
-static unsigned sent_stat[XFS_MSG_COUNT];
-
-/* number of times each type of message has been received */
-
-static unsigned recv_stat[XFS_MSG_COUNT];
-
-/* count of the number of messages in a write */
-
-static unsigned send_count[9];	/* 8 is the max the multiple stuff handles */
-
-static char *rcvfuncs_name[] = 
-{
-  "version",
-  "wakeup",
-  "getroot",
-  "installroot",
-  "getnode",
-  "installnode",
-  "getattr",
-  "installattr",
-  "getdata",
-  "installdata",
-  "inactivenode",
-  "invalidnode",
-  "open",
-  "put_data",
-  "put_attr",
-  "create",
-  "mkdir",
-  "link",
-  "symlink",
-  "remove",
-  "rmdir",
-  "rename",
-  "pioctl",
-  "wakeup_data",
-  "updatefid",
-  "advlock"
+NULL,						/* advlock */
+NULL						/* gc nodes */
 };
 
 /*
@@ -279,6 +229,10 @@ fcacheentry2xfsnode (VenusFid *fid,
     }
 }
 
+/*
+ * convert `xa' into `storestatus'
+ */
+
 int
 xfs_attr2afsstorestatus(struct xfs_attr *xa,
 			AFSStoreStatus *storestatus)
@@ -308,228 +262,6 @@ xfs_attr2afsstorestatus(struct xfs_attr *xa,
     return 0;
 }
 
-/*
- *
- */
-
-void
-xfs_message_init (void)
-{
-     unsigned i;
-
-     seqnums = (u_int *)malloc (sizeof (*seqnums) * getdtablesize ());
-     if (seqnums == NULL)
-	 arla_err (1, ADEBERROR, errno, "xfs_message_init: malloc");
-     for (i = 0; i < getdtablesize (); ++i)
-	  seqnums[i] = 0;
-     sleepers = listnew ();
-     if (sleepers == NULL)
-	 arla_err (1, ADEBERROR, errno, "xfs_message_init: listnew");
-     assert (sizeof(rcvfuncs) / sizeof(*rcvfuncs) == XFS_MSG_COUNT);
-     assert (sizeof(rcvfuncs_name) / sizeof(*rcvfuncs_name) == XFS_MSG_COUNT);
-}
-
-/*
- *
- */
-
-int
-xfs_message_receive (int fd, struct xfs_message_header *h, u_int size)
-{
-     unsigned opcode = h->opcode;
-
-     if (opcode >= XFS_MSG_COUNT || rcvfuncs[opcode] == NULL ) {
-	  arla_warnx (ADEBMSG, "Bad message opcode = %u", opcode);
-	  return -1;
-     }
-
-     ++recv_stat[opcode];
-
-     arla_warnx (ADEBMSG, "Rec message: opcode = %u (%s), size = %u",
-		 opcode, rcvfuncs_name[opcode], h->size);
-
-     return (*rcvfuncs[opcode])(fd, h, size);
-}
-
-/*
- *
- */
-
-static int
-xfs_message_send (int fd, struct xfs_message_header *h, u_int size)
-{
-     int res;
-     unsigned opcode = h->opcode;
-
-     h->size = size;
-     h->sequence_num = seqnums[fd]++;
-
-     if (opcode >= XFS_MSG_COUNT) {
-	  arla_warnx (ADEBMSG, "Bad message opcode = %u", opcode);
-	  return -1;
-     }
-
-     ++sent_stat[opcode];
-     ++send_count[1];
-
-     arla_warnx (ADEBMSG, "Send message: opcode = %u (%s), size = %u",
-		 opcode, rcvfuncs_name[opcode], h->size);
-
-     if ((res = write (fd, h, size)) < 0) {
-	 arla_warn (ADEBMSG, errno, "xfs_message_send: write");
-	 return -1;
-     } else
-	 return 0;
-}
-
-static int
-xfs_message_wakeup (int fd, struct xfs_message_wakeup *h, u_int size)
-{
-     Listitem *i;
-     struct xfs_message_header *w;
-
-     for (i = listhead (sleepers); i; i = listnext (sleepers, i)) {
-	  w = (struct xfs_message_header *)listdata(i);
-	  if (w->sequence_num == h->sleepers_sequence_num) {
-	       listdel (sleepers, i);
-	       memcpy (w, h, size);
-	       LWP_SignalProcess ((char *)w);
-	  }
-     }
-     return 0;
-}
-
-static int
-xfs_message_sleep (struct xfs_message_header *h)
-{
-     listaddtail (sleepers, h);
-     LWP_WaitProcess ((char *)h);
-     return ((struct xfs_message_wakeup *)h)->error;
-}
-
-static int __attribute__ ((unused))
-xfs_message_rpc (int fd, struct xfs_message_header *h, u_int size)
-{
-     if (size < sizeof (struct xfs_message_wakeup)) {
-	  arla_warnx (ADEBMSG, "xfs_message_rpc: Too small packet for rpc");
-	  return -1;
-     }
-     return xfs_message_send (fd, h, size) || xfs_message_sleep (h);
-}
-
-static int
-xfs_send_message_wakeup (int fd, u_int seqnum, int error)
-{
-     struct xfs_message_wakeup msg;
-     
-     msg.header.opcode = XFS_MSG_WAKEUP;
-     msg.sleepers_sequence_num = seqnum;
-     msg.error = error;
-     arla_warnx (ADEBMSG, "sending wakeup: seq = %u, error = %d",
-		 seqnum, error);
-     return xfs_message_send (fd, (struct xfs_message_header *)&msg, 
-			      sizeof(msg));
-}
-
-/*
- *
- */
-
-static int
-xfs_send_message_wakeup_vmultiple (int fd,
-				   u_int seqnum,
-				   int error,
-				   va_list args)
-{
-    struct iovec iovec[8];
-    struct xfs_message_header *h;
-    struct xfs_message_wakeup msg;
-    size_t size;
-    int i = 0;
-    int ret;
-
-    h = va_arg (args, struct xfs_message_header *);
-    size = va_arg (args, size_t);
-    while (h != NULL) {
-	h->size = size;
-	h->sequence_num = seqnums[fd]++;
-	assert (h->opcode >= 0 && h->opcode < XFS_MSG_COUNT);
-	assert (i < 8);
-	iovec[i].iov_base = (char *)h;
-	iovec[i].iov_len  = size;
-
-	++sent_stat[h->opcode];
-
-	arla_warnx (ADEBMSG, "Multi-send: opcode = %u (%s), size = %u",
-		    h->opcode, rcvfuncs_name[h->opcode], h->size);
-
-	h = va_arg (args, struct xfs_message_header *);
-	size = va_arg (args, size_t);
-	++i;
-    }
-    msg.header.opcode = XFS_MSG_WAKEUP;
-    msg.header.size  = sizeof(msg);
-    msg.header.sequence_num = seqnums[fd]++;
-    msg.sleepers_sequence_num = seqnum;
-    msg.error = error;
-    iovec[i].iov_base = (char *)&msg;
-    iovec[i].iov_len  = sizeof(msg);
-
-    ++sent_stat[XFS_MSG_WAKEUP];
-
-    arla_warnx (ADEBMSG, "multi-sending wakeup: seq = %u, error = %d",
-		seqnum, error);
-
-    ++i;
-    
-    ++send_count[i];
-
-    ret = writev (fd, iovec, i);
-    if (ret < 0) {
-	arla_warn (ADEBMSG, errno,
-		   "xfs_send_message_wakeup_vmultiple: writev");
-	return -1;
-    }
-    return 0;
-}
-
-static int
-xfs_send_message_wakeup_multiple (int fd,
-				  u_int seqnum,
-				  int error,
-				  ...)
-{
-    va_list args;
-    int ret;
-
-    va_start (args, error);
-    ret = xfs_send_message_wakeup_vmultiple (fd, seqnum, error, args);
-    va_end (args);
-    return ret;
-}
-
-static int
-xfs_send_message_wakeup_data (int fd, u_int seqnum, int error,
-			      void *data, int size)
-{
-     struct xfs_message_wakeup_data msg;
-     
-     msg.header.opcode = XFS_MSG_WAKEUP_DATA;
-     msg.sleepers_sequence_num = seqnum;
-     msg.error = error;
-     arla_warnx (ADEBMSG,
-		 "sending wakeup: seq = %u, error = %d", seqnum, error);
-
-     if (sizeof(msg) >= size && size != 0) {
-	 memcpy(msg.msg, data, size);
-     }
-
-     msg.len = size;
-
-     return xfs_message_send (fd, (struct xfs_message_header *)&msg, 
-			      sizeof(msg));
-}
-
 /*
  * Return true iff we should retry the operation.
  * Magic with `ce' has to be done as well.
@@ -541,6 +273,7 @@ try_again (int *ret, CredCacheEntry **ce, xfs_cred *cred, VenusFid *fid)
     switch (*ret) {
 #ifdef KERBEROS
     case RXKADEXPIRED :
+	conn_clearcred (0, cred->pag, 2);
 	cred_expire (*ce);
 	cred_free (*ce);
 	*ce = cred_get (0, cred->pag, CRED_ANY);
@@ -555,7 +288,7 @@ try_again (int *ret, CredCacheEntry **ce, xfs_cred *cred, VenusFid *fid)
 	return FALSE;
     case ARLA_VMOVED :
     case ARLA_VNOVOL :
-	if (fid) {
+	if (fid && !volcache_reliable (fid->fid.Volume, fid->Cell)) {
 	    volcache_invalidate (fid->fid.Volume, fid->Cell);
 	    return TRUE;
 	} else {
@@ -569,7 +302,11 @@ try_again (int *ret, CredCacheEntry **ce, xfs_cred *cred, VenusFid *fid)
 	*ret = ENOSPC;
 	return FALSE;
     case ARLA_VOVERQUOTA:
+#ifdef EDQUOT
 	*ret = EDQUOT;
+#else
+	*ret = ENOSPC;
+#endif
 	return FALSE;
     case ARLA_VBUSY :
 	arla_warnx (ADEBWARN, "Waiting for busy volume...");
@@ -650,7 +387,7 @@ xfs_message_getnode (int fd, struct xfs_message_getnode *h, u_int size)
      assert (ce != NULL);
 
      do {
-	 res = cm_lookup (*dirfid, h->name, &fid, &ce);
+	 res = cm_lookup (*dirfid, h->name, &fid, &ce, TRUE);
 	 if (res.res == 0)
 	     res = cm_getattr (fid, &status, &real_fid, ce, &ae);
 
@@ -741,12 +478,24 @@ xfs_message_putattr (int fd, struct xfs_message_putattr *h, u_int size)
      struct xfs_message_header *h0 = NULL;
      size_t h0_len = 0;
      int ret;
+     struct vcache log_cache;
+     FCacheEntry *fce;
+     int log_err;
 
      fid = (VenusFid *)&h->handle;
      xfs_attr2afsstorestatus(&h->attr, &status);
      ce = cred_get (fid->Cell, h->cred.pag, CRED_ANY);
      assert (ce != NULL);
 
+     if (connected_mode != CONNECTED) {
+	 ret = fcache_find (&fce, *fid);
+	 ReleaseWriteLock (&fce->lock);
+
+	 log_cache.fid = *fid;
+	 log_cache.DataVersion  = fce->status.DataVersion;
+	 log_cache.cred = h->cred;
+     }
+
      do {
 	 res.res = 0;
 	 if (XA_VALID_SIZE(&h->attr))
@@ -776,6 +525,10 @@ xfs_message_putattr (int fd, struct xfs_message_putattr *h, u_int size)
 	 }
      }
 
+     if (connected_mode != CONNECTED && ret == 0) {
+	 log_err = log_dis_setattr (&log_cache, &(h->attr));
+     }
+
      cred_free (ce);
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num, 
@@ -807,6 +560,20 @@ xfs_message_create (int fd, struct xfs_message_create *h, u_int size)
 
      parent_fid = (VenusFid *)&h->parent_handle;
      xfs_attr2afsstorestatus(&h->attr, &store_status);
+     if (connected_mode != CONNECTED) {
+	 if (!(store_status.Mask & SS_OWNER)) {
+	     store_status.Owner = h->cred.uid;
+	     store_status.Mask |= SS_OWNER;
+	 }
+	 if (!(store_status.Mask & SS_MODTIME)) {
+	     struct timeval now;
+
+	     gettimeofday (&now, NULL);
+
+	     store_status.ClientModTime = now.tv_sec;
+	     store_status.Mask |= SS_MODTIME;
+	 }
+     }
      ce = cred_get (parent_fid->Cell, h->cred.pag, CRED_ANY);
      assert (ce != NULL);
 
@@ -826,6 +593,7 @@ xfs_message_create (int fd, struct xfs_message_create *h, u_int size)
 	 ret = fcache_get (&dir_entry, *parent_fid, ce);
 	 if (ret)
 	     goto out;
+#if 1
 
 	 ret = fcache_get_data (dir_entry, ce);
 	 if (ret) {
@@ -851,6 +619,9 @@ xfs_message_create (int fd, struct xfs_message_create *h, u_int size)
 			      &msg1.node, 
 			      dir_entry->acccache);
 	 msg1.flag = 0;
+#else
+	 break_callback (*parent_fid);
+#endif
 	 fcache_release(dir_entry);
 	 
 	 ret = fcache_get (&child_entry, child_fid, ce);
@@ -869,9 +640,11 @@ xfs_message_create (int fd, struct xfs_message_create *h, u_int size)
 	 msg3.flag = 0;
 	 fcache_release(child_entry);
 
+#if 1
 	 msg1.header.opcode = XFS_MSG_INSTALLDATA;
 	 h0 = (struct xfs_message_header *)&msg1;
 	 h0_len = sizeof(msg1);
+#endif
 
 	 fcacheentry2xfsnode (&child_fid, &child_fid,
 			      &fetch_status, &msg2.node, dir_entry->acccache);
@@ -909,7 +682,9 @@ out:
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num,
 				       ret,
+#if 1
 				       h0, h0_len,
+#endif
 				       h1, h1_len,
 				       h2, h2_len,
 				       NULL, 0);
@@ -937,12 +712,30 @@ xfs_message_mkdir (int fd, struct xfs_message_mkdir *h, u_int size)
      struct xfs_message_header *h2 = NULL;
      size_t h2_len = 0;
 
-     parent_fid = (VenusFid *)&h->parent_handle;
+     struct vcache log_ent_parent, log_ent_child;
+     FCacheEntry parent_entry;
+     AFSStoreStatus log_store_status;
+
+     parent_fid = fid_translate((VenusFid *)&h->parent_handle);
 
      ce = cred_get (parent_fid->Cell, h->cred.pag, CRED_ANY);
      assert (ce != NULL);
 
      xfs_attr2afsstorestatus(&h->attr, &store_status);
+     if (connected_mode != CONNECTED) {
+	 if (!(store_status.Mask & SS_OWNER)) {
+	     store_status.Owner = h->cred.uid;
+	     store_status.Mask |= SS_OWNER;
+	 }
+	 if (!(store_status.Mask & SS_MODTIME)) {
+	     struct timeval now;
+
+	     gettimeofday (&now, NULL);
+
+	     store_status.ClientModTime = now.tv_sec;
+	     store_status.Mask |= SS_MODTIME;
+	 }
+     }
 
      do {
 	 res = cm_mkdir(*parent_fid, h->name, &store_status,
@@ -961,6 +754,7 @@ xfs_message_mkdir (int fd, struct xfs_message_mkdir *h, u_int size)
 	 if (ret)
 	     goto out;
 
+#if 1
 	 ret = fcache_get_data (dir_entry, ce);
 	 if (ret) {
 	     fcache_release(dir_entry);
@@ -987,6 +781,9 @@ xfs_message_mkdir (int fd, struct xfs_message_mkdir *h, u_int size)
 	 msg1.header.opcode = XFS_MSG_INSTALLDATA;
 	 h0 = (struct xfs_message_header *)&msg1;
 	 h0_len = sizeof(msg1);
+#else
+	 break_callback (*parent_fid);
+#endif
 	 fcache_release(dir_entry);
 
 	 ret = fcache_get (&child_entry, child_fid, ce);
@@ -1028,6 +825,19 @@ xfs_message_mkdir (int fd, struct xfs_message_mkdir *h, u_int size)
 	 msg3.node = msg2.node;
 	 h2 = (struct xfs_message_header *)&msg3;
 	 h2_len = sizeof(msg3);
+	 if (connected_mode != CONNECTED)
+	     parent_entry = *dir_entry;
+     }
+
+     if (connected_mode != CONNECTED) {
+	 log_ent_parent.fid = *parent_fid;
+	 log_ent_parent.DataVersion = parent_entry.status.DataVersion;
+	 log_ent_parent.cred = h->cred;
+	 log_store_status = store_status;
+	 log_ent_child.fid = child_fid;
+	 log_ent_child.DataVersion = 1;
+	 log_dis_mkdir (&log_ent_parent, &log_ent_child, &log_store_status,
+			h->name);
      }
 
 out:
@@ -1035,7 +845,9 @@ out:
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num,
 				       ret,
+#if 1
 				       h0, h0_len,
+#endif
 				       h1, h1_len,
 				       h2, h2_len,
 				       NULL, 0);
@@ -1080,6 +892,7 @@ xfs_message_link (int fd, struct xfs_message_link *h, u_int size)
 	 if (ret)
 	     goto out;
 
+#if 1
 	 ret = fcache_get_data (dir_entry, ce);
 	 if (ret) {
 	     fcache_release(dir_entry);
@@ -1106,6 +919,9 @@ xfs_message_link (int fd, struct xfs_message_link *h, u_int size)
 	 msg1.header.opcode = XFS_MSG_INSTALLDATA;
 	 h0 = (struct xfs_message_header *)&msg1;
 	 h0_len = sizeof(msg1);
+#else
+	 break_callback (*parent_fid);
+#endif
 	 fcache_release(dir_entry);
 
 	 fcacheentry2xfsnode (existing_fid, existing_fid,
@@ -1126,7 +942,9 @@ out:
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num,
 				       ret,
+#if 1
 				       h0, h0_len,
+#endif
 				       h1, h1_len,
 				       NULL, 0);
 
@@ -1174,6 +992,7 @@ xfs_message_symlink (int fd, struct xfs_message_symlink *h, u_int size)
 	 if (ret)
 	     goto out;
 
+#if 1
 	 ret = fcache_get_data (dir_entry, ce);
 	 if (ret) {
 	     fcache_release(dir_entry);
@@ -1200,6 +1019,9 @@ xfs_message_symlink (int fd, struct xfs_message_symlink *h, u_int size)
 	 msg1.header.opcode = XFS_MSG_INSTALLDATA;
 	 h0 = (struct xfs_message_header *)&msg1;
 	 h0_len = sizeof(msg1);
+#else
+	 break_callback (*parent_fid);
+#endif
 	 fcache_release(dir_entry);
 
 	 fcacheentry2xfsnode (&child_fid, &child_fid,
@@ -1220,28 +1042,74 @@ out:
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num,
 				       ret,
+#if 1
 				       h0, h0_len,
+#endif
 				       h1, h1_len,
 				       NULL, 0);
      return ret;
 }
 
+/* 
+ * Handle the XFS remove message in `h', that is, remove name
+ * `h->name' in directory `h->parent' with the creds from `h->cred'.
+ */
+
 static int 
 xfs_message_remove (int fd, struct xfs_message_remove *h, u_int size)
 {
      VenusFid *parent_fid;
+     VenusFid fid;
      Result res;
      CredCacheEntry *ce;
      int ret;
-     struct xfs_message_installdata msg;
+     struct xfs_message_installdata msg1;
+     struct xfs_message_installnode msg2;
      struct xfs_message_header *h0 = NULL;
      size_t h0_len = 0;
+     struct xfs_message_header *h1 = NULL;
+     size_t h1_len = 0;
+     FCacheEntry *limbo_entry = NULL;
+     unsigned link_count;
+     int limbo_entry_in_kernelp;
 
      parent_fid = (VenusFid *)&h->parent_handle;
 
      ce = cred_get (parent_fid->Cell, h->cred.pag, CRED_ANY);
      assert (ce != NULL);
 
+     do {
+	 res = cm_lookup (*parent_fid, h->name, &fid, &ce, FALSE);
+	 if (res.res)
+	     ret = res.error;
+	 else
+	     ret = res.res;
+     } while (try_again (&ret, &ce, &h->cred, parent_fid));
+
+     if (ret)
+	 goto out;
+
+     /*
+      * Fetch the linkcount of the to be removed node
+      */
+
+     ret = fcache_get (&limbo_entry, fid, ce);
+     if (ret)
+	 goto out;
+
+     ret = fcache_get_attr (limbo_entry, ce);
+     if (ret)
+	 goto out;
+     link_count = limbo_entry->status.LinkCount;
+     limbo_entry_in_kernelp = limbo_entry->flags.kernelp;
+
+     fcache_release (limbo_entry);
+     limbo_entry = NULL;
+
+     /*
+      * Do the actual work
+      */
+
      do {
 	 res = cm_remove(*parent_fid, h->name, ce);
 
@@ -1251,7 +1119,7 @@ xfs_message_remove (int fd, struct xfs_message_remove *h, u_int size)
 	     ret = res.res;
      } while (try_again (&ret, &ce, &h->cred, parent_fid));
 
-     if (res.res == 0) {
+     if (ret == 0) {
 	 FCacheEntry *dir_entry;
 
 	 ret = fcache_get (&dir_entry, *parent_fid, ce);
@@ -1264,36 +1132,81 @@ xfs_message_remove (int fd, struct xfs_message_remove *h, u_int size)
 	     goto out;
 	 }
 	     
-	 res = conv_dir (dir_entry, ce, 0,
-			 &msg.cache_handle,
-			 msg.cache_name,
-			 sizeof(msg.cache_name));
-	 if (res.res == -1) {
-	     fcache_release(dir_entry);
-	     ret = res.error;
-	     goto out;
+	 if (!dir_entry->flags.extradirp
+	     || dir_remove_name (dir_entry, h->name,
+				 &msg1.cache_handle,
+				 msg1.cache_name,
+				 sizeof(msg1.cache_name))) {
+	     res = conv_dir (dir_entry, ce, 0,
+			     &msg1.cache_handle,
+			     msg1.cache_name,
+			     sizeof(msg1.cache_name));
+	     if (res.res == -1) {
+		 fcache_release(dir_entry);
+		 ret = res.error;
+		 goto out;
+	     }
 	 }
 
-	 msg.flag = XFS_INVALID_DNLC;
-	 msg.node.tokens = res.tokens;
+	 msg1.flag = XFS_INVALID_DNLC;
+	 msg1.node.tokens = res.tokens | XFS_DATA_R;
 
 	 fcacheentry2xfsnode (parent_fid,
 			      &dir_entry->realfid,
-			      &dir_entry->status, &msg.node,
+			      &dir_entry->status, &msg1.node,
 			      dir_entry->acccache);
 	 
-	 msg.header.opcode = XFS_MSG_INSTALLDATA;
-	 h0 = (struct xfs_message_header *)&msg;
-	 h0_len = sizeof(msg);
+	 msg1.header.opcode = XFS_MSG_INSTALLDATA;
+	 h0 = (struct xfs_message_header *)&msg1;
+	 h0_len = sizeof(msg1);
 	 fcache_release(dir_entry);
+
+	 /*
+	  * Make sure that if the removed node is in the
+	  * kernel it has the right linkcount since some
+	  * might hold a reference to it.
+	  */
+
+	 ret = fcache_get (&limbo_entry, fid, ce);
+	 if (ret)
+	     goto out;
+
+	 /* Only a silly rename when this is the last file */
+	 if (link_count == 1)
+	     limbo_entry->flags.silly = TRUE;
+
+	 if (/* limbo_entry_in_kernelp */ 1) {
+
+	     ret = fcache_get_attr (limbo_entry, ce);
+	     if (ret)
+		 goto out;
+
+	     msg2.header.opcode = XFS_MSG_INSTALLNODE;
+	     msg2.node.tokens   = limbo_entry->tokens;
+	     msg2.parent_handle = h->parent_handle;
+	     strcpy(msg2.name, h->name);
+	     if (link_count == 1 && limbo_entry->status.LinkCount == 1)
+		 --limbo_entry->status.LinkCount;
+	     fcacheentry2xfsnode (&fid,
+				  &limbo_entry->realfid,
+				  &limbo_entry->status,
+				  &msg2.node,
+				  limbo_entry->acccache);
+	     
+	     h1 = (struct xfs_message_header *)&msg2;
+	     h1_len = sizeof(msg2);
+	 }
      }
 
 out:
+     if (limbo_entry)
+	 fcache_release (limbo_entry);
      cred_free (ce);
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num,
 				       ret,
 				       h0, h0_len,
+				       h1, h1_len,
 				       NULL, 0);
      return ret;
 }
@@ -1301,19 +1214,59 @@ out:
 static int 
 xfs_message_rmdir (int fd, struct xfs_message_rmdir *h, u_int size)
 {
-     VenusFid *parent_fid;
+     VenusFid *parent_fid, fid;
      Result res;
      CredCacheEntry *ce;
      int ret;
-     struct xfs_message_installdata msg;
+     struct xfs_message_installdata msg0;
      struct xfs_message_header *h0 = NULL;
      size_t h0_len = 0;
+     struct xfs_message_installdata msg1;
+     struct xfs_message_header *h1 = NULL;
+     size_t h1_len = 0;
+     FCacheEntry *limbo_entry = NULL;
+     unsigned link_count = 0;
 
      parent_fid = (VenusFid *)&h->parent_handle;
 
      ce = cred_get (parent_fid->Cell, h->cred.pag, CRED_ANY);
      assert (ce != NULL);
 
+     /*
+      * Fetch the child-entry fid.
+      */
+
+     do {
+	 res = cm_lookup (*parent_fid, h->name, &fid, &ce, FALSE);
+	 if (res.res)
+	     ret = res.error;
+	 else
+	     ret = res.res;
+     } while (try_again (&ret, &ce, &h->cred, parent_fid));
+
+     if (ret)
+	 goto out;
+
+     /*
+      * Need to get linkcount for silly rename.
+      */
+
+     ret = fcache_get (&limbo_entry, fid, ce);
+     if (ret)
+	 goto out;
+
+     ret = fcache_get_attr (limbo_entry, ce);
+     if (ret)
+	 goto out;
+     link_count = limbo_entry->status.LinkCount;
+
+     fcache_release (limbo_entry);
+     limbo_entry = NULL;
+
+     /*
+      * Do the actual work
+      */
+
      do {
 	 res = cm_rmdir(*parent_fid, h->name, ce);
 	 if (res.res)
@@ -1336,34 +1289,68 @@ xfs_message_rmdir (int fd, struct xfs_message_rmdir *h, u_int size)
 	 }
 	     
 	 res = conv_dir (dir_entry, ce, 0,
-			 &msg.cache_handle,
-			 msg.cache_name,
-			 sizeof(msg.cache_name));
+			 &msg0.cache_handle,
+			 msg0.cache_name,
+			 sizeof(msg0.cache_name));
 	 if (res.res == -1) {
 	     fcache_release(dir_entry);
 	     ret = res.error;
 	     goto out;
 	 }
-	 msg.flag = XFS_INVALID_DNLC;
-	 msg.node.tokens = res.tokens;
+	 msg0.flag = XFS_INVALID_DNLC;
+	 msg0.node.tokens = res.tokens;
 
 	 fcacheentry2xfsnode (parent_fid,
 			      &dir_entry->realfid,
-			      &dir_entry->status, &msg.node,
+			      &dir_entry->status, &msg0.node,
 			      dir_entry->acccache);
 	 
-	 msg.header.opcode = XFS_MSG_INSTALLDATA;
-	 h0 = (struct xfs_message_header *)&msg;
-	 h0_len = sizeof(msg);
+	 msg0.header.opcode = XFS_MSG_INSTALLDATA;
+	 h0 = (struct xfs_message_header *)&msg0;
+	 h0_len = sizeof(msg0);
 	 fcache_release(dir_entry);
+
+
+	 ret = fcache_get (&limbo_entry, fid, ce);
+	 if (ret)
+	     goto out;
+
+	 /* Only silly rename when this is the last reference. */
+
+	 if (link_count == 2)
+	     limbo_entry->flags.silly = TRUE;
+
+	 if (limbo_entry->flags.kernelp) {
+
+	     ret = fcache_get_attr (limbo_entry, ce);
+	     if (ret)
+		 goto out;
+
+	     msg1.header.opcode = XFS_MSG_INSTALLATTR;
+	     msg1.node.tokens   = limbo_entry->tokens;
+	     if (link_count == 2 && limbo_entry->status.LinkCount == 2)
+		 limbo_entry->status.LinkCount = 0;
+	     fcacheentry2xfsnode (&fid,
+				  &limbo_entry->realfid,
+				  &limbo_entry->status,
+				  &msg1.node,
+				  limbo_entry->acccache);
+	     
+	     h1 = (struct xfs_message_header *)&msg1;
+	     h1_len = sizeof(msg1);
+	 }
      }
 
 out:
+     if (limbo_entry)
+	 fcache_release (limbo_entry);
+
      cred_free (ce);
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num,
 				       ret,
 				       h0, h0_len,
+				       h1, h1_len,
 				       NULL, 0);
      return ret;
 }
@@ -1489,6 +1476,9 @@ xfs_message_putdata (int fd, struct xfs_message_putdata *h, u_int size)
      CredCacheEntry *ce;
      int ret;
      AFSStoreStatus status;
+     struct vcache log_cache;
+     FCacheEntry *fce;
+     int log_err;
 
      fid = (VenusFid *)&h->handle;
      xfs_attr2afsstorestatus(&h->attr, &status);
@@ -1496,6 +1486,15 @@ xfs_message_putdata (int fd, struct xfs_message_putdata *h, u_int size)
      ce = cred_get (fid->Cell, h->cred.pag, CRED_ANY);
      assert (ce != NULL);
 
+     if (connected_mode != CONNECTED) {
+	 ret = fcache_find (&fce, *fid);
+
+	 ReleaseWriteLock (&fce->lock);
+
+	 log_cache.fid  = *fid;
+	 log_cache.cred = h->cred;
+     }
+
      do {
 	 res = cm_close(*fid, h->flag, &status, ce);
 	 if (res.res)
@@ -1504,8 +1503,14 @@ xfs_message_putdata (int fd, struct xfs_message_putdata *h, u_int size)
 	     ret = res.res;
      } while (try_again (&ret, &ce, &h->cred, fid));
 	 
-     if (ret)
+     if (ret == 0) {
+	 if (connected_mode != CONNECTED) {
+	     log_cache.DataVersion = ++fce->status.DataVersion;
+	     log_err = log_dis_store (&log_cache);
+	 }
+     } else {
 	 arla_warn (ADEBMSG, ret, "xfs_message_putdata: cm_close");
+     }
 
      cred_free (ce);
      xfs_send_message_wakeup (fd, h->header.sequence_num,
@@ -1540,7 +1545,7 @@ xfs_message_getdata (int fd, struct xfs_message_getdata *h, u_int size)
 	     ret = res.res;
      } while (try_again (&ret, &ce, &h->cred, fid));
 
-     if (res.res == 0) {
+     if (ret == 0) {
 	  fcacheentry2xfsnode (fid, &real_fid, &status, &msg.node, ae);
 	  if (status.FileType == TYPE_DIR) {
 	       FCacheEntry *entry;
@@ -1567,12 +1572,13 @@ xfs_message_getdata (int fd, struct xfs_message_getdata *h, u_int size)
 	  } else {
 	       res = cm_open (*fid, ce, h->tokens, &msg.cache_handle,
 			      msg.cache_name, sizeof(msg.cache_name));
-	       if (res.res != -1)
+	       ret = res.error;
+	       if (ret == 0)
 		   msg.node.tokens = res.tokens;
 	  }
      }
 
-     if (res.res != -1) {
+     if (ret == 0) {
 	 msg.header.opcode = XFS_MSG_INSTALLDATA;
 	 h0 = (struct xfs_message_header *)&msg;
 	 h0_len = sizeof(msg);
@@ -1582,7 +1588,7 @@ out:
      cred_free (ce);
      xfs_send_message_wakeup_multiple (fd,
 				       h->header.sequence_num,
-				       res.res ? res.error : res.res,
+				       ret,
 				       h0, h0_len,
 				       NULL, 0);
 
@@ -1731,7 +1737,8 @@ viocgetacl(int fd, struct xfs_message_pioctl *h, u_int size)
  
     xfs_send_message_wakeup_data (fd, h->header.sequence_num, error,
 				  opaque.val, opaque.len);
-    free (opaque.val);
+    if (error == 0)
+	free (opaque.val);
     return 0;
 }
 
@@ -1895,6 +1902,7 @@ viocsetvolstat(int fd, struct xfs_message_pioctl *h, u_int size)
 
     ptr = h->msg + sizeof (AFSFetchVolumeStatus);
 
+#if 0
     if (*ptr) {
 	strncpy (volumename, ptr, AFSNAMEMAX);
 	ptr += strlen (ptr);
@@ -1914,6 +1922,11 @@ viocsetvolstat(int fd, struct xfs_message_pioctl *h, u_int size)
     }
 
     strncpy (motd, ptr, AFSOPAQUEMAX);
+#else
+    volumename[0] = '\0';
+    offlinemsg[0] = '\0';
+    motd[0] = '\0';
+#endif
 
     do {
 	error = setvolstat (fid, ce, &outvolstat, volumename,
@@ -1959,6 +1972,11 @@ read_mount_point (VenusFid fid, const char *filename, fbuf *the_fbuf,
 	return error;
     }
 
+    if (mp_entry->status.Length == 0) {
+	fcache_release(mp_entry);
+	return EINVAL;
+    }
+
     error = fcache_get_attr (mp_entry, ce);
     if (error) {
 	fcache_release(mp_entry);
@@ -2122,7 +2140,7 @@ viocwhereis(int fd, struct xfs_message_pioctl *h, u_int size)
 
 /*
  * Return all db servers for a particular cell.
- */
+ */ 
 
 static int
 vioc_get_cell(int fd, struct xfs_message_pioctl *h, u_int size)
@@ -2301,13 +2319,15 @@ token_for_cell (int fd, struct xfs_message_pioctl *h, u_int size,
 
     ct.AuthHandle = cred->kvno;
     memcpy (ct.HandShakeKey, cred->session, sizeof(cred->session));
-    ct.ViceId = h->cred.uid;
+    ct.ViceId         = ce->uid;
     ct.BeginTimestamp = cred->issue_date + 1;
     ct.EndTimestamp   = ce->expire;
 
-    tmp = 0;
+    tmp = cred->ticket_st.length;
     memcpy (p, &tmp, sizeof(tmp));
     p += sizeof(tmp);
+    memcpy (p, cred->ticket_st.dat, tmp);
+    p += tmp;
     tmp = sizeof(ct);
     memcpy (p, &tmp, sizeof(tmp));
     p += sizeof(tmp);
@@ -2430,6 +2450,9 @@ viocsettok (int fd, struct xfs_message_pioctl *h, u_int size)
 	
     cell = cell_name2num(strlwr(c.realm));
 
+    if (cell == -1)
+	return ENOENT;
+
     conn_clearcred (cell, h->cred.pag, 2);
     fcache_purge_cred(h->cred.pag, cell);
     cred_add (h->cred.pag, CRED_KRB4, 2, cell, ct.EndTimestamp,
@@ -2511,8 +2534,7 @@ viocconnect(int fd, struct xfs_message_pioctl *h, u_int size)
 		DARLA_Close(&log_data);
 		Log_is_open = 0;
 
-		do_replay(ARLACACHEDIR"/discon_log",
-			  log_data.log_entries, 0);
+		do_replay("discon_log", log_data.log_entries, 0);
 	    }
 	    if (connected_mode == DISCONNECTED) {
 		connected_mode = CONNECTED ;
@@ -2681,10 +2703,11 @@ viocvenuslog (int fd, struct xfs_message_pioctl *h, u_int size)
     if (!all_powerful_p(&h->cred))
 	return EPERM;
 	    
-    conn_status (stderr);
-    volcache_status (stderr);
-    cred_status (stderr);
-    fcache_status (stderr);
+    conn_status ();
+    volcache_status ();
+    cred_status ();
+    fcache_status ();
+    fprio_status ();
 #ifdef RXDEBUG
     rx_PrintStats(stderr);
 #endif
@@ -2780,7 +2803,7 @@ viocsetcachesize (int fd, struct xfs_message_pioctl *h, u_int size)
 static int
 viocckserv (int fd, struct xfs_message_pioctl *h, u_int size)
 {
-    int32_t cell = 0; /* Default local cell */
+    int32_t cell = cell_name2num (cell_getthiscell());
     int flags = 0;
     int num_entries;
     u_int32_t hosts[CKSERV_MAXSERVERS + 1];
@@ -2827,6 +2850,8 @@ viocgetcacheparms (int fd, struct xfs_message_pioctl *h, u_int size)
     parms[1] = fcache_usedbytes() / 1024;
     parms[2] = fcache_highvnodes();
     parms[3] = fcache_usedvnodes();
+    parms[4] = fcache_highbytes();
+    parms[5] = fcache_usedbytes();
 
     h->outsize = sizeof(parms);
     return xfs_send_message_wakeup_data(fd, h->header.sequence_num, 0,
@@ -2892,6 +2917,29 @@ vioc_gcpags (int fd, struct xfs_message_pioctl *h, u_int size)
     return 0;
 }
 
+/*
+ *
+ */
+
+static int
+vioc_calculate_cache (int fd, struct xfs_message_pioctl *h, u_int size)
+{
+    u_int32_t parms[16];
+    
+    memset(parms, 0, sizeof(parms));
+
+    h->outsize = sizeof(parms);
+
+    parms[0] = fcache_calculate_usage();
+    parms[1] = fcache_usedbytes();
+
+    arla_warnx (ADEBMISC, 
+		"diskusage = %d, usedbytes = %d", 
+		parms[0], parms[1]);
+    
+    return xfs_send_message_wakeup_data (fd, h->header.sequence_num, 0,
+					 &parms, sizeof(parms));
+}
 /*
  * Handle a pioctl message in `h'
  */
@@ -2904,89 +2952,194 @@ xfs_message_pioctl (int fd, struct xfs_message_pioctl *h, u_int size)
     switch(h->opcode) {
 #ifdef KERBEROS
     case VIOCSETTOK:
+#ifdef VIOCSETTOK_32
+    case VIOCSETTOK_32:
+#endif
 	error = viocsettok (fd, h, size);
 	break;
     case VIOCGETTOK :
+#ifdef VIOCGETTOK_32
+    case VIOCGETTOK_32:
+#endif
 	return viocgettok (fd, h, size);
     case VIOCUNPAG:
     case VIOCUNLOG:
+#ifdef VIOCUNPAG_32
+    case VIOCUNPAG_32:
+#endif
+#ifdef VIOCUNLOG_32
+    case VIOCUNLOG_32:
+#endif
 	error = viocunlog (fd, h, size);
 	break;
 #endif /* KERBEROS */
     case VIOCCONNECTMODE:
+#ifdef VIOCCONNECTMODE_32
+    case VIOCCONNECTMODE_32:
+#endif
 	error = viocconnect(fd, h, size);
 	break;
     case VIOCFLUSH:
+#ifdef VIOCFLUSH_32
+    case VIOCFLUSH_32:
+#endif
         error = viocflush(fd, h, size);
 	break;
     case VIOC_FLUSHVOLUME:
+#ifdef VIOC_FLUSHVOLUME_32
+    case VIOC_FLUSHVOLUME_32:
+#endif
 	error = viocflushvolume(fd, h, size);
 	break;
     case VIOCGETFID:
+#ifdef VIOCGETFID_32
+    case VIOCGETFID_32:
+#endif
 	return viocgetfid (fd, h, size);
     case VIOCGETAL:
+#ifdef VIOCGETAL_32
+    case VIOCGETAL_32:
+#endif
 	error = viocgetacl(fd, h, size);
 	break;
     case VIOCSETAL:
+#ifdef VIOCSETAL_32
+    case VIOCSETAL_32:
+#endif
 	error = viocsetacl(fd, h, size);
 	break;
     case VIOCGETVOLSTAT:
+#ifdef VIOCGETVOLSTAT_32
+    case VIOCGETVOLSTAT_32:
+#endif
 	error = viocgetvolstat(fd, h, size);
 	break;
     case VIOCSETVOLSTAT:
+#ifdef VIOCSETVOLSTAT_32
+    case VIOCSETVOLSTAT_32:
+#endif
 	error = viocsetvolstat(fd, h, size);
 	break;
     case VIOC_AFS_STAT_MT_PT:
+#ifdef VIOC_AFS_STAT_MT_PT_32
+    case VIOC_AFS_STAT_MT_PT_32:
+#endif
 	error = vioc_afs_stat_mt_pt(fd, h, size);
 	break;
     case VIOC_AFS_DELETE_MT_PT:
+#ifdef VIOC_AFS_DELETE_MT_PT_32
+    case VIOC_AFS_DELETE_MT_PT_32:
+#endif
 	error = vioc_afs_delete_mt_pt(fd, h, size);
 	break;
     case VIOCWHEREIS:
+#ifdef VIOCWHEREIS_32
+    case VIOCWHEREIS_32:
+#endif
 	error = viocwhereis(fd, h, size);
 	break;
     case VIOCNOP:
+#ifdef VIOCNOP_32
+    case VIOCNOP_32:
+#endif
 	error = EINVAL;
 	break;
     case VIOCGETCELL:
+#ifdef VIOCGETCELL_32
+    case VIOCGETCELL_32:
+#endif
 	return vioc_get_cell(fd, h, size);
     case VIOC_GETCELLSTATUS:
+#ifdef VIOC_GETCELLSTATUS_32
+    case VIOC_GETCELLSTATUS_32:
+#endif
 	return vioc_get_cellstatus(fd, h, size);
     case VIOC_SETCELLSTATUS:
+#ifdef VIOC_SETCELLSTATUS_32
+    case VIOC_SETCELLSTATUS_32:
+#endif
 	return vioc_set_cellstatus(fd, h, size);
     case VIOCNEWCELL:
+#ifdef VIOCNEWCELL_32
+    case VIOCNEWCELL_32:
+#endif
 	return vioc_new_cell(fd, h, size);
     case VIOC_VENUSLOG:
+#ifdef VIOC_VENUSLOG_32
+    case VIOC_VENUSLOG_32:
+#endif
 	error = viocvenuslog (fd, h, size);
 	break;
     case VIOC_AFS_SYSNAME:
+#ifdef VIOC_AFS_SYSNAME_32
+    case VIOC_AFS_SYSNAME_32:
+#endif
 	return vioc_afs_sysname (fd, h, size);
     case VIOC_FILE_CELL_NAME:
+#ifdef VIOC_FILE_CELL_NAME_32
+    case VIOC_FILE_CELL_NAME_32:
+#endif
 	return viocfilecellname (fd, h, size);
     case VIOC_GET_WS_CELL:
+#ifdef VIOC_GET_WS_CELL_32
+    case VIOC_GET_WS_CELL_32:
+#endif
 	return viocgetwscell (fd, h, size);
     case VIOCSETCACHESIZE:
+#ifdef VIOCSETCACHESIZE_32
+    case VIOCSETCACHESIZE_32:
+#endif
 	error = viocsetcachesize (fd, h, size);
 	break;
     case VIOCCKSERV:
+#ifdef VIOCCKSERV_32
+    case VIOCCKSERV_32:
+#endif
 	return viocckserv (fd, h, size);
     case VIOCGETCACHEPARAMS:
+#ifdef VIOCGETCACHEPARAMS_32
+    case VIOCGETCACHEPARAMS_32:
+#endif
 	return viocgetcacheparms (fd, h, size);
-    case VIOC_GETRXKCRYPT :
+    case VIOC_GETRXKCRYPT:
+#ifdef VIOC_GETRXKCRYPT_32
+    case VIOC_GETRXKCRYPT_32:
+#endif
 	return getrxkcrypt(fd, h, size);
-    case VIOC_SETRXKCRYPT :
+    case VIOC_SETRXKCRYPT:
+#ifdef VIOC_SETRXKCRYPT_32
+    case VIOC_SETRXKCRYPT_32:
+#endif
 	error = setrxkcrypt(fd, h, size);
 	break;
     case VIOC_FPRIOSTATUS:
+#ifdef VIOC_FPRIOSTATUS_32
+    case VIOC_FPRIOSTATUS_32:
+#endif
 	error = vioc_fpriostatus(fd, h, size);
 	break;
     case VIOC_AVIATOR:
+#ifdef VIOC_AVIATOR_32
+    case VIOC_AVIATOR_32:
+#endif
 	return viocaviator (fd, h, size);
-    case VIOC_ARLADEBUG :
+    case VIOC_ARLADEBUG:
+#ifdef VIOC_ARLADEBUG_32
+    case VIOC_ARLADEBUG_32:
+#endif
 	return vioc_arladebug (fd, h, size);
-    case VIOC_GCPAGS :
+    case VIOC_GCPAGS:
+#ifdef VIOC_GCPAGS_32
+    case VIOC_GCPAGS_32:
+#endif
 	error = vioc_gcpags (fd, h, size);
 	break;
+    case VIOC_CALCULATE_CACHE:
+#ifdef VIOC_CALCULATE_CACHE_32
+    case VIOC_CALCULATE_CACHE_32:
+#endif
+	error = vioc_calculate_cache (fd, h, size);
+	break;
     default:
 	arla_warnx (ADEBMSG, "unknown pioctl call %d", h->opcode);
 	error = EINVAL ;
@@ -2996,3 +3149,4 @@ xfs_message_pioctl (int fd, struct xfs_message_pioctl *h, u_int size)
     
     return 0;
 }
+
