@@ -33,7 +33,7 @@
 
 #include "arla_local.h"
 
-RCSID("$Id: reconnect.c,v 1.11 1999/02/15 04:49:43 art Exp $");
+RCSID("$Id: reconnect.c,v 1.19 1999/07/24 22:24:02 lha Exp $");
 
 static int reconnect_nonmute(struct vcache *, int, struct timeval);
 static int reconnect_putattr(struct vcache *, struct xfs_attr *);
@@ -120,7 +120,7 @@ do_replay(char *log_file, int log_entries, VenusFid *changed_fid)
 
     fid_AD_tail = fid_AD_head;
     cur_log = (log_ent_t *) malloc(sizeof(log_ent_t));
-    fd = open(log_file, O_RDONLY | O_BINARY);
+    fd = open(log_file, O_RDWR | O_BINARY);
 
     set_fid_value(&new_fid , 0);
 
@@ -321,6 +321,9 @@ do_replay(char *log_file, int log_entries, VenusFid *changed_fid)
    	 
   	fid_KP_head = 0;
     }
+    i = ftruncate (fd, 0);
+    assert (i == 0);
+
  terminate:
 
     arla_warnx (ADEBDISCONN,"We have done total %d replays",count-1);
@@ -536,7 +539,7 @@ find_venus (char *name, VenusFid *fid)
  *
  */
 
-static VenusFid *
+VenusFid *
 fid_translate(VenusFid *fid_in)
 {
     fid_trans *fid_tmp;
@@ -793,7 +796,7 @@ int reconnect_remove(struct vcache *vcp, FCacheEntry *childentry, char *name)
     fs_server_context context;
     AFSFetchStatus status;
     AFSVolSync volsync; 
-    char tmp[5];
+    char tmp[2 * sizeof(int) + 2];
     xfs_cache_handle cache_handle;
 
     fid = &(vcp->fid); /* points to the VenusFid structure */
@@ -903,7 +906,7 @@ reconnect_rmdir(struct vcache *vcp, FCacheEntry *childEntry, char *name)
     int error;               
     VenusFid *fid, tempfid; /* Ba san: to check the deletion of file*/
     Result res;
-    char tmp[5];
+    char tmp[2 * sizeof(int) + 2];
     int ret = 0; 
     Result tempres; 
     xfs_cache_handle cache_handle;
@@ -1033,16 +1036,13 @@ void fcache_backfile_name(char *name, size_t len)
 void
 copy_cached_file(int from, int to)
 {
-    char tmpname[5],name_from[21],name_to[27];
+    char name_from[2 * sizeof(int) + 1], name_to[2 * sizeof(int) + 1];
     int fd_from, n, fd_to;
     char buf[BUFSIZE];
 
-    strcpy(name_from,ARLACACHEDIR);
-    strcpy(name_to,ARLACACHEDIR);
-    sprintf(tmpname,"%04X",from);
-    strcat(name_from, tmpname);
-    sprintf(tmpname,"%04X",to);
-    strcat(name_to, tmpname);
+    snprintf (name_from, sizeof(name_from), "%04X", from);
+    snprintf (name_to,   sizeof(name_to),   "%04X", to);
+
     fd_from = open(name_from,O_RDONLY | O_BINARY);  
     fd_to   = open(name_to,  O_WRONLY | O_CREAT | O_BINARY);
 
@@ -1059,6 +1059,17 @@ copy_cached_file(int from, int to)
     close(fd_to);
 }
 
+/*
+ *
+ */
+
+static void
+update_fid (FCacheEntry *entry, VenusFid oldfid)
+{
+    if (entry->flags.kernelp)
+	update_kernelfid (oldfid, entry->fid);
+}
+
 /*
  *
  */
@@ -1069,7 +1080,7 @@ reconnect_mut_newfile(FCacheEntry **fcep, pag_t cred,VenusFid *new_fid)
 
     FCacheEntry *parent_fce;
     u_long host;
-    char name[9],tmp[5];
+    char name[2 * sizeof(int) + 3 + 1], tmp[2 * sizeof(int) + 2];
     AFSStoreStatus store_attr;
     AFSFetchStatus fetch_attr;
     CredCacheEntry *ce; 
@@ -1172,6 +1183,7 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
 
     sizefs=fce->status.Length;
 
+#if 0 /* XXX */
     /* some people have written to the file while we are disconnected */
     /* we have to give it a different name on the server  */
     if (reconnect_mut_chk(fce, ce, vcp->DataVersion))
@@ -1186,6 +1198,7 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
 	tempce->flags.attrp = FALSE;
 	tempce->flags.kernelp = FALSE;
     }   
+#endif
 
 #if 0
     usedbytes -= entry->status.Length;
@@ -1197,56 +1210,59 @@ int reconnect_putattr(struct vcache *vcp, struct xfs_attr *xap)
     /* code from truncate file XXX join */
     conn = find_first_fs (fce, ce, &context);
 
-    call = rx_NewCall (conn->connection);
-    if (call == NULL) {
-	arla_log (ADEBDISCONN, "Cannot call");
-	res.res = ENOMEM;
-	goto out;
+    if (fce->status.FileType != TYPE_DIR) {
+
+	call = rx_NewCall (conn->connection);
+	if (call == NULL) {
+	    arla_log (ADEBDISCONN, "Cannot call");
+	    res.res = ENOMEM;
+	    goto out;
+	}
+
+	storestatus.Mask = 0;
+	res.res = StartRXAFS_StoreData (call,
+					&(fce->fid.fid),
+					&storestatus,
+					0, 0, fce->status.Length);
+	if(res.res) {
+	    arla_log (ADEBDISCONN, "Could not start store, %s (%d)",
+		      koerr_gettext(res.res), res.res);
+	    rx_EndCall(call, 0);
+	    goto out;
+	}
+
+	sizefs = htonl (sizefs);
+	if (rx_Write (call, &sizefs, sizeof(sizefs)) != sizeof(sizefs)) {
+	    res.res = conv_to_arla_errno(rx_Error(call));
+	    arla_log (ADEBDISCONN, "Error writing length: %d", res.res);
+	    rx_EndCall(call, 0);
+	    goto out;
+	}
+
+	if (rx_Write (call, 0, 0) != 0) {
+	    res.res = conv_to_arla_errno(rx_Error(call));
+	    arla_log (ADEBDISCONN, "Error writing: %d", res.res);
+	    rx_EndCall(call, 0);
+	    goto out;
+	}
+
+	res.res = rx_EndCall (call, EndRXAFS_StoreData (call,
+							&status,
+							&callback,
+							&volsync));
+	if (res.res) {
+	    arla_log (ADEBDISCONN, "Error rx_EndCall: %s (%d)",
+		      koerr_gettext(res.res), res.res);
+	    goto out;
+	}
+
+	fce->status   = status;
+	fce->callback = callback;
+	fce->volsync  = volsync;
+
+	volcache_update_volsync (fce->volume, fce->volsync);
+
     }
-
-    storestatus.Mask = 0;
-    res.res = StartRXAFS_StoreData (call,
-				    &(fce->fid.fid),
-				    &storestatus,
-				    0, 0, fce->status.Length);
-    if(res.res) {
-	arla_log (ADEBDISCONN, "Could not start store, %s (%d)",
-                  koerr_gettext(res.res), res.res);
-	rx_EndCall(call, 0);
-	goto out;
-    }
-
-    sizefs = htonl (sizefs);
-    if (rx_Write (call, &sizefs, sizeof(sizefs)) != sizeof(sizefs)) {
-	res.res = rx_Error(call);
-	arla_log (ADEBDISCONN, "Error writing length: %d", res.res);
-	rx_EndCall(call, 0);
-	goto out;
-    }
-
-    if (rx_Write (call, 0, 0) != 0) {
-	res.res = rx_Error(call);
-	arla_log (ADEBDISCONN, "Error writing: %d", res.res);
-	rx_EndCall(call, 0);
-	goto out;
-    }
-
-    res.res = rx_EndCall (call, EndRXAFS_StoreData (call,
-						    &status,
-						    &callback,
-						    &volsync));
-    if (res.res) {
-	arla_log (ADEBDISCONN, "Error rx_EndCall: %s (%d)",
-		  koerr_gettext(res.res), res.res);
-	goto out;
-    }
-
-    fce->status   = status;
-    fce->callback = callback;
-    fce->volsync  = volsync;
-
-    volcache_update_volsync (fce->volume, fce->volsync);
-
     /* code from write_attr XXX join */
     xfs_attr2afsstorestatus(xap, &storestatus);
 
@@ -1386,7 +1402,7 @@ reconnect_putdata(struct vcache *vcp)
 	goto out;
     }
 
-    res.res = copyfd2rx (fd, call, sizefs);
+    res.res = copyfd2rx (fd, call, 0, sizefs);
     if (res.res) {
 	rx_EndCall(call, res.res);
 	arla_log (ADEBDISCONN, "copyfd2rx failed: %d", res.res);
@@ -1438,7 +1454,7 @@ int reconnect_rename(struct vcache *vcp_old, struct vcache *vcp_new,
 
     int ret = 0;
     Result res;
-    char tmp[5];
+    char tmp[2 * sizeof(int) + 2];
     int isnewpar = 0;
     ConnCacheEntry *conn;
     fs_server_context context;
@@ -1628,26 +1644,6 @@ int reconnect_rename(struct vcache *vcp_old, struct vcache *vcp_new,
     return error;
 }
 
-/* XXX */
-
-/*
- *
- */
-
-static void
-recon_hashtabdel (FCacheEntry *e)
-{
-}
-
-/*
- *
- */
-
-static void
-recon_hashtabadd (FCacheEntry *e)
-{
-}
-
 /*
  *
  */
@@ -1673,7 +1669,7 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
     AFSCallBack callback;
     AFSVolSync volsync;
     int ret; 
-    char tmp[5];
+    char tmp[2 * sizeof(int) + 2];
     xfs_cache_handle cache_handle;
     int32_t type;
 
@@ -1759,7 +1755,7 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
 	    fill_fid_trans(&childEntry->fid);
 	    recon_hashtabadd(childEntry);
 	    childEntry->host = rx_HostOf (rx_PeerOf (conn->connection));
-	    update_kernelfid(fakeFid, childEntry->fid); 
+	    update_fid (childEntry, fakeFid);
 	} else {
 	    arla_log (ADEBDISCONN, "Could not CreateFile: %s (%d)",
 		      koerr_gettext(ret), ret);
@@ -1840,7 +1836,7 @@ int reconnect_create(struct vcache *parent, struct vcache *child, char *name)
 		childEntry->fid.fid.Vnode,
 		childEntry->fid.fid.Unique);
 
-    update_kernelfid(fakeFid, childEntry->fid); 
+    update_fid (childEntry, fakeFid);
 
     conv_dir(parentEntry, ce, 0, &cache_handle, tmp, sizeof(tmp));
 
@@ -1875,7 +1871,7 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
     int    ret = 0;
     int    tempret = 0;
     struct timeval tv;
-    char tmp[5];
+    char tmp[2 * sizeof(int) + 2];
 
     AFSFid  Outfid;   /* Ba Wu: These are used to get the info from FS*/
     AFSFetchStatus fetch_attr;
@@ -1972,12 +1968,10 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
 
     tempret = adir_creat (parentEntry, name, childEntry->fid.fid); 
 
-    tempret = adir_mkdir (childEntry, childEntry->fid.fid,parentEntry->fid.fid); 
-
     childEntry->host = rx_HostOf (rx_PeerOf (conn->connection));
     assert(childEntry->host);
 
-    update_kernelfid(fakeFid, childEntry->fid);
+    update_fid(childEntry, fakeFid);
      
     tempres = conv_dir(parentEntry, ce, 0, &cache_handle,
 		       tmp, sizeof(tmp));
@@ -1988,7 +1982,7 @@ int reconnect_mkdir(struct vcache *parent, struct vcache *curdir,
     ret = fcache_find (&tempEntry, childEntry->fid);  
 
     assert (ret == 0);
-    assert (tempEntry == NULL);
+    ReleaseWriteLock(&tempEntry->lock);
 
  out:
 
@@ -2011,7 +2005,7 @@ int reconnect_link(struct vcache *parent, struct vcache *existing,
     CredCacheEntry *ce;
     VenusFid *parent_fid;  
     VenusFid *existing_fid;
-    char tmp[5];
+    char tmp[2 * sizeof(int) + 2];
     int ret = 0;
     FCacheEntry *dir_entry,*existing_entry;
     Result res;
@@ -2084,7 +2078,7 @@ int reconnect_symlink(struct vcache *parent, struct vcache *child,
     fs_server_context context;
     CredCacheEntry *ce;
     VenusFid *parent_fid, *child_fid, fakeFid;
-    char tmp[5];
+    char tmp[2 * sizeof(int) + 2];
     int ret = 0;
     FCacheEntry *dir_entry, *childEntry;
     Result res;
@@ -2158,7 +2152,7 @@ int reconnect_symlink(struct vcache *parent, struct vcache *child,
     childEntry->host = rx_HostOf (rx_PeerOf (conn->connection));
     assert(childEntry->host);
 
-    update_kernelfid(fakeFid, childEntry->fid);
+    update_fid(childEntry, fakeFid);
     res = conv_dir (dir_entry, ce, 0, &cache_handle, tmp, sizeof(tmp));
 
  out: 
