@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -37,7 +37,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: solaris-subr.c,v 1.14 1998/11/10 03:35:18 rb Exp $");
+RCSID("$Id: solaris-subr.c,v 1.17 1999/07/24 19:48:32 lha Exp $");
 
 static long blocksize = DIRBUF;
 
@@ -103,6 +103,8 @@ conv_dir (FCacheEntry *e, CredCacheEntry *ce, u_int tokens,
 {
      struct args args;
      Result res;
+     fbuf the_fbuf;
+     int ret;
 
      e->flags.extradirp = TRUE;
      fcache_extra_file_name (e, cache_name, cache_name_sz);
@@ -127,11 +129,22 @@ conv_dir (FCacheEntry *e, CredCacheEntry *ce, u_int tokens,
 	 close (args.fd);
 	 return res;
      }
+     ret = fcache_get_fbuf (e, &the_fbuf, O_RDONLY, FBUF_READ);
+     if (ret) {
+	 res.res = -1;
+	 res.error = ret;
+	 close (args.fd);
+	 free (args.buf);
+	 return res;
+     }
+     
      args.ptr  = args.buf;
      args.last = NULL;
-     ReleaseWriteLock (&e->lock);
-     adir_readdir (e->fid, write_dirent, (void *)&args, ce);
-     ObtainWriteLock (&e->lock);
+     
+     fdir_readdir (&the_fbuf, write_dirent, (void *)&args, &e->fid);
+
+     fbuf_end (&the_fbuf);
+
      if (args.last)
 	  flushbuf (&args);
      free (args.buf);
@@ -139,3 +152,54 @@ conv_dir (FCacheEntry *e, CredCacheEntry *ce, u_int tokens,
 	  res.error = errno;
      return res;
 }
+
+/*
+ * remove `filename` from the converted directory for `e'
+ */
+
+int
+dir_remove_name (FCacheEntry *e, const char *filename,
+		 xfs_cache_handle *cache_handle,
+		 char *cache_name, size_t cache_name_sz)
+{
+    int ret;
+    int fd;
+    fbuf fb;
+    struct stat sb;
+    char *buf;
+    char *p;
+    size_t len;
+    struct dirent64 *dp;
+    struct dirent64 *last_dp;
+
+    fcache_extra_file_name (e, cache_name, cache_name_sz);
+    fd = open (cache_name, O_RDWR, 0);
+    if (fd < 0)
+	return errno;
+    fcache_fhget (cache_name, cache_handle);
+    if (fstat (fd, &sb) < 0) {
+	ret = errno;
+	close (fd);
+	return ret;
+    }
+    len = sb.st_size;
+
+    ret = fbuf_create (&fb, fd, len, FBUF_WRITE);
+    if (ret)
+	return ret;
+    last_dp = NULL;
+    for (p = buf = fbuf_buf (&fb); p < buf + len; p += dp->d_reclen) {
+	dp = (struct dirent64 *)p;
+
+	if (strcmp (filename, dp->d_name) == 0) {
+	    if (last_dp != NULL)
+		last_dp->d_reclen += dp->d_reclen;
+	    dp->d_ino = 0;
+	    fbuf_end (&fb);
+	    return 0;
+	}
+	last_dp = dp;
+    }
+    fbuf_end (&fb);
+    return ENOENT;
+}
