@@ -41,7 +41,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: volcache.c,v 1.65 1999/03/19 08:31:03 lha Exp $") ;
+RCSID("$Id: volcache.c,v 1.73 1999/07/28 00:51:10 assar Exp $") ;
 
 /*
  * Suffixes for volume names.
@@ -72,11 +72,13 @@ volcache_get_rootvolume (void)
 void
 volcache_set_rootvolume (const char *volname)
 {
+    assert (volname != NULL);
+
     root_volume_name = volname;
 }
 
-#define VOLCACHE_SIZE 2048
-#define VOLCACHE_INC 300
+#define VOLCACHE_SIZE	2053
+#define VOLCACHE_INC	300
 
 /* Hashtable of entries by name */
 static Hashtab *volnamehashtab;
@@ -136,6 +138,54 @@ volidhash (void *a)
     return n->cell + n->vol;
 }
 
+/*
+ * Compare two `nvldbentry' and return 0 if they are equal.
+ */
+
+static int
+cmp_nvldbentry (const nvldbentry *n1, const nvldbentry *n2)
+{
+    int i;
+
+    if (strcmp (n1->name, n2->name) != 0)
+	return 1;
+    if (n1->nServers != n2->nServers)
+	return 1;
+    for (i = 0; i < n1->nServers; ++i)
+	if (n1->serverNumber[i] != n2->serverNumber[i]
+	    || n1->serverPartition[i] != n2->serverPartition[i]
+	    || n1->serverFlags[i] != n2->serverFlags[i])
+	    return 1;
+    if (n1->flags != n2->flags)
+	return 1;
+    if (n1->flags & VLF_RWEXISTS
+	&& n1->volumeId[RWVOL] != n2->volumeId[RWVOL])
+	return 1;
+    if (n1->flags & VLF_ROEXISTS
+	&& n1->volumeId[ROVOL] != n2->volumeId[ROVOL])
+	return 1;
+    if (n1->flags & VLF_BOEXISTS
+	&& n1->volumeId[BACKVOL] != n2->volumeId[BACKVOL])
+	return 1;
+    if (n1->cloneId != n2->cloneId)
+	return 1;
+    return 0;
+}
+
+/*
+ * Do consistency checks and simple clean-ups.
+ */
+
+static void
+sanitize_nvldbentry (nvldbentry *n)
+{
+    if (n->nServers > NMAXNSERVERS) {
+	arla_warnx (ADEBVOLCACHE, "too many servers %d > %d",
+		    n->nServers, NMAXNSERVERS);
+	n->nServers = NMAXNSERVERS;
+    }
+}
+
 /*
  * Create `n' entries and add at the end of `lrulist'
  */
@@ -190,7 +240,8 @@ recycle_entry (VolCacheEntry *e)
     }
     memset (&e->entry, 0, sizeof(e->entry));
     memset (&e->volsync, 0, sizeof(e->volsync));
-    e->flags.validp = FALSE;
+    e->flags.validp  = FALSE;
+    e->flags.stablep = FALSE;
     memset (&e->name_ptr, 0, sizeof(e->name_ptr));
     memset (&e->num_ptr, 0, sizeof(e->num_ptr));
 }
@@ -212,7 +263,7 @@ get_free_entry (void)
 	item;
 	item = listprev (lrulist, item)) {
 	e = (VolCacheEntry *)listdata(item);
-	if (e->refcount == 0) {
+	if (e->refcount == 0 /* || e->refcount == 1 */) {
 	    listdel (lrulist, item);
 	    recycle_entry (e);
 	    listaddhead (lrulist, e);
@@ -280,7 +331,8 @@ volcache_recover_state (void)
 		hashtabadd (volidhashtab, (void *)&e->num_ptr[i]);
 	    }
 	}
-	e->flags.validp = FALSE;
+	e->flags.validp  = FALSE;
+	e->flags.stablep = FALSE;
     }
     close (fd);
     arla_warnx (ADEBVOLCACHE, "recovered %u entries to volcache", n);
@@ -443,6 +495,11 @@ get_info_loop (VolCacheEntry *e, const char *name, CredCacheEntry *ce)
     int i, j;
     Bool try_again;
 
+    if (dynroot_isvolumep (cell, name)) {
+	dynroot_fetch_vldbN (&e->entry);
+	return 0;
+    }
+
     db_servers = cell_dbservers (cell, &num_db_servers);
     if (db_servers == NULL || num_db_servers == 0) {
 	arla_warnx (ADEBWARN,
@@ -490,7 +547,9 @@ get_info_loop (VolCacheEntry *e, const char *name, CredCacheEntry *ce)
 		    conn_dead (conns[i]);
 		switch (error) {
 		case 0 :
+		    sanitize_nvldbentry (&e->entry);
 		    try_again = FALSE;
+		    e->last_fetch = time(NULL);
 		    break;
 		case VL_NOENT :
 		    error = ENOENT;
@@ -527,7 +586,9 @@ get_info_loop (VolCacheEntry *e, const char *name, CredCacheEntry *ce)
 }
 
 /*
- *
+ * Retrieve the information for the volume `id' into `e' using `ce' as
+ * the creds.
+ * Return 0 or error.
  */
 
 static int
@@ -545,7 +606,8 @@ get_info_byid (VolCacheEntry *e, u_int32_t id, CredCacheEntry *ce)
 
 
 /*
- *
+ * Retrieve the information for `volname' into `e' using `ce' as the creds.
+ * Return 0 or error.
  */
 
 static int
@@ -677,11 +739,14 @@ volcache_getbyname (const char *volname, int32_t cell, CredCacheEntry *ce,
 	if (volume_uptodatep (*e)) {
 	    return 0;
 	} else {
+	    nvldbentry old_entry = (*e)->entry;
+
 	    recycle_entry (*e);
 	    error = get_info_byname (*e, volname, ce);
 	    --(*e)->refcount;
 	    if (error)
 		return error;
+	    (*e)->flags.stablep = cmp_nvldbentry (&old_entry, &(*e)->entry);
 	}
     }
 }
@@ -716,11 +781,14 @@ volcache_getbyid (u_int32_t volid, int32_t cell, CredCacheEntry *ce,
 	if (volume_uptodatep (*e)) {
 	    return 0;
 	} else {
+	    nvldbentry old_entry = (*e)->entry;
+
 	    recycle_entry (*e);
 	    error = get_info_byid (*e, volid, ce);
 	    --(*e)->refcount;
 	    if (error)
 		return error;
+	    (*e)->flags.stablep = cmp_nvldbentry (&old_entry, &(*e)->entry);
 	}
     }
 }
@@ -742,10 +810,34 @@ volcache_invalidate (u_int32_t id, int32_t cell)
     if (n != NULL) {
 	VolCacheEntry *e = n->ptr;
 
-	e->flags.validp = FALSE;
+	e->flags.validp  = FALSE;
+	e->flags.stablep = FALSE;
     }
 }
 
+/*
+ * Return TRUE if this should be considered reliable (if it's validp,
+ * stablep and fresh).
+ */
+
+Bool
+volcache_reliable (u_int32_t id, int32_t cell)
+{
+    VolCacheEntry *e;
+    struct num_ptr *n;
+    struct num_ptr key;
+
+    key.cell = cell;
+    key.vol  = id;
+
+    n = (struct num_ptr *)hashtabsearch (volidhashtab, (void *)&key);
+    if (n == NULL)
+	return FALSE;
+    e = n->ptr;
+    return e->flags.validp && e->flags.stablep
+	&& (time(NULL) - e->last_fetch) < VOLCACHE_OLD;
+}
+
 /*
  * Save `volsync'
  */
@@ -778,32 +870,31 @@ print_entry (void *ptr, void *arg)
 {
     struct num_ptr *n = (struct num_ptr *)ptr;
     VolCacheEntry *e = n->ptr;
-    FILE *f = (FILE *)arg;
     int i;
     struct in_addr tmp;
 
     if (n->vol != e->entry.volumeId[RWVOL])
 	return FALSE;
 
-    fprintf (f, "cell = %d (%s)\n"
+    arla_log(ADEBVLOG, "cell = %d (%s)\n"
 	     "name = \"%s\", nServers = %d\n",
 	     e->cell, cell_num2name (e->cell),
 	     e->entry.name,
 	     e->entry.nServers);
     for (i = 0; i < e->entry.nServers; ++i) {
 	tmp.s_addr = htonl(e->entry.serverNumber[i]);
-	fprintf (f, "%d: server = %s, part = %d(%c), flags = %d\n",
+	arla_log(ADEBVLOG, "%d: server = %s, part = %d(%c), flags = %d\n",
 		 i, inet_ntoa(tmp), e->entry.serverPartition[i],
 		 'a' + e->entry.serverPartition[i],
 		 e->entry.serverFlags[i]);
     }
     if (e->entry.flags & VLF_RWEXISTS)
-	fprintf (f, "rw clone: %d\n", e->entry.volumeId[RWVOL]);
+	arla_log(ADEBVLOG, "rw clone: %d\n", e->entry.volumeId[RWVOL]);
     if (e->entry.flags & VLF_ROEXISTS)
-	fprintf (f, "ro clone: %d\n", e->entry.volumeId[ROVOL]);
+	arla_log(ADEBVLOG, "ro clone: %d\n", e->entry.volumeId[ROVOL]);
     if (e->entry.flags & VLF_BACKEXISTS)
-	fprintf (f, "rw clone: %d\n", e->entry.volumeId[BACKVOL]);
-    fprintf (f, "refcount = %u\n\n", e->refcount);
+	arla_log(ADEBVLOG, "rw clone: %d\n", e->entry.volumeId[BACKVOL]);
+    arla_log(ADEBVLOG, "refcount = %u\n\n", e->refcount);
     return FALSE;
 }
 
@@ -826,9 +917,9 @@ volume_make_uptodate (VolCacheEntry *e, CredCacheEntry *ce)
  */
 
 void
-volcache_status (FILE *f)
+volcache_status (void)
 {
-    fprintf (f, "%u(%u) volume entries\n",
+    arla_log(ADEBVLOG, "%u(%u) volume entries\n",
 	     nactive_volcacheentries, nvolcacheentries);
-    hashtabforeach (volidhashtab, print_entry, f);
+    hashtabforeach (volidhashtab, print_entry, NULL);
 }
