@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -40,7 +40,7 @@
  * Our cache of volume information.
  */
 
-/* $Id: volcache.h,v 1.16 1999/03/19 08:31:28 lha Exp $ */
+/* $Id: volcache.h,v 1.18 1999/07/28 00:34:09 assar Exp $ */
 
 #ifndef _VOLCACHE_
 #define _VOLCACHE_
@@ -51,6 +51,10 @@
 #define BACKSUFFIX ".backup"
 #define ROSUFFIX   ".readonly"
 
+/*
+ * index for number into a VolCacheEntry
+ */
+
 struct num_ptr {
     int32_t cell;
     u_int32_t vol;
@@ -58,6 +62,10 @@ struct num_ptr {
     int32_t type;
 };
 
+/*
+ * index for name into a VolCacheEntry
+ */
+
 struct name_ptr {
     int32_t cell;
     char name[VLDB_MAXNAMELEN];
@@ -70,8 +78,10 @@ struct volcacheentry {
     AFSVolSync volsync;
     int32_t cell;
     unsigned refcount;
+    time_t last_fetch;
     struct {
-	unsigned validp : 1;
+	unsigned validp  : 1;
+	unsigned stablep : 1;
     } flags;
     struct name_ptr name_ptr[MAXTYPES];
     struct num_ptr num_ptr[MAXTYPES];
@@ -105,10 +115,13 @@ void volcache_invalidate (u_int32_t id, int32_t cell);
 
 int volume_make_uptodate (VolCacheEntry *e, CredCacheEntry *ce);
 
-void volcache_status (FILE *f);
+Bool volcache_reliable (u_int32_t id, int32_t cell);
+
+void volcache_status (void);
 
 int
 volcache_store_state (void);
 
-#endif /* _VOLCACHE_ */
+enum { VOLCACHE_OLD = 120 };
 
+#endif /* _VOLCACHE_ */
