@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: kafs.h,v 1.21 1999/01/04 23:07:49 assar Exp $ */
+/* $Id: kafs.h,v 1.24 1999/08/04 13:44:48 assar Exp $ */
 
 #ifndef __KAFS_H
 #define __KAFS_H
@@ -54,6 +54,22 @@
 #define _VICEIOCTL(id)  ((unsigned int ) _IOW('V', id, struct ViceIoctl))
 #endif /* _VICEIOCTL */
 
+/* 
+ * this is for operating systems that support a 32-bit API while being
+ * 64-bit (such as solaris 7 in 64 bit mode).  The ioctls will get
+ * assigned different numbers in 32bits mode and we need to support
+ * both.
+ */
+
+#ifdef NEED_VICEIOCTL32
+
+#ifndef _VICEIOCTL32
+#define _VICEIOCTL32(id)  ((unsigned int ) _IOW('V', id, struct ViceIoctl32))
+#endif /* _VICEIOCTL32 */
+
+#endif /* NEED_VICEIOCTL32 */
+
+
 /*
  * ioctls
  */
@@ -62,6 +78,10 @@
 #define VIOCABORT		_VICEIOCTL(2)
 #define VIOIGETCELL		_VICEIOCTL(3)
 
+#define VIOCCLOSEWAIT_32	_VICEIOCTL32(1)
+#define VIOCABORT_32		_VICEIOCTL32(2)
+#define VIOIGETCELL_32		_VICEIOCTL32(3)
+
 /*
  * pioctls
  */
@@ -129,6 +149,79 @@
 
 #define VIOC_XFSDEBUG_PRINT	_VICEIOCTL(57)
 
+#define VIOC_CALCULATE_CACHE	_VICEIOCTL(58)
+
+#ifdef NEED_VICEIOCTL32
+
+/* and now for the 32-bit versions */
+
+#define VIOCSETAL_32			_VICEIOCTL32(1)
+#define VIOCGETAL_32			_VICEIOCTL32(2)
+#define VIOCSETTOK_32			_VICEIOCTL32(3)
+#define VIOCGETVOLSTAT_32		_VICEIOCTL32(4)
+#define VIOCSETVOLSTAT_32		_VICEIOCTL32(5)
+#define VIOCFLUSH_32			_VICEIOCTL32(6)
+#define VIOCSTAT_32			_VICEIOCTL32(7)
+#define VIOCGETTOK_32			_VICEIOCTL32(8)
+#define VIOCUNLOG_32			_VICEIOCTL32(9)
+#define VIOCCKSERV_32			_VICEIOCTL32(10)
+#define VIOCCKBACK_32			_VICEIOCTL32(11)
+#define VIOCCKCONN_32			_VICEIOCTL32(12)
+#define VIOCGETTIME_32			_VICEIOCTL32(13)
+#define VIOCWHEREIS_32			_VICEIOCTL32(14)
+#define VIOCPREFETCH_32			_VICEIOCTL32(15)
+#define VIOCNOP_32			_VICEIOCTL32(16)
+#define VIOCENGROUP_32			_VICEIOCTL32(17)
+#define VIOCDISGROUP_32			_VICEIOCTL32(18)
+#define VIOCLISTGROUPS_32		_VICEIOCTL32(19)
+#define VIOCACCESS_32			_VICEIOCTL32(20)
+#define VIOCUNPAG_32			_VICEIOCTL32(21)
+#define VIOCGETFID_32			_VICEIOCTL32(22)
+#define VIOCWAITFOREVER_32		_VICEIOCTL32(23)
+#define VIOCSETCACHESIZE_32		_VICEIOCTL32(24)
+#define VIOCFLUSHCB_32			_VICEIOCTL32(25)
+#define VIOCNEWCELL_32			_VICEIOCTL32(26)
+#define VIOCGETCELL_32			_VICEIOCTL32(27)
+#define VIOC_AFS_DELETE_MT_PT_32	_VICEIOCTL32(28)
+#define VIOC_AFS_STAT_MT_PT_32		_VICEIOCTL32(29)
+#define VIOC_FILE_CELL_NAME_32		_VICEIOCTL32(30)
+#define VIOC_GET_WS_CELL_32		_VICEIOCTL32(31)
+#define VIOC_AFS_MARINER_HOST_32	_VICEIOCTL32(32)
+#define VIOC_GET_PRIMARY_CELL_32	_VICEIOCTL32(33)
+#define VIOC_VENUSLOG_32		_VICEIOCTL32(34)
+#define VIOC_GETCELLSTATUS_32		_VICEIOCTL32(35)
+#define VIOC_SETCELLSTATUS_32		_VICEIOCTL32(36)
+#define VIOC_FLUSHVOLUME_32		_VICEIOCTL32(37)
+#define VIOC_AFS_SYSNAME_32		_VICEIOCTL32(38)
+#define VIOC_EXPORTAFS_32		_VICEIOCTL32(39)
+#define VIOCGETCACHEPARAMS_32		_VICEIOCTL32(40)
+#define VIOCCONNECTMODE_32	        _VICEIOCTL32(41)
+#define VIOCGETVCXSTATUS_32		_VICEIOCTL32(41)
+#define VIOC_SETSPREFS_3233		_VICEIOCTL32(42)
+#define VIOC_GETSPREFS_32		_VICEIOCTL32(43)
+#define VIOC_GAG_32			_VICEIOCTL32(44)
+#define VIOC_TWIDDLE_32			_VICEIOCTL32(45)
+#define VIOC_SETSPREFS_32		_VICEIOCTL32(46)
+#define VIOC_STORBEHIND_32		_VICEIOCTL32(47)
+#define VIOC_GCPAGS_32			_VICEIOCTL32(48)
+#define VIOC_GETRXKCRYPT_32		_VICEIOCTL32(49) /* 48 in some implementations */
+#define VIOC_SETRXKCRYPT_32		_VICEIOCTL32(50) /* with cryptosupport in afs */
+#define VIOC_FPRIOSTATUS_32		_VICEIOCTL32(51)
+
+#define VIOC_FHGET_32			_VICEIOCTL32(52)
+#define VIOC_FHOPEN_32			_VICEIOCTL32(53)
+
+#define VIOC_XFSDEBUG_32		_VICEIOCTL32(54)
+#define VIOC_ARLADEBUG_32		_VICEIOCTL32(55)
+
+#define VIOC_AVIATOR_32			_VICEIOCTL32(56)
+
+#define VIOC_XFSDEBUG_PRINT_32		_VICEIOCTL32(57)
+
+#define VIOC_CALCULATE_CACHE_32		_VICEIOCTL32(58)
+
+#endif /* NEED_VICEIOCTL32 */
+
 /*
  * GETCELLSTATUS flags
  */
@@ -187,6 +280,12 @@ struct ViceIoctl {
   short in_size;
   short out_size;
 };
+
+struct ViceIoctl32 {
+  u_int32_t in, out;		/* really caddr_t in 32 bits */
+  short in_size;
+  short out_size;
+};
 #endif
 
 struct ClearToken {
