@@ -1,4 +1,4 @@
-/* $Id: rx.h,v 1.10 1998/10/20 02:49:53 art Exp $ */
+/* $Id: rx.h,v 1.11 1999/05/15 16:06:56 assar Exp $ */
 
 /*
 ****************************************************************************
@@ -768,7 +768,7 @@ struct rx_call {
  */
 
 struct rx_ackPacket {
-    u_short bufferSpace;	       /*
+    u_int16_t bufferSpace;	       /*
 				        * Number of packet buffers available.
 					* That is: the number of buffers that
 					* the sender of the ack packet is
@@ -776,20 +776,20 @@ struct rx_ackPacket {
 				        * on this or subsequent calls. Lying is
 				        * permissable.
 				        */
-    u_short maxSkew;		       /*
+    u_int16_t maxSkew;		       /*
 				        * Maximum difference between serial# of
 				        * packet acknowledged and highest
 					* packet yet received
 				        */
-    u_long firstPacket;		       /*
+    u_int32_t firstPacket;	       /*
 				        * The first packet in the list of
 				        * acknowledged packets
 				        */
-    u_long previousPacket;	       /*
+    u_int32_t previousPacket;	       /*
 				        * The previous packet number received
 					* (obsolete?)
 				        */
-    u_long serial;		       /*
+    u_int32_t serial;		       /*
 				        * Serial number of the packet which
 				        * prompted the acknowledge
 				        */
@@ -897,64 +897,64 @@ struct rx_ackPacket {
  */
 
 struct rx_stats {		       /* General rx statistics */
-    int packetRequests;		       /* Number of packet allocation
+    u_int32_t packetRequests;		       /* Number of packet allocation
 				        * requests */
-    int noPackets[RX_N_PACKET_CLASSES];/*
+    u_int32_t noPackets[RX_N_PACKET_CLASSES];/*
 				        * Number of failed packet requests,
 				        * per allocation class
 				        */
-    int socketGreedy;		       /* Whether SO_GREEDY succeeded */
-    int bogusPacketOnRead;	       /*
+    u_int32_t socketGreedy;		       /* Whether SO_GREEDY succeeded */
+    u_int32_t bogusPacketOnRead;	       /*
 				        * Number of inappropriately short
 					* packets  received
 				        */
-    int bogusHost;		       /* Host address from bogus packets */
-    int noPacketOnRead;		       /*
+    u_int32_t bogusHost;		       /* Host address from bogus packets */
+    u_int32_t noPacketOnRead;		       /*
 				        * Number of read packets attempted
 					* when there was actually no packet
 				        * to read off the wire
 				        */
-    int noPacketBuffersOnRead;	       /*
+    u_int32_t noPacketBuffersOnRead;	       /*
 					* Number of dropped data packets due
 					* to lack of packet buffers
 					*/
-    int selects;		       /*
+    u_int32_t selects;		       /*
 				        * Number of selects waiting for packet
 					* or timeout
 					*/
-    int sendSelects;		       /*
+    u_int32_t sendSelects;		       /*
 					* Number of selects forced when
 					* sending packet
 					*/
-    int packetsRead[RX_N_PACKET_TYPES];/*
+    u_int32_t packetsRead[RX_N_PACKET_TYPES];/*
 				        * Total number of packets read, per
 				        * type
 				        */
-    int dataPacketsRead;	       /*
+    u_int32_t dataPacketsRead;	       /*
 				        * Number of unique data packets read
 					* off the wire
 				        */
-    int ackPacketsRead;		       /* Number of ack packets read */
-    int dupPacketsRead;		       /* Number of duplicate data packets
+    u_int32_t ackPacketsRead;		       /* Number of ack packets read */
+    u_int32_t dupPacketsRead;		       /* Number of duplicate data packets
 				        * read */
-    int spuriousPacketsRead;	       /* Number of inappropriate data
+    u_int32_t spuriousPacketsRead;	       /* Number of inappropriate data
 				        * packets */
-    int packetsSent[RX_N_PACKET_TYPES];/*
+    u_int32_t packetsSent[RX_N_PACKET_TYPES];/*
 				        * Number of rxi_Sends: packets sent
 				        * over the wire, per type
 				        */
-    int ackPacketsSent;		       /* Number of acks sent */
-    int pingPacketsSent;	       /* Total number of ping packets sent */
-    int abortPacketsSent;	       /* Total number of aborts */
-    int busyPacketsSent;	       /* Total number of busies sent
+    u_int32_t ackPacketsSent;		       /* Number of acks sent */
+    u_int32_t pingPacketsSent;	       /* Total number of ping packets sent */
+    u_int32_t abortPacketsSent;	       /* Total number of aborts */
+    u_int32_t busyPacketsSent;	       /* Total number of busies sent
 				        * received */
-    int dataPacketsSent;	       /* Number of unique data packets sent */
-    int dataPacketsReSent;	       /* Number of retransmissions */
-    int dataPacketsPushed;	       /*
+    u_int32_t dataPacketsSent;	       /* Number of unique data packets sent */
+    u_int32_t dataPacketsReSent;	       /* Number of retransmissions */
+    u_int32_t dataPacketsPushed;	       /*
 				        * Number of retransmissions pushed early by
 				        * a NACK
 				        */
-    int ignoreAckedPacket;	       /*
+    u_int32_t ignoreAckedPacket;	       /*
 				        * Number of packets with acked flag,
 					* on rxi_Start
 				        */
@@ -964,27 +964,27 @@ struct rx_stats {		       /* General rx statistics */
 				        */
     struct clock minRtt;	       /* Minimum round trip time measured */
     struct clock maxRtt;	       /* Maximum round trip time measured */
-    int nRttSamples;		       /* Total number of round trip samples */
-    int nServerConns;		       /* Total number of server connections */
-    int nClientConns;		       /* Total number of client connections */
-    int nPeerStructs;		       /* Total number of peer structures */
-    int nCallStructs;		       /* Total number of call structures
+    u_int32_t nRttSamples;		       /* Total number of round trip samples */
+    u_int32_t nServerConns;		       /* Total number of server connections */
+    u_int32_t nClientConns;		       /* Total number of client connections */
+    u_int32_t nPeerStructs;		       /* Total number of peer structures */
+    u_int32_t nCallStructs;		       /* Total number of call structures
 				        * allocated */
-    int nFreeCallStructs;	       /*
+    u_int32_t nFreeCallStructs;	       /*
 				        * Total number of previously allocated
 					* free call structures
 				        */
-    int netSendFailures;
-    long fatalErrors;
-    int spares[8];
+    u_int32_t netSendFailures;
+    u_int32_t fatalErrors;
+    u_int32_t spares[8];
 };
 
 /* structures for debug input and output packets */
 
 /* debug input types */
 struct rx_debugIn {
-    long type;
-    long index;
+    u_int32_t type;
+    u_int32_t index;
 };
 
 /* Invalid rx debug package type */
@@ -1007,24 +1007,24 @@ struct rx_debugIn {
 #define	RX_DEBUGI_RXSTATS	4      /* get all rx stats */
 
 struct rx_debugStats {
-    long nFreePackets;
-    long packetReclaims;
-    long callsExecuted;
-    char waitingForPackets;
-    char usedFDs;
-    char version;
-    char spare1;
-    long nWaiting;
-    long spare2[9];
+    u_int32_t nFreePackets;
+    u_int32_t packetReclaims;
+    u_int32_t callsExecuted;
+    u_int8_t waitingForPackets;
+    u_int8_t usedFDs;
+    u_int8_t version;
+    u_int8_t spare1;
+    u_int32_t nWaiting;
+    u_int32_t spare2[9];
 };
 
 struct rx_debugConn_vL {
-    long host;
-    long cid;
-    long serial;
-    long callNumber[RX_MAXCALLS];
-    long error;
-    short port;
+    u_int32_t host;
+    u_int32_t cid;
+    u_int32_t serial;
+    u_int32_t callNumber[RX_MAXCALLS];
+    u_int32_t error;
+    u_int16_t port;
     char flags;
     char type;
     char securityIndex;
@@ -1034,16 +1034,16 @@ struct rx_debugConn_vL {
     char callOther[RX_MAXCALLS];
     /* old style getconn stops here */
     struct rx_securityObjectStats secStats;
-    long sparel[10];
+    u_int32_t sparel[10];
 };
 
 struct rx_debugConn {
-    long host;
-    long cid;
-    long serial;
-    long callNumber[RX_MAXCALLS];
-    long error;
-    short port;
+    u_int32_t host;
+    u_int32_t cid;
+    u_int32_t serial;
+    u_int32_t callNumber[RX_MAXCALLS];
+    u_int32_t error;
+    u_int16_t port;
     char flags;
     char type;
     char securityIndex;
@@ -1054,9 +1054,9 @@ struct rx_debugConn {
     char callOther[RX_MAXCALLS];
     /* old style getconn stops here */
     struct rx_securityObjectStats secStats;
-    long epoch;
-    long maxPacketSize;
-    long sparel[9];
+    u_int32_t epoch;
+    u_int32_t maxPacketSize;
+    u_int32_t sparel[9];
 };
 
 #define	RX_OTHER_IN	1	       /* packets avail in in queue */
