@@ -38,7 +38,25 @@
 
 #include "rxkad_locl.h"
 
-RCSID("$Id: rxk_crpt.c,v 1.8 1998/07/12 14:38:58 joda Exp $");
+#ifdef HAVE_MACHINE_ENDIAN_H
+
+#undef WORDS_BIGENDIAN
+
+#include <machine/endian.h>
+
+#if BYTE_ORDER == BIG_ENDIAN
+#define WORDS_BIGENDIAN 1
+#elif BYTE_ORDER == LITTLE_ENDIAN
+/* nothing */
+#elif BYTE_ORDER == PDP_ENDIAN
+#error sorry, PDP endian not supported (yet)
+#else
+#error sorry, unknown endianess in <machine/endian.h>
+#endif
+ 
+#endif /* HAVE_MACHINE_ENDIAN_H */
+
+RCSID("$Id: rxk_crpt.c,v 1.9 1999/07/20 13:59:32 art Exp $");
 
 /*
  * Unrolling of the inner loops helps the most on pentium chips
@@ -76,10 +94,10 @@ RCSID("$Id: rxk_crpt.c,v 1.8 1998/07/12 14:38:58 joda Exp $");
 #define ROT32L(x, n) ((((u_int32) x) << (n)) | (((u_int32) x) >> (32-(n))))
 #define bswap32(x) (((ROT32L(x, 16) & 0x00ff00ff)<<8) | ((ROT32L(x, 16)>>8) & 0x00ff00ff))
 
-#if defined(__alpha) || defined(i386) || defined(MIPSEL)
-#define NTOH(x) bswap32(x)
-#else
+#if WORDS_BIGENDIAN
 #define NTOH(x) (x)
+#else
+#define NTOH(x) bswap32(x)
 #endif
 
 /*
@@ -93,9 +111,8 @@ RCSID("$Id: rxk_crpt.c,v 1.8 1998/07/12 14:38:58 joda Exp $");
  * all else	ntohl
  */
 
-#if defined(__GNUC__) && defined(i386)
-static inline
-u_int32
+#if defined(__GNUC__) && (defined(i386) || defined(__i386__))
+static inline u_int32
 bswap(u_int32 x)
 {
   asm("bswap %0" : "=r" (x) : "0" (x));
@@ -229,7 +246,7 @@ static const u_int32 sbox3[256] = {
  u.l = sched ^ R; \
  L ^= sbox0[u.c[0]] ^ sbox1[u.c[1]] ^ sbox2[u.c[2]] ^ sbox3[u.c[3]]; }
 
-#if defined(i386) || defined(__alpha)
+#ifndef WORDS_BIGENDIAN
 /* BEWARE: this code is endian dependent.
  * This should really be inline assembler on the x86.
  */
