@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_locl.h,v 1.21 1999/01/19 19:55:21 art Exp $ */
+/* $Id: xfs_locl.h,v 1.27 1999/08/01 10:36:21 lha Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -108,9 +108,15 @@ typedef struct nameidata xfs_componentname;
 #define xfs_uio_to_proc(uiop) (u.u_procp)
 #define xfs_cnp_to_proc(cnp) (u.u_procp)
 #define xfs_proc_to_cred(p) ((p)->p_rcred)
+#define xfs_proc_to_ruid(p) ((p)->p_ruid)
 
 #define xfs_curproc() (u.u_procp)
 
+#define xfs_vop_read VOP_READ
+#define xfs_vop_write VOP_WRITE
+#define xfs_vop_getattr(t, attr, cred, proc, error) VOP_GETATTR((t), (attr), (cred), (error))
+#define xfs_vop_access(dvp, mode, cred, proc, error) VOP_ACCESS((dvp), (mode), (cred), (error))
+
 #else /* __osf__ */
 
 typedef struct componentname xfs_componentname;
@@ -134,7 +140,9 @@ typedef struct componentname xfs_componentname;
 #ifdef HAVE_SYS_SYSENT_H
 #include <sys/sysent.h>
 #endif
+#ifdef HAVE_SYS_LKM_H
 #include <sys/lkm.h>
+#endif
 #include <sys/errno.h>
 #include <sys/file.h>
 #include <sys/namei.h>
@@ -164,11 +172,40 @@ typedef struct componentname xfs_componentname;
 #define xfs_uio_to_proc(uiop) ((uiop)->uio_procp)
 #define xfs_cnp_to_proc(cnp) ((cnp)->cn_proc)
 #define xfs_proc_to_cred(p) ((p)->p_ucred)
+#define xfs_proc_to_ruid(p) ((p)->p_cred->p_ruid)
 
 #define xfs_curproc() (curproc)
 
+#define xfs_vop_read(t, uio, ioflag, cred, error) (error) = VOP_READ((t), (uio), (ioflag), (cred))
+#define xfs_vop_write(t, uio, ioflag, cred, error) (error) = VOP_WRITE((t), (uio), (ioflag), (cred))
+#define xfs_vop_getattr(t, attr, cred, proc, error) (error) = VOP_GETATTR((t), (attr), (cred), (proc))
+#define xfs_vop_access(dvp, mode, cred, proc, error) (error) = VOP_ACCESS((dvp), (mode), (cred), (proc))
+
 #endif /* __osf__ */
 
+#if defined(HAVE_DEF_STRUCT_SETGROUPS_ARGS)
+#define xfs_setgroups_args setgroups_args
+#elif defined(HAVE_DEF_STRUCT_SYS_SETGROUPS_ARGS)
+#define xfs_setgroups_args sys_setgroups_args
+#else
+#error what is you setgroups named ?
+#endif
+
+
+#ifdef HAVE_KERNEL_VFS_GETVFS
+#define xfs_vfs_getvfs vfs_getvfs
+#else
+#define xfs_vfs_getvfs getvfs
+#endif
+
+#ifdef HAVE_FOUR_ARGUMENT_VFS_OBJECT_CREATE
+#define xfs_vfs_object_create(vp,proc,ucred) vfs_object_create(vp,proc,ucred,TRUE)
+#else
+#define xfs_vfs_object_create(vp,proc,ucred) vfs_object_create(vp,proc,ucred)
+#endif
+
+#include <xfs/xfs_syscalls.h>
+
 /* 
  *  The VOP table
  *
