@@ -44,8 +44,9 @@
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_vfsops.h>
 #include <xfs/xfs_vnodeops.h>
+#include <xfs/xfs_dev.h>
 
-RCSID("$Id: xfs_message.c,v 1.35 1999/03/20 01:18:48 lha Exp $");
+RCSID("$Id: xfs_message.c,v 1.40 1999/08/01 10:23:45 lha Exp $");
 
 int
 xfs_message_installroot(int fd,
@@ -80,11 +81,13 @@ xfs_message_installnode(int fd,
 
     XFSDEB(XDEBMSG, ("xfs_message_installnode\n"));
 
+retry:
     dp = xfs_node_find(&xfs[fd], &message->parent_handle);
     if (dp) {
 	struct vnode *t_vnode = XNODE_TO_VNODE(dp);
 
-	xfs_do_vget (t_vnode, LK_INTERLOCK|LK_SHARED, p);
+	if (xfs_do_vget (t_vnode, LK_INTERLOCK|LK_SHARED, p))
+		goto retry;
 
 	error = new_xfs_node(&xfs[fd], &message->node, &n, p);
 	if (error) {
@@ -119,7 +122,9 @@ xfs_message_installattr(int fd,
     if (t != 0) {
 	t->tokens = message->node.tokens;
 	xfs_attr2vattr(&message->node.attr, &t->attr);
-#ifdef HAVE_KERNEL_VNODE_PAGER_SETSIZE
+#ifdef UVM
+	uvm_vnp_setsize(XNODE_TO_VNODE(t), t->attr.va_size);
+#elif HAVE_KERNEL_VNODE_PAGER_SETSIZE
 	vnode_pager_setsize(XNODE_TO_VNODE(t), t->attr.va_size);
 #endif
 	bcopy(message->node.id, t->id, sizeof(t->id));
@@ -144,24 +149,20 @@ xfs_message_installdata(int fd,
 
     XFSDEB(XDEBMSG, ("xfs_message_installdata\n"));
 
+retry:
     t = xfs_node_find(&xfs[fd], &message->node.handle);
-
     if (t != NULL) {
-	struct xfs_fh_args *fh_args = (struct xfs_fh_args *)&message->cache_handle;
+	struct xfs_fhandle_t *fh = (struct xfs_fhandle_t *)&message->cache_handle;
 	struct vnode *vp;
 	struct vnode *t_vnode = XNODE_TO_VNODE(t);
 
 	XFSDEB(XDEBMSG, ("cache_name = '%s'\n", message->cache_name));
-	XFSDEB(XDEBMSG, ("fileno = %ld, gen = %ld\n", 
-			 SCARG(fh_args, fileid),
-			 SCARG(fh_args, gen)));
 
-	xfs_do_vget (t_vnode, LK_INTERLOCK|LK_SHARED, p);
+	if (xfs_do_vget(t_vnode, LK_INTERLOCK|LK_SHARED, p))
+		goto retry;
 
 	error = xfs_fhlookup (p,
-			      SCARG(fh_args,fsid),
-			      SCARG(fh_args,fileid),
-			      SCARG(fh_args,gen),
+			      fh,
 			      &vp);
 
 	if (error != 0) {
@@ -183,7 +184,9 @@ xfs_message_installdata(int fd,
 	}
 
 	if (error == 0) {
+#ifndef __osf__
 	    xfs_vfs_unlock(vp, p);
+#endif
 	    if (DATA_FROM_XNODE(t))
 		vrele(DATA_FROM_XNODE(t));
 	    DATA_FROM_XNODE(t) = vp;
@@ -267,3 +270,120 @@ xfs_message_updatefid(int fd,
     }
     return error;
 }
+
+/*
+ * Try to clean out nodes for the userland daemon
+ */
+
+static void
+gc_vnode (struct vnode *vp,
+	  struct proc *p)
+{
+    simple_lock(&vp->v_interlock);
+    
+    /* This node is on the freelist */
+    if (vp->v_usecount <= 0) {
+	
+	/*  DIAGNOSTIC */
+	if (vp->v_usecount < 0 || vp->v_writecount != 0) {
+		    vprint("vrele: bad ref count", vp);
+		    panic("vrele: ref cnt");
+	}
+	
+	XFSDEB(XDEBMSG, ("xfs_message_gc: success\n"));
+	
+#ifdef HAVE_KERNEL_FUNC_VGONEL
+	vgonel (vp, p);
+#else
+	simple_unlock(&vp->v_interlock); 
+	vgone (vp);
+#endif
+
+    } else {
+	XFSDEB(XDEBMSG, ("xfs_message_gc: used\n"));
+    }
+
+}
+
+int
+xfs_message_gc_nodes(int fd,
+		     struct xfs_message_gc_nodes *message,
+		     u_int size,
+		     struct proc *p)
+{
+    XFSDEB(XDEBMSG, ("xfs_message_gc\n"));
+
+    if (message->len == 0) {
+	struct vnode *vp;
+
+	/* XXX see comment in xfs_node_find */
+	/* XXX osf */
+	/* XXXSMP do gone[l] need to get mntvnode_slock ? */
+
+	for(vp = XFS_TO_VFS(&xfs[fd])->mnt_vnodelist.lh_first;
+	    vp != NULL; 
+	    vp = vp->v_mntvnodes.le_next) {
+
+	    gc_vnode (vp, p);
+	}
+
+    } else {
+	struct xfs_node *t;
+	int i;
+
+	for (i = 0; i < message->len; i++) {
+	    t = xfs_node_find (&xfs[fd], &message->handle[i]);
+	    if (t == NULL)
+		continue;
+
+	    gc_vnode(XNODE_TO_VNODE(t), p);
+	}
+    }
+
+    return 0;
+}
+
+
+/*
+ * Probe what this xfs support
+ */
+
+int
+xfs_message_version(int fd,
+		    struct xfs_message_version *message,
+		    u_int size,
+		    struct proc *p)
+{
+    struct xfs_message_wakeup msg;
+    int ret;
+
+    switch (message->probe) {
+    case XFS_VERSION_VERSION:
+	ret = XFS_VERSION_YES;
+	break;
+    case XFS_VERSION_FHGET:
+	ret = XFS_VERSION_YES;
+	break;
+#if defined(HAVE_GETFH) && defined(HAVE_FHOPEN)
+    case XFS_VERSION_FHGET_NATIVE:
+	ret = XFS_VERSION_YES;
+	break;
+#endif	
+    case XFS_VERSION_GC_NODES:
+	ret = XFS_VERSION_YES;
+	break;
+    default:
+	ret = XFS_VERSION_NO;
+	break;
+    }
+
+    msg.header.opcode = XFS_MSG_WAKEUP;
+    msg.sleepers_sequence_num = message->header.sequence_num;
+    msg.error = ret;
+
+    return xfs_message_send(fd, (struct xfs_message_header *) &msg, sizeof(msg));
+}
+
+
+
+
