@@ -41,7 +41,7 @@
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_node-bsd.c,v 1.24 1999/02/26 00:47:48 lha Exp $");
+RCSID("$Id: xfs_node-bsd.c,v 1.25 1999/06/08 17:49:11 art Exp $");
 
 extern vop_t **xfs_vnodeop_p;
 
@@ -219,6 +219,18 @@ xfs_node_find(struct xfs *xfsp, xfs_handle *handlep)
     XFSDEB(XDEBNODE, ("xfs_node_find: xfsp = %p handlep = %p\n", 
 		      xfsp, handlep));
 
+    /*
+     * XXXSMP - the vnodes on mnt_vnodelist are invalid unless we hold
+     *          mntvnode_slock (same on Open,Free and Net - current).
+     * XXX - Another problem here is that the data in the vnode doesn't
+     *       have to be correct unless we do a vget first (if usecount on
+     *       vnode == 0). This should only be a problem when someone uses
+     *       revoke, when unmounting or when arlad dies (or on SMP systems
+     *       with a shortage of vnodes).
+     *       We might want to vget here, but that was a problem
+     *       on FreeBSD once.
+     */
+
     for(t = XFS_TO_VFS(xfsp)->mnt_vnodelist.lh_first;
 	t != NULL; 
 	t = t->v_mntvnodes.le_next) {
