@@ -38,7 +38,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls-wrap-bsd.c,v 1.2 1999/03/27 04:18:12 assar Exp $");
+RCSID("$Id: xfs_syscalls-wrap-bsd.c,v 1.4 1999/06/09 20:30:53 joda Exp $");
 
 /*
  * XFS system calls.
@@ -63,6 +63,10 @@ static struct sysent xfs_syscallent = {
     xfspioctl			       /* function pointer */
 };
 
+static struct sysent old_setgroups;
+
+int (*old_setgroups_func)(struct proc *, void *);
+
 /* XXX really defined in kern/kern_lkm.c */
 extern int
 sys_lkmnosys(struct proc *p, void *v, register_t *retval);
@@ -71,17 +75,32 @@ sys_lkmnosys(struct proc *p, void *v, register_t *retval);
  *
  */
 
+#ifdef HAVE_KERNEL_AOUT_SYSENT
+/* XXX if we are running with a.out compatibility, we need to add all
+   syscalls to both the ELF, and the a.out syscall tables */
+/* XXX this should be made generic, so we can add support for, say,
+   linux afs binaries */
+extern struct sysent aout_sysent[];
+#endif
+
 static int
 try_install_syscall (int offset,
 		     struct sysent new_sysent,
 		     struct sysent *old_sysent)
 {
-    if (sysent[offset].sy_call == sys_lkmnosys) {
-	*old_sysent = sysent[offset];
-	sysent[offset] = new_sysent;
-	return 0;
-    }
-    return EBUSY;
+    if(sysent[offset].sy_call != sys_lkmnosys)
+       return EBUSY;
+#ifdef HAVE_KERNEL_AOUT_SYSENT
+    if(aout_sysent[offset].sy_call != sys_lkmnosys)
+	return EBUSY;
+#endif
+    
+    *old_sysent = sysent[offset];
+    sysent[offset] = new_sysent;
+#ifdef HAVE_KERNEL_AOUT_SYSENT
+    aout_sysent[offset] = new_sysent;
+#endif
+    return 0;
 }
 
 /*
@@ -116,6 +135,19 @@ xfs_install_syscalls(void)
 {
     int ret = ENOENT;
 
+#ifdef HAVE_KERNEL_AOUT_SYSENT
+    /* XXX make sure that the ELF, and the a.out syscall are the same;
+       if they're not we don't know what to do (infact you could just
+       add two different setgroups calls, but this is probably not
+       worth the trouble */
+
+    if(sysent[SYS_setgroups].sy_call != 
+       aout_sysent[SYS_setgroups].sy_call){
+	printf("%s: ELF and a.out setgroups syscalls differ!\n", 
+	       __FUNCTION__);
+	return ret;
+    }
+#endif
 #ifdef AFS_SYSCALL
     if (ret != 0) {
 	ret = try_install_syscall(AFS_SYSCALL,
@@ -133,14 +165,31 @@ xfs_install_syscalls(void)
 	XFSDEB(XDEBSYS, ("xfs_syscall installed at %d\n", syscall_offset));
     else
 	XFSDEB(XDEBSYS, ("failed installing xfs_syscall\n"));
+    if (ret == 0) {
+	old_setgroups = sysent[SYS_setgroups];
+	old_setgroups_func = old_setgroups.sy_call;
+	sysent[SYS_setgroups].sy_call = xfs_setgroups;
+#ifdef HAVE_KERNEL_AOUT_SYSENT
+	aout_sysent[SYS_setgroups].sy_call = xfs_setgroups;
+#endif
+    }
     return ret;
 }
 
 int
 xfs_uninstall_syscalls(void)
 {
-    if (syscall_offset)
+    if (syscall_offset) {
 	sysent[syscall_offset] = old_syscallent;
+#ifdef HAVE_KERNEL_AOUT_SYSENT
+	aout_sysent[syscall_offset] = old_syscallent;
+#endif
+    }
+    sysent[SYS_setgroups] = old_setgroups;
+#ifdef HAVE_KERNEL_AOUT_SYSENT
+    aout_sysent[SYS_setgroups] = old_setgroups;
+#endif
+
     return 0;
 }
 
