@@ -49,9 +49,11 @@
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_vnodeops.h>
 #include <vm/vm.h>
+#ifdef HAVE_VM_VNODE_PAGER_H
 #include <vm/vnode_pager.h>
+#endif
 
-RCSID("$Id: xfs_vnodeops-bsd.c,v 1.47 1999/03/01 08:57:15 assar Exp $");
+RCSID("$Id: xfs_vnodeops-bsd.c,v 1.53 1999/08/02 02:19:53 assar Exp $");
 
 /*
  * vnode functions
@@ -512,13 +514,33 @@ xfs_link(struct vop_link_args * ap)
 	*/
 {
     struct componentname *cnp = ap->a_cnp;
-    int error = xfs_link_common(
+    struct vnode *vp = ap->a_vp;
+    struct vnode *dvp;
+    struct proc *p = cnp->cn_proc;
+    int error;
+
 #if defined (__OpenBSD__) || defined(__NetBSD__)
-			   ap->a_dvp, 
+    dvp = ap->a_dvp;
 #elif defined(__FreeBSD__)
-			   ap->a_tdvp, 
+    dvp = ap->a_tdvp;
 #endif
-			   ap->a_vp, 
+
+    if (vp->v_type == VDIR) {
+	    VOP_ABORTOP(dvp, cnp);
+	    return EPERM;
+    }
+    if (dvp->v_mount != vp->v_mount) {
+	    VOP_ABORTOP(dvp, cnp);
+	    return EXDEV;
+    }
+    if (dvp != vp && (error = xfs_vfs_writelock(vp, p))) {
+	    VOP_ABORTOP(dvp, cnp);
+	    return error;
+    }
+
+    error = xfs_link_common(
+			   dvp,
+			   vp,
 			   cnp->cn_nameptr,
 			   cnp->cn_cred);
 
@@ -529,6 +551,13 @@ xfs_link(struct vop_link_args * ap)
 	free (cnp->cn_pnbuf, M_NAMEI);
 #endif
 
+    if (dvp != vp)
+	xfs_vfs_unlock(vp, p);
+
+#if defined(__OpenBSD__) || defined(__NetBSD__) || (defined(__FreeBSD__) && !defined(__FreeBSD_version))
+    vput(dvp);
+#endif
+
     return error;
 }
 #endif /* HAVE_VOP_LINK */
