@@ -49,7 +49,7 @@
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_vnodeops.h>
 
-RCSID("$Id: xfs_vnodeops-common.c,v 1.31 1999/03/19 04:53:22 lha Exp $");
+RCSID("$Id: xfs_vnodeops-common.c,v 1.34 1999/07/07 16:51:12 art Exp $");
 
 int
 xfs_open_valid(struct vnode * vp, struct ucred * cred, u_int tok)
@@ -255,11 +255,7 @@ xfs_read_common(struct vnode *vp, struct uio *uio, int ioflag, struct ucred *cre
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
 	xfs_vfs_readlock(t, xfs_uio_to_proc(uio));
-#ifdef __osf__
-	VOP_READ(t, uio, ioflag, cred, error);
-#else /* __osf__ */
-	error = VOP_READ(t, uio, ioflag, cred);
-#endif /* __osf__ */
+	xfs_vop_read(t, uio, ioflag, cred, error);
 	xfs_vfs_unlock(t, xfs_uio_to_proc(uio));
     }
     return error;
@@ -281,20 +277,16 @@ xfs_write_common(struct vnode *vp, struct uio *uiop, int ioflag, struct ucred *c
 	int error2 = 0;
  
 	xfs_vfs_writelock(t, xfs_uio_to_proc(uiop));
-#ifdef __osf__
-	VOP_WRITE(t, uiop, ioflag, cred, error);
+	xfs_vop_write(t, uiop, ioflag, cred, error);
 	VNODE_TO_XNODE(vp)->flags |= XFS_DATA_DIRTY;
-	VOP_GETATTR(t, &sub_attr, cred, error2);
-#else /* __osf__ */
-	error = VOP_WRITE(t, uiop, ioflag, cred);
-	VNODE_TO_XNODE(vp)->flags |= XFS_DATA_DIRTY;
-	error2 = VOP_GETATTR(t, &sub_attr, cred, uiop->uio_procp);
-#endif /* __osf__ */
+	xfs_vop_getattr(t, &sub_attr, cred, xfs_uio_to_proc(uiop), error2);
 
 	if (error2 == 0) {
 	    xn->attr.va_size  = sub_attr.va_size;
 	    xn->attr.va_mtime = sub_attr.va_mtime;
-#ifdef HAVE_KERNEL_VNODE_PAGER_SETSIZE
+#ifdef UVM
+	    uvm_vnp_setsize(vp, sub_attr.va_size);
+#elif HAVE_KERNEL_VNODE_PAGER_SETSIZE
 	    vnode_pager_setsize(vp, sub_attr.va_size);
 #endif
 	}
@@ -443,11 +435,7 @@ xfs_lookup_common(struct vnode *dvp,
  again:
 
     do {
-#ifdef __osf__
-	VOP_ACCESS(dvp, VEXEC, cred, error);
-#else
-	error = VOP_ACCESS(dvp, VEXEC, cred, xfs_cnp_to_proc(cnp));
-#endif
+	xfs_vop_access(dvp, VEXEC, cred, xfs_cnp_to_proc(cnp), error);
 	if (error != 0)
 	    goto done;
 
@@ -717,12 +705,8 @@ xfs_readdir_common(struct vnode *vp,
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
 	xfs_vfs_readlock(t, xfs_uio_to_proc(uiop));
-#ifdef __osf__
-	VOP_READ(t, uiop, 0, cred, error);
-#else
-	error = VOP_READ(t, uiop, 0, cred);
-#endif
-	if(eofflag)
+	xfs_vop_read(t, uiop, 0, cred, error);
+	if (eofflag)
 	    *eofflag = VNODE_TO_XNODE(vp)->attr.va_size <= uiop->uio_offset;
 	xfs_vfs_unlock(t, xfs_uio_to_proc(uiop));
     }
@@ -812,11 +796,7 @@ xfs_readlink_common(struct vnode *vp, struct uio *uiop, struct ucred *cred)
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
 	xfs_vfs_readlock(t, xfs_uio_to_proc(uiop));
-#ifdef __osf__
-	VOP_READ(t, uiop, 0, cred, error);
-#else
-	error = VOP_READ(t, uiop, 0, cred);
-#endif
+	xfs_vop_read(t, uiop, 0, cred, error);
 	xfs_vfs_unlock(t, xfs_uio_to_proc(uiop));
     }
     return error;
