@@ -43,7 +43,7 @@
 #include <linux/poll.h>
 #endif /* LINUX2_1 */
 
-RCSID("$Id: xfs_dev.c,v 1.43 1999/01/10 18:48:02 map Exp $");
+RCSID("$Id: xfs_dev.c,v 1.48 1999/07/25 03:47:07 map Exp $");
 
 #include <xfs/xfs_message.h>
 #include <xfs/xfs_msg_locl.h>
@@ -52,6 +52,18 @@ RCSID("$Id: xfs_dev.c,v 1.43 1999/01/10 18:48:02 map Exp $");
 #include <xfs/xfs_deb.h>
 #include <xfs/nxfs.h>
 
+#ifndef HAVE_WAIT_QUEUE_HEAD_T
+typedef struct wait_queue *wait_queue_head_t;
+#endif
+
+#ifndef HAVE_INIT_WAITQUEUE_HEAD
+static void
+init_waitqueue_head(wait_queue_head_t *h)
+{
+    *h = NULL;
+}
+#endif
+
 /*
  * Queues of xfs_links hold outbound messages and processes sleeping
  * for replies. The last field is used to return error to sleepers and
@@ -62,14 +74,15 @@ struct xfs_link {
     struct xfs_link *prev, *next;
     struct xfs_message_header *message;
     u_int error_or_size;	/* error on sleepq and size on messageq */
-    struct wait_queue *wait_queue;
+    u_int woken;
+    wait_queue_head_t wait_queue;
 };  
 
 struct xfs_channel {
     struct xfs_link messageq;	/* Messages not yet read */
     struct xfs_link sleepq;	/* Waiting for reply message */
     u_int nsequence;
-    struct wait_queue *wait_queue;
+    wait_queue_head_t wait_queue;
     struct xfs_message_header *message_buffer;
     int status;
 #define CHANNEL_OPENED	0x1
@@ -83,7 +96,8 @@ xfs_initq(struct xfs_link *q)
 {
     q->next = q;
     q->prev = q;
-    q->wait_queue = NULL;
+    init_waitqueue_head(&(q->wait_queue));
+    q->woken = 0;
 }
 
 /* Is this queue empty? */
@@ -182,6 +196,7 @@ xfs_devclose(struct inode * inode, struct file * file)
 	up(&chan->channel_sem);
 	first->error_or_size = -ENODEV;
 	wake_up(&first->wait_queue);
+	first->woken = 1;
 	XFSDEB(XDEBDEV, ("after outq(sleepq)\n"));
 	down(&chan->channel_sem);
     }
@@ -407,13 +422,18 @@ xfs_message_send(int fd, struct xfs_message_header *message, u_int size)
     t->this_message.message = &t->msg;
     down(&chan->channel_sem);
     xfs_appendq(&chan->messageq, &t->this_message);
-    t->this_message.wait_queue = NULL;
+    init_waitqueue_head(&(t->this_message.wait_queue));
     up(&chan->channel_sem);
-    if ( chan->wait_queue )
-	wake_up(&chan->wait_queue);
+    wake_up(&chan->wait_queue);
     return 0;
 }
 
+static inline int sigisactive(struct task_struct *t, int signal)
+{
+    return (sigismember(&t->signal, signal) &&
+	    !sigismember(&t->blocked, signal));
+}
+
 /*
  * Send a message to user space and wait for reply.
  */
@@ -425,7 +445,6 @@ xfs_message_rpc(int fd, struct xfs_message_header *message, u_int size)
     struct xfs_link *this_message;
     struct xfs_link *this_process;
     struct xfs_message_header *msg;
-    sigset_t oldblocked;
     
     XFSDEB(XDEBMSG, ("xfs_message_rpc opcode = %d\n", message->opcode));
     
@@ -447,8 +466,8 @@ xfs_message_rpc(int fd, struct xfs_message_header *message, u_int size)
     XFSDEB(XDEBMSG, ("xfs_message_rpc seq = %d\n", msg->sequence_num));
     this_message->error_or_size = 0;
     
-    this_message->wait_queue = NULL;
-    this_process->wait_queue = NULL;
+    init_waitqueue_head(&(this_message->wait_queue));
+    init_waitqueue_head(&(this_process->wait_queue));
     
     this_message->message = msg; /* message;*/
     this_process->message = msg; /* message; */
@@ -460,19 +479,17 @@ xfs_message_rpc(int fd, struct xfs_message_header *message, u_int size)
     
     up(&chan->channel_sem);
     
-    if ( chan->wait_queue )
-	wake_up(&chan->wait_queue);
+    wake_up(&chan->wait_queue);
     
 #ifdef LINUX2_1
-    spin_lock_irq(&current->sigmask_lock);
-    oldblocked = current->blocked;
-    siginitsetinv(&current->blocked,
-		  sigmask(SIGINT) | sigmask(SIGTERM) | sigmask(SIGSTOP) |
-		  sigmask(SIGKILL));
-    recalc_sigpending(current);
-    spin_unlock_irq(&current->sigmask_lock);
-    
-    interruptible_sleep_on(&this_process->wait_queue);
+    this_process->woken = 0;
+    while (!sigisactive(current, SIGINT) &&
+	   !sigisactive(current, SIGTERM) &&
+	   !sigisactive(current, SIGSTOP) &&
+	   !sigisactive(current, SIGKILL) &&
+	   !this_process->woken) {
+	interruptible_sleep_on(&this_process->wait_queue);
+    }
 #else
     /* 
      * XXX
@@ -481,21 +498,15 @@ xfs_message_rpc(int fd, struct xfs_message_header *message, u_int size)
      * has been written.
      */
     
+    this_process->woken = 0;
     sleep_on(&this_process->wait_queue);
 #endif
     
-    if (signal_pending(current)) {
+    if (!this_process->woken) {
 	XFSDEB(XDEBMSG, ("caught signal\n"));
 	this_process->error_or_size = -EINTR;
     }
     
-#ifdef LINUX2_1
-    spin_lock_irq(&current->sigmask_lock);
-    current->blocked = oldblocked;
-    recalc_sigpending(current);
-    spin_unlock_irq(&current->sigmask_lock);
-#endif
-    
     XFSDEB(XDEBMSG, ("xfs_message_rpc wakeup: seq = %d\n",
 		     msg->sequence_num));
     /*
@@ -573,6 +584,14 @@ xfs_message_receive(int fd, struct xfs_message_header *message, u_int size)
 	return xfs_message_updatefid(fd,
 				     (struct xfs_message_updatefid *)message,
 				     message->size);
+    case XFS_MSG_GC_NODES:
+	return xfs_message_gc_nodes(fd,
+				    (struct xfs_message_gc_nodes *)message,
+				    message->size);
+    case XFS_MSG_VERSION:
+	return xfs_message_version(fd,
+				   (struct xfs_message_version *)message,
+				   message->size);
     default:
 	printk(KERN_EMERG "XFS Panic: xfs_dev: Unknown message opcode == %d\n",
 	       message->opcode);
@@ -603,8 +622,8 @@ xfs_message_wakeup(int fd, struct xfs_message_wakeup *message, u_int size)
 		else
 		    t->error_or_size = -message->error;
 	    }
-	    if (t->wait_queue)
-		wake_up(&t->wait_queue);
+	    wake_up(&t->wait_queue);
+	    t->woken = 1;
 	    break;
 	}
     up(&chan->channel_sem);
@@ -636,8 +655,8 @@ xfs_message_wakeup_data(int fd, struct xfs_message_wakeup_data *message,
 		else
 		    t->error_or_size = -message->error;
 	    }
-	    if (t->wait_queue)
-		wake_up(&t->wait_queue);
+	    wake_up(&t->wait_queue);
+	    t->woken = 1;
 	    break;
 	}
     up(&chan->channel_sem);
@@ -671,10 +690,14 @@ xfs_init_device(void)
     
     for (i = 0; i < NXFS; i++) {
 	XFSDEB(XDEBDEV, ("before initq(messageq and sleepq)\n"));
-	xfs_channel[i].wait_queue = NULL;
+	init_waitqueue_head(&(xfs_channel[i].wait_queue));
 	xfs_initq(&xfs_channel[i].messageq);
 	xfs_initq(&xfs_channel[i].sleepq);
+#ifdef HAVE_INIT_MUTEX	
+	init_MUTEX(&(xfs_channel[i].channel_sem));
+#else
 	xfs_channel[i].channel_sem = MUTEX;
+#endif
     }
     return 0;
 }
