@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -47,7 +47,7 @@
 #include <linux/file.h>
 #include <linux/mm.h>
 
-RCSID("$Id: xfs_inodeops.c,v 1.78 1999/02/06 00:42:42 map Exp $");
+RCSID("$Id: xfs_inodeops.c,v 1.85 1999/07/25 05:21:45 map Exp $");
 
 static int
 #ifndef LINUX2_1
@@ -189,6 +189,11 @@ static int
 xfs_read_file(struct inode *inode, struct file *file, char *buf, int count);
 #endif
 
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_GETATTR
+static int
+xfs_getattr (struct inode *inode, struct stat *sb);
+#endif
+
 #if 0
 void xfs_truncate (struct inode * inode);
 #endif
@@ -244,7 +249,16 @@ struct inode_operations xfs_file_inode_operations = {
         NULL,                   /* bmap */
         NULL,                   /* truncate */
         xfs_permission,         /* permission */
-        NULL                    /* smap */
+        NULL,			/* smap */
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_UPDATEPAGE
+	NULL,			/* updatepage */
+#endif
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_REVALIDATE
+	NULL,			/* revalidate */
+#endif
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_GETATTR
+	xfs_getattr		/* getattr */
+#endif
 };
 
 static struct file_operations xfs_dead_operations = {
@@ -328,7 +342,16 @@ struct inode_operations xfs_dir_inode_operations = {
         NULL,                   /* bmap */
         NULL,                   /* truncate */
         xfs_permission,         /* permission */
-        NULL                    /* smap */
+        NULL,                   /* smap */
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_UPDATEPAGE
+	NULL,			/* updatepage */
+#endif
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_REVALIDATE
+	NULL,			/* revalidate */
+#endif
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_GETATTR
+	xfs_getattr		/* getattr */
+#endif
 };
 
 static struct file_operations xfs_link_operations = {
@@ -368,7 +391,16 @@ struct inode_operations xfs_link_inode_operations = {
         NULL,                   /* bmap */
         NULL,                   /* truncate */
         NULL,                   /* permission */
-        NULL                    /* smap */
+        NULL,			/* smap */
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_UPDATEPAGE
+	NULL,			/* updatepage */
+#endif
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_REVALIDATE
+	NULL,			/* revalidate */
+#endif
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_GETATTR
+	xfs_getattr		/* getattr */
+#endif
 };
 
 #ifdef LINUX2_1
@@ -466,13 +498,13 @@ done:
 void
 xfs_print_lock(char *s, struct semaphore *sem)
 {
-    XFSDEB(XDEBLOCK, ("lock: %s sem: %p count: %d waking: %d wait: %p",
-		      s, sem, atomic_read(&sem->count),
-		      sem->waking, sem->wait));
-    if (sem->wait) {
-	XFSDEB(XDEBLOCK, (" task: %p next: %p",
-			  sem->wait->task, sem->wait->next));
-    }
+    XFSDEB(XDEBLOCK, ("lock: %s sem: %p count: %d",
+		      s, sem, (int)atomic_read(&sem->count)));
+#if 0				/* XXX - this is different on
+different architectures which makes it hard to print */
+" waking: %d",
+(int)atomic_read(&sem->waking)));
+#endif
     XFSDEB(XDEBLOCK, ("\n"));
 }
 
@@ -853,13 +885,13 @@ xfs_read_file(struct inode *inode, struct file *file, char *buf, int count)
     return error;
 }
 #else
-static int
+static ssize_t
 xfs_read_file(struct file *file, char *buf, size_t count, loff_t *ppos)
 {
     int error = 0;
     struct inode *inode = file->f_dentry->d_inode;
     
-    XFSDEB(XDEBVNOPS, ("xfs_read_file file->f_pos: %d count: %d\n",(int) file->f_pos,count));
+    XFSDEB(XDEBVNOPS, ("xfs_read_file file->f_pos: %d count: %u\n",(int) file->f_pos,(unsigned)count));
     xfs_print_path(file->f_dentry);
     XFSDEB(XDEBVNOPS, ("xfs_read_file aliases:"));
     print_aliases(inode);
@@ -985,9 +1017,9 @@ xfs_write_file(struct inode *inode, struct file *file, const char *buf, int coun
 #ifdef LINUX2_1
     struct inode *inode = file->f_dentry->d_inode;
 #endif
-    unsigned long page;
     
-    XFSDEB(XDEBVNOPS, ("xfs_write_file file->f_pos: %d count: %d inode: %p ",(int) file->f_pos,count,inode));
+    XFSDEB(XDEBVNOPS, ("xfs_write_file file->f_pos: %d count: %u inode: %p ",
+		       (int) file->f_pos, (unsigned) count, inode));
 #ifdef LINUX2_1
     xfs_print_path(file->f_dentry);
 #else
@@ -1051,25 +1083,29 @@ xfs_write_file(struct inode *inode, struct file *file, const char *buf, int coun
 			/* c) both? */
 			/* d) neither? */
 			/* e) none of the above? */
-	page = get_free_page(GFP_KERNEL);
-	if (!page)
-	    invalidate_inode_pages(inode);
-	else {
-	    int pos = file->f_pos;
-	    int cnt = count;
-	    int blk;
-	    char *data=(char *)page;
+	{
+	    unsigned long page;
 
-	    while (cnt > 0) {
-		blk = cnt;
-		if (blk > PAGE_SIZE)
-		    blk = PAGE_SIZE;
-		copy_from_user(data, buf, blk);
-		update_vm_cache(inode, pos, data, blk);
-		pos += blk;
-		cnt -= blk;
+	    page = get_free_page(GFP_KERNEL);
+	    if (!page)
+		invalidate_inode_pages(inode);
+	    else {
+		int pos = file->f_pos;
+		int cnt = count;
+		int blk;
+		char *data=(char *)page;
+
+		while (cnt > 0) {
+		    blk = cnt;
+		    if (blk > PAGE_SIZE)
+			blk = PAGE_SIZE;
+		    copy_from_user(data, buf, blk);
+		    update_vm_cache(inode, pos, data, blk);
+		    pos += blk;
+		    cnt -= blk;
+		}
+		free_page(page);
 	    }
-	    free_page(page);
 	}
 #endif
     }
@@ -1263,8 +1299,10 @@ xfs_rename (struct inode * old_dir, struct dentry *old_dentry,
     iput(old_dir);
     iput(new_dir);
 #else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,2,7)
     if (error == 0)
 	d_move(old_dentry, new_dentry);
+#endif
 #endif /* LINUX2_1 */
 
     return error;
@@ -1888,6 +1926,32 @@ void xfs_truncate (struct inode *inode)
 }
 #endif
 
+#ifdef HAVE_STRUCT_INODE_OPERATIONS_GETATTR
+static int
+xfs_getattr (struct inode *inode, struct stat *sb)
+{
+    struct xfs_node *xn = VNODE_TO_XNODE(inode);
+
+    XFSDEB(XDEBVNOPS, ("xfs_getattr\n"));
+
+    memset (sb, 0, sizeof(*sb));
+    sb->st_dev     = kdev_t_to_nr(inode->i_dev);
+    sb->st_ino     = xn->attr.xa_fileid;
+    sb->st_mode    = inode->i_mode;
+    sb->st_nlink   = inode->i_nlink;
+    sb->st_uid     = inode->i_uid;
+    sb->st_gid     = inode->i_gid;
+    sb->st_rdev    = kdev_t_to_nr(inode->i_rdev);
+    sb->st_size    = inode->i_size;
+    sb->st_atime   = inode->i_atime;
+    sb->st_mtime   = inode->i_mtime;
+    sb->st_ctime   = inode->i_ctime;
+    sb->st_blocks  = inode->i_blocks;
+    sb->st_blksize = inode->i_blksize;
+    return 0;
+}
+#endif /* HAVE_STRUCT_INODE_OPERATIONS_GETATTR */
+
 #ifdef LINUX2_1
 static int
 xfs_d_validate(struct dentry *dentry)
@@ -1974,6 +2038,7 @@ xfs_d_delete(struct dentry *dentry)
     }
 
     XFSDEB(XDEBVNOPS,("xfs_d_delete: total d_count: %d\n", d_count));
+#if 0
     if (d_count == 0 &&
 	dentry->d_inode &&
 	VNODE_TO_XNODE(dentry->d_inode)) {
@@ -1989,6 +2054,7 @@ xfs_d_delete(struct dentry *dentry)
 	msg.flag   = XFS_NOREFS;
 	xfs_message_send(xfsp->fd, &msg.header, sizeof(msg));
     }
+#endif
 }
 #endif
 
