@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -48,7 +48,7 @@
 #include <linux/smp_lock.h>
 #include <linux/smp.h>
 
-RCSID("$Id: xfs_syscalls.c,v 1.45 1999/01/24 22:53:56 map Exp $");
+RCSID("$Id: xfs_syscalls.c,v 1.47.2.1 1999/08/21 23:45:49 map Exp $");
 
 /* 
  * Def pag:
@@ -105,10 +105,95 @@ pag_t xfs_get_pag(void)
 	return current->uid;
 }
 
+#ifndef LINUX2_1
+
+static int
+store_pag (gid_t part1, gid_t part2)
+{
+    int i;
+
+    if (!xfs_is_pag ()) {
+	int ngroups = xfs_get_ngroups ();
+
+	if (ngroups + 2 >= NGROUPS)
+	    return -E2BIG;
+	for (i = ngroups - 1; i >= 0; i--) {
+	    current->groups[i + 2] = current->groups[i];
+	}
+	ngroups += 2;
+	if (ngroups < NGROUPS)
+	    current->groups[ngroups] = NOGROUP;
+    }
+    current->groups[0] = part1;
+    current->groups[1] = part2;
+    return 0;
+}
+
+/* remove the current process's PAG (i.e. go back to default PAG) */
+
+static int
+unpag (void)
+{
+    int i;
+
+    while(xfs_is_pag ()) {
+	int ngroups = xfs_get_ngroups ();
+
+	for (i = 0; i < ngroups - 2; i++) {
+	    current->groups[i] = current->groups[i+2];
+	}
+	ngroups -= 2;
+	current->groups[ngroups] = NOGROUP;
+    }
+    return 0;
+}
+
+
+#else /* LINUX2_1 */
+
+static int
+store_pag (gid_t part1, gid_t part2)
+{
+    int i;
+
+    if (!xfs_is_pag ()) {
+	if (current->ngroups + 2 >= NGROUPS)
+	    return -E2BIG;
+	for (i = current->ngroups - 1; i >= 0; i--) {
+	    current->groups[i + 2] = current->groups[i];
+	}
+	current->ngroups += 2;
+    }
+    current->groups[0] = part1;
+    current->groups[1] = part2;
+    return 0;
+}
+
+/*
+ * Nuke any existing pag.
+ */
+
+static int
+unpag (void)
+{
+    int i;
+
+    while(xfs_is_pag ()) {
+	for (i = 0; i < current->ngroups - 2; i++) {
+	    current->groups[i] = current->groups[i+2];
+	}
+	/* Since ngroups >= 2 if xfs_is_pag(), this can't go negative. */
+	current->ngroups -= 2;
+    }
+    return 0;
+}
+#endif /* LINUX2_1 */
+
 static int
 xfs_setpag_call(void)
 {
-    int i ;
+    int i;
+    int ret;
 
 #ifdef DEBUG
     XFSDEB(XDEBSYS, ("xfs_setpag_call groups: "));
@@ -121,39 +206,9 @@ xfs_setpag_call(void)
     XFSDEB(XDEBSYS, ("\n"));
 #endif
 
-    if (!xfs_is_pag()) {
-#ifndef LINUX2_1
-	int ngroups = xfs_get_ngroups();
-#endif /* LINUX2_1 */
-
-	/* Check if it fits */
-#ifndef LINUX2_1
-	if (ngroups + 2 >= NGROUPS) 
-#else
-	if (current->ngroups + 2 >= NGROUPS) 
-#endif /* LINUX2_1 */
-	    return -E2BIG;             /* XXX Hmmm, better error ? */
-
-	/* Copy the groups */
-#ifndef LINUX2_1
-	for (i = ngroups - 1; i >= 0 ; i--) {
-#else
-	for (i = current->ngroups - 1; i >= 0 ; i--) {
-#endif /* LINUX2_1 */
-	    current->groups[i+2] = current->groups[i] ; 
-	}
-#ifndef LINUX2_1
-	ngroups += 2 ;
-	if (ngroups < NGROUPS)
-	    current->groups[ngroups] = NOGROUP;
-#else
-	current->ngroups += 2 ;
-#endif /* LINUX2_1 */
-
-    }
-
-    current->groups[0] = pag_part_one ;
-    current->groups[1] = pag_part_two++ ;
+    ret = store_pag (pag_part_one, pag_part_two++);
+    if (ret)
+	return ret;
 
     if (pag_part_two > XFS_PAG2_ULIM) {
 	pag_part_one++ ;
@@ -167,6 +222,70 @@ xfs_setpag_call(void)
     return 0;
 }
 
+typedef int (*sys_afs_function)(int operation,
+				char *a_pathP,
+				int a_opcode,
+				struct ViceIoctl *a_paramsP,
+				int a_followSymlinks);
+
+typedef int (*sys_setgroups_function)(int, gid_t *);
+
+typedef int (*sys_call_function)(void);
+
+extern sys_call_function sys_call_table[];
+
+static sys_call_function old_afs_syscall=NULL;
+
+static sys_call_function old_setgroups = NULL;
+
+/*
+ * A wrapper around sys_setgroups that tries to preserve the pag. If
+ * the pag wouldn't fit after setting the group list, we return E2BIG.
+ * If the user tried to create a pag and didn't have one before, we use
+ * unpag() to clear out the fake pag, and return EINVAL, so as not to
+ * allow anyone, even root, from attaching to somebody else's pag.
+ */
+
+static asmlinkage int
+xfs_setgroups (int gidsetsize, gid_t *grouplist)
+{
+    sys_setgroups_function setgroups = (sys_setgroups_function)old_setgroups;
+
+    if (xfs_is_pag ()) {
+	gid_t part1, part2;
+	int ret;
+	/* Always save room for the PAG. */
+	if(gidsetsize + 2 >= NGROUPS)
+	    return -E2BIG;
+
+	part1 = current->groups[0];
+	part2 = current->groups[1];
+	ret = (*setgroups) (gidsetsize, grouplist);
+	if (ret) {
+	    /* setgroups may return an error after partially setting the
+	       groups, i.e. if the user's group buffer was partially
+	       out of its address space. Prevent this trick from being
+	       used to store a fake PAG. */
+	    if(xfs_is_pag ())
+		store_pag (part1, part2);
+
+	    return ret;
+	}
+	return store_pag (part1, part2);
+    } else {
+	int ret;
+	ret = (*setgroups) (gidsetsize, grouplist);
+
+	/* Did a naughty superuser try to create a PAG? */
+	if (xfs_is_pag ()) {
+	    unpag();
+	    /* Let that be a lesson to ya' */
+	    return -EINVAL;
+	}
+	return ret;
+    }
+}
+
 struct file_handle {
     unsigned int dev_index;
     ino_t inode;
@@ -541,21 +660,13 @@ sys_afs (int operation,
     return error;
 }
 
-typedef int (*sys_call_routine)(int operation,
-				char *a_pathP,
-				int a_opcode,
-				struct ViceIoctl *a_paramsP,
-				int a_followSymlinks);
-
-extern sys_call_routine sys_call_table[];
-
-static sys_call_routine old_afs_syscall=NULL;
-
 void
 install_afs_syscall (void)
 {
     old_afs_syscall = sys_call_table[__NR_afs_syscall];
-    sys_call_table[__NR_afs_syscall] = &sys_afs;
+    sys_call_table[__NR_afs_syscall] = (sys_call_function)&sys_afs;
+    old_setgroups = sys_call_table[__NR_setgroups];
+    sys_call_table[__NR_setgroups] = (sys_call_function)&xfs_setgroups;
 }
 
 void
@@ -565,4 +676,8 @@ restore_afs_syscall (void)
 	sys_call_table[__NR_afs_syscall] = old_afs_syscall;
 	old_afs_syscall = NULL;
     }
+    if (old_setgroups) {
+	sys_call_table[__NR_setgroups] = old_setgroups;
+	old_setgroups = NULL;
+    }
 }
