@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -41,7 +41,7 @@
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_node.c,v 1.7 1998/12/22 13:16:52 lha Exp $");
+RCSID("$Id: xfs_node.c,v 1.8 1999/06/16 20:23:39 assar Exp $");
 
 /*
  * Create a new xfs_node and make a VN_HOLD()!
@@ -285,7 +285,7 @@ vattr2xfs_attr(const struct vattr *va, struct xfs_attr *xa)
 {
     bzero ((caddr_t)xa, sizeof(*xa));
     if (va->va_mask & AT_MODE)
-	XA_SET_MODE(xa, va->va_mode);
+	XA_SET_MODE(xa, va->va_mode & S_IAMB);
     if (va->va_mask & AT_NLINK)
 	XA_SET_NLINK(xa, va->va_nlink);
     if (va->va_mask & AT_SIZE)
