@@ -38,15 +38,13 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: output.c,v 1.56 1999/03/05 05:13:56 assar Exp $");
+RCSID("$Id: output.c,v 1.58 1999/07/24 17:54:37 assar Exp $");
 #endif
 
 #include <stdio.h>
 #include <assert.h>
 #include <string.h>
 #include <list.h>
-#include <eefile.h>
-#include <strutil.h>
 #include <err.h>
 #include <roken.h>
 #include "sym.h"
@@ -82,10 +80,11 @@ fileblob headerfile,
 long tmpcnt = 0;
 
 /*
- * Function to convert error codes with (if non-NULL)
+ * Function to convert error codes with.
+ * (the default, the empty string, conveniently means no conversion.)
  */
 
-char *error_function = NULL;
+char *error_function = "";
 
 typedef enum { ENCODE_RX, DECODE_RX, ENCODE_MEM, DECODE_MEM } EncodeType;
 
@@ -1591,13 +1590,14 @@ generate_simple_stub (Symbol *s, FILE *f, FILE *headerf)
      listiter (s->u.proc.arguments, genencodein, f);
      listiter (s->u.proc.arguments, gendecodeout, f);
      fprintf (f,
-	      "return rx_EndCall (call,0);\n"
+	      "return %s(rx_EndCall (call,0));\n"
 	      "fail:\n"
 	      "ret = %s(rx_Error(call));\n"
 	      "rx_EndCall (call, 0);\n"
 	      "return ret;\n"
 	      "}\n",
-	      error_function ? error_function : "");
+	      error_function,
+	      error_function);
 }
 
 static void
@@ -1627,7 +1627,7 @@ generate_split_stub (Symbol *s, FILE *f, FILE *headerf)
 	 findargtype(s->u.proc.arguments, TINOUT))
 	 fprintf (f, "fail:\n"
                  "return %s(rx_Error(call));\n",
-		  error_function ? error_function : "");
+		  error_function);
 
      fprintf (f, "}\n\n");
 
@@ -1648,7 +1648,7 @@ generate_split_stub (Symbol *s, FILE *f, FILE *headerf)
 	 findargtype(s->u.proc.arguments, TINOUT))
 	 fprintf (f, "fail:\n"
                  "return %s(rx_Error(call));\n",
-		  error_function ? error_function : "");
+		  error_function);
 
      fprintf (f, "}\n\n");
 }
@@ -1731,7 +1731,7 @@ generate_standard_c_prologue (FILE *f,
 	      "#define bcopy(a,b,c) memcpy((b),(a),(c))\n"
 	      "#endif /* !HAVE_BCOPY */\n\n");
      fprintf (f, "#ifdef RCSID\n"
-	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.56 1999/03/05 05:13:56 assar Exp $\");\n"
+	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.58 1999/07/24 17:54:37 assar Exp $\");\n"
 	      "#endif\n\n", filename, basename);
 }
 
@@ -2023,7 +2023,7 @@ generate_tcpdump_patches(FILE *td_file, const char *filename)
 {
     Type optype = {TULONG};
 
-    fprintf(td_file, "/* generated by $Id: output.c,v 1.56 1999/03/05 05:13:56 assar Exp $ */\n\n");
+    fprintf(td_file, "/* generated by $Id: output.c,v 1.58 1999/07/24 17:54:37 assar Exp $ */\n\n");
 
     fprintf (td_file, 
 	     "#include <stdio.h>\n"
